package testcasesupport;

public class Main {

	public static void main(String[] args) {
		
		if(args.length != 0) {
		
			if(args[0].equals("-h") || args[0].equals("--help")) {
		
				System.err.println("To use this main, you can either run the program with no " +
				"command line arguments to run all test cases or you can specify one or more classes to test");
				System.err.println("For example:");
				System.err.println("java testcasesupport.Main testcases.CWE690_Unchecked_Return_Value_to_NULL_Pointer_Dereference.custom_function.CWE690_Unchecked_Return_Value_to_NULL_Pointer_Dereference__custom_function_01 testcases.CWE481_Assigning_instead_of_Comparing.bool.CWE481_Assigning_instead_of_Comparing__bool_01");
				System.exit(1);
			}
			
			/* User supplied some class names on the command line, just use those with introspection
			 *
			 * String classNames[] = { "CWE481_Assigning_instead_of_Comparing__boolean_01",
			 *		"CWE476_Null_Pointer_Dereference__getProperty_01" };
 			 * could read class names from command line or use
			 * http://sadun-util.sourceforge.net/api/org/sadun/util/
			 * ClassPackageExplorer.html
			 */

			for (String className : args) {

				try {
					
					/* String classNameWithPackage = "testcases." + className; */
					
					/* IO.writeLine("classNameWithPackage = " + classNameWithPackage); */

					Class<?> myClass = Class.forName(className);

					AbstractTestCase myObject = (AbstractTestCase) myClass
							.newInstance();

					myObject.runTest(className);

				} catch (Exception ex) {

					IO.writeLine("Could not run test for class " + className);
					ex.printStackTrace();

				}
				
				IO.writeLine(""); /* leave a blank line between classes */

			}

		} else {
		
			/* No command line args were used, we want to run every testcase */
			
			/* needed to separate these calls into other methods because
			   we were running into the size limit Java has for a single method */
			runTestCWE1();
			runTestCWE2();
			runTestCWE3();
			runTestCWE4();
			runTestCWE5();
			runTestCWE6();
			runTestCWE7();
			runTestCWE8();
			runTestCWE9();
		}			
	}

private static void runTestCWE1() {
/* BEGIN-AUTOGENERATED-JAVA-TESTS-1 */
			(new testcases.CWE111_Unsafe_JNI.CWE111_Unsafe_JNI__console_01()).runTest("CWE111_Unsafe_JNI__console_01");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_01()).runTest("CWE114_Process_Control__basic_01");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_02()).runTest("CWE114_Process_Control__basic_02");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_03()).runTest("CWE114_Process_Control__basic_03");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_04()).runTest("CWE114_Process_Control__basic_04");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_05()).runTest("CWE114_Process_Control__basic_05");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_06()).runTest("CWE114_Process_Control__basic_06");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_07()).runTest("CWE114_Process_Control__basic_07");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_08()).runTest("CWE114_Process_Control__basic_08");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_09()).runTest("CWE114_Process_Control__basic_09");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_10()).runTest("CWE114_Process_Control__basic_10");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_11()).runTest("CWE114_Process_Control__basic_11");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_12()).runTest("CWE114_Process_Control__basic_12");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_13()).runTest("CWE114_Process_Control__basic_13");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_14()).runTest("CWE114_Process_Control__basic_14");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_15()).runTest("CWE114_Process_Control__basic_15");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_16()).runTest("CWE114_Process_Control__basic_16");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_17()).runTest("CWE114_Process_Control__basic_17");
			(new testcases.CWE114_Process_Control.CWE114_Process_Control__basic_19()).runTest("CWE114_Process_Control__basic_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_01()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_02()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_03()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_04()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_05()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_06()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_07()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_08()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_09()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_10()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_11()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_12()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_13()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_14()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_15()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_16()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_17()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_19()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_31()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_41()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_42()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_45()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_51a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_52a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_53a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_54a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_61a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_66a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_67a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_68a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_format_71a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_format_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_01()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_02()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_03()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_04()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_05()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_06()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_07()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_08()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_09()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_10()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_11()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_12()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_13()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_14()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_15()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_16()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_17()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_19()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_31()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_41()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_42()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_45()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_51a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_52a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_53a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_54a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_61a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_66a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_67a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_68a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__connect_tcp_printf_71a()).runTest("CWE134_Uncontrolled_Format_String__connect_tcp_printf_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_01()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_02()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_03()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_04()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_05()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_06()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_07()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_08()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_09()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_10()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_11()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_12()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_13()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_14()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_15()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_16()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_17()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_19()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_31()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_41()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_42()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_45()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_51a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_52a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_53a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_54a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_61a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_66a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_67a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_68a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_format_71a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_format_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_01()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_02()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_03()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_04()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_05()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_06()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_07()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_08()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_09()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_10()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_11()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_12()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_13()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_14()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_15()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_16()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_17()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_19()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_31()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_41()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_42()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_45()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_51a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_52a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_53a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_54a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_61a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_66a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_67a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_68a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__console_readLine_printf_71a()).runTest("CWE134_Uncontrolled_Format_String__console_readLine_printf_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_01()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_02()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_03()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_04()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_05()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_06()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_07()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_08()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_09()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_10()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_11()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_12()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_13()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_14()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_15()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_16()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_17()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_19()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_31()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_41()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_42()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_45()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_51a()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_52a()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_53a()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_54a()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_61a()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_66a()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_67a()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_68a()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_format_71a()).runTest("CWE134_Uncontrolled_Format_String__Environment_format_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_01()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_02()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_03()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_04()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_05()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_06()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_07()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_08()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_09()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_10()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_11()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_12()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_13()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_14()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_15()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_16()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_17()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_19()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_31()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_41()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_42()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_45()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_51a()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_52a()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_53a()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_54a()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_61a()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_66a()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_67a()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_68a()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__Environment_printf_71a()).runTest("CWE134_Uncontrolled_Format_String__Environment_printf_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_01()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_02()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_03()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_04()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_05()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_06()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_07()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_08()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_09()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_10()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_11()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_12()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_13()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_14()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_15()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_16()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_17()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_19()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_31()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_41()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_42()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_45()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_51a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_52a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_53a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_54a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_61a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_66a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_67a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_68a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_format_71a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_format_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_01()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_02()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_03()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_04()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_05()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_06()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_07()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_08()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_09()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_10()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_11()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_12()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_13()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_14()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_15()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_16()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_17()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_19()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_31()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_41()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_42()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_45()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_51a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_52a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_53a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_54a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_61a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_66a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_67a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_68a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromDB_printf_71a()).runTest("CWE134_Uncontrolled_Format_String__fromDB_printf_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_01()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_02()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_03()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_04()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_05()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_06()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_07()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_08()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_09()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_10()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_11()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_12()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_13()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_14()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_15()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_16()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_17()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_19()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_31()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_41()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_42()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_45()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_51a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_52a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_53a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_54a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_61a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_66a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_67a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_68a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_format_71a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_format_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_01()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_02()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_03()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_04()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_05()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_06()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_07()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_08()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_09()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_10()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_11()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_12()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_13()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_14()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_15()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_16()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_17()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_19()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_31()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_41()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_42()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_45()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_51a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_52a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_53a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_54a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_61a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_66a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_67a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_68a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__fromFile_printf_71a()).runTest("CWE134_Uncontrolled_Format_String__fromFile_printf_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_01()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_02()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_03()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_04()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_05()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_06()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_07()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_08()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_09()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_10()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_11()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_12()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_13()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_14()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_15()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_16()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_17()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_19()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_31()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_41()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_42()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_45()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_51a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_52a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_53a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_54a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_61a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_66a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_67a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_68a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_format_71a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_format_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_01()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_02()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_03()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_04()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_05()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_06()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_07()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_08()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_09()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_10()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_11()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_12()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_13()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_14()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_15()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_16()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_17()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_19()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_31()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_41()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_42()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_45()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_51a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_52a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_53a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_54a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_61a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_66a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_67a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_68a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__listen_tcp_printf_71a()).runTest("CWE134_Uncontrolled_Format_String__listen_tcp_printf_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_01()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_02()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_03()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_04()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_05()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_06()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_07()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_08()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_09()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_10()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_11()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_12()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_13()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_14()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_15()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_16()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_17()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_19()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_31()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_41()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_42()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_45()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_51a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_52a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_53a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_54a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_61a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_66a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_67a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_68a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_format_71a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_format_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_01()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_02()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_03()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_04()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_05()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_06()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_07()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_08()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_09()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_10()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_11()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_12()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_13()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_14()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_15()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_16()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_17()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_19()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_31()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_41()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_42()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_45()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_51a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_52a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_53a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_54a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_61a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_66a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_67a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_68a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__PropertiesFile_printf_71a()).runTest("CWE134_Uncontrolled_Format_String__PropertiesFile_printf_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_01()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_02()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_03()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_04()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_05()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_06()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_07()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_08()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_09()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_10()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_11()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_12()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_13()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_14()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_15()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_16()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_17()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_19()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_31()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_41()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_42()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_45()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_51a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_52a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_53a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_54a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_61a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_66a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_67a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_68a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_format_71a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_format_71a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_01()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_01");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_02()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_02");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_03()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_03");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_04()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_04");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_05()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_05");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_06()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_06");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_07()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_07");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_08()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_08");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_09()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_09");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_10()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_10");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_11()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_11");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_12()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_12");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_13()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_13");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_14()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_14");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_15()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_15");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_16()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_16");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_17()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_17");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_19()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_19");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_31()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_31");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_41()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_41");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_42()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_42");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_45()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_45");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_51a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_51a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_52a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_52a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_53a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_53a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_54a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_54a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_61a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_61a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_66a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_66a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_67a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_67a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_68a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_68a");
			(new testcases.CWE134_Uncontrolled_Format_String.CWE134_Uncontrolled_Format_String__URLConnection_printf_71a()).runTest("CWE134_Uncontrolled_Format_String__URLConnection_printf_71a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_01()).runTest("CWE190_Integer_Overflow__connect_tcp_add_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_02()).runTest("CWE190_Integer_Overflow__connect_tcp_add_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_03()).runTest("CWE190_Integer_Overflow__connect_tcp_add_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_04()).runTest("CWE190_Integer_Overflow__connect_tcp_add_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_05()).runTest("CWE190_Integer_Overflow__connect_tcp_add_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_06()).runTest("CWE190_Integer_Overflow__connect_tcp_add_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_07()).runTest("CWE190_Integer_Overflow__connect_tcp_add_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_08()).runTest("CWE190_Integer_Overflow__connect_tcp_add_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_09()).runTest("CWE190_Integer_Overflow__connect_tcp_add_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_10()).runTest("CWE190_Integer_Overflow__connect_tcp_add_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_11()).runTest("CWE190_Integer_Overflow__connect_tcp_add_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_12()).runTest("CWE190_Integer_Overflow__connect_tcp_add_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_13()).runTest("CWE190_Integer_Overflow__connect_tcp_add_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_14()).runTest("CWE190_Integer_Overflow__connect_tcp_add_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_15()).runTest("CWE190_Integer_Overflow__connect_tcp_add_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_16()).runTest("CWE190_Integer_Overflow__connect_tcp_add_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_17()).runTest("CWE190_Integer_Overflow__connect_tcp_add_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_19()).runTest("CWE190_Integer_Overflow__connect_tcp_add_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_31()).runTest("CWE190_Integer_Overflow__connect_tcp_add_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_41()).runTest("CWE190_Integer_Overflow__connect_tcp_add_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_42()).runTest("CWE190_Integer_Overflow__connect_tcp_add_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_45()).runTest("CWE190_Integer_Overflow__connect_tcp_add_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_51a()).runTest("CWE190_Integer_Overflow__connect_tcp_add_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_52a()).runTest("CWE190_Integer_Overflow__connect_tcp_add_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_53a()).runTest("CWE190_Integer_Overflow__connect_tcp_add_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_54a()).runTest("CWE190_Integer_Overflow__connect_tcp_add_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_61a()).runTest("CWE190_Integer_Overflow__connect_tcp_add_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_66a()).runTest("CWE190_Integer_Overflow__connect_tcp_add_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_67a()).runTest("CWE190_Integer_Overflow__connect_tcp_add_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_add_68a()).runTest("CWE190_Integer_Overflow__connect_tcp_add_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_01()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_02()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_03()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_04()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_05()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_06()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_07()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_08()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_09()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_10()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_11()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_12()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_13()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_14()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_15()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_16()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_17()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_19()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_31()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_41()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_42()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_45()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_51a()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_52a()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_53a()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_54a()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_61a()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_66a()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_67a()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_multiply_68a()).runTest("CWE190_Integer_Overflow__connect_tcp_multiply_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_01()).runTest("CWE190_Integer_Overflow__connect_tcp_square_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_02()).runTest("CWE190_Integer_Overflow__connect_tcp_square_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_03()).runTest("CWE190_Integer_Overflow__connect_tcp_square_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_04()).runTest("CWE190_Integer_Overflow__connect_tcp_square_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_05()).runTest("CWE190_Integer_Overflow__connect_tcp_square_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_06()).runTest("CWE190_Integer_Overflow__connect_tcp_square_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_07()).runTest("CWE190_Integer_Overflow__connect_tcp_square_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_08()).runTest("CWE190_Integer_Overflow__connect_tcp_square_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_09()).runTest("CWE190_Integer_Overflow__connect_tcp_square_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_10()).runTest("CWE190_Integer_Overflow__connect_tcp_square_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_11()).runTest("CWE190_Integer_Overflow__connect_tcp_square_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_12()).runTest("CWE190_Integer_Overflow__connect_tcp_square_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_13()).runTest("CWE190_Integer_Overflow__connect_tcp_square_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_14()).runTest("CWE190_Integer_Overflow__connect_tcp_square_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_15()).runTest("CWE190_Integer_Overflow__connect_tcp_square_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_16()).runTest("CWE190_Integer_Overflow__connect_tcp_square_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_17()).runTest("CWE190_Integer_Overflow__connect_tcp_square_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_19()).runTest("CWE190_Integer_Overflow__connect_tcp_square_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_31()).runTest("CWE190_Integer_Overflow__connect_tcp_square_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_41()).runTest("CWE190_Integer_Overflow__connect_tcp_square_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_42()).runTest("CWE190_Integer_Overflow__connect_tcp_square_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_45()).runTest("CWE190_Integer_Overflow__connect_tcp_square_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_51a()).runTest("CWE190_Integer_Overflow__connect_tcp_square_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_52a()).runTest("CWE190_Integer_Overflow__connect_tcp_square_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_53a()).runTest("CWE190_Integer_Overflow__connect_tcp_square_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_54a()).runTest("CWE190_Integer_Overflow__connect_tcp_square_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_61a()).runTest("CWE190_Integer_Overflow__connect_tcp_square_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_66a()).runTest("CWE190_Integer_Overflow__connect_tcp_square_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_67a()).runTest("CWE190_Integer_Overflow__connect_tcp_square_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__connect_tcp_square_68a()).runTest("CWE190_Integer_Overflow__connect_tcp_square_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_01()).runTest("CWE190_Integer_Overflow__console_readLine_add_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_02()).runTest("CWE190_Integer_Overflow__console_readLine_add_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_03()).runTest("CWE190_Integer_Overflow__console_readLine_add_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_04()).runTest("CWE190_Integer_Overflow__console_readLine_add_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_05()).runTest("CWE190_Integer_Overflow__console_readLine_add_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_06()).runTest("CWE190_Integer_Overflow__console_readLine_add_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_07()).runTest("CWE190_Integer_Overflow__console_readLine_add_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_08()).runTest("CWE190_Integer_Overflow__console_readLine_add_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_09()).runTest("CWE190_Integer_Overflow__console_readLine_add_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_10()).runTest("CWE190_Integer_Overflow__console_readLine_add_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_11()).runTest("CWE190_Integer_Overflow__console_readLine_add_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_12()).runTest("CWE190_Integer_Overflow__console_readLine_add_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_13()).runTest("CWE190_Integer_Overflow__console_readLine_add_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_14()).runTest("CWE190_Integer_Overflow__console_readLine_add_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_15()).runTest("CWE190_Integer_Overflow__console_readLine_add_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_16()).runTest("CWE190_Integer_Overflow__console_readLine_add_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_17()).runTest("CWE190_Integer_Overflow__console_readLine_add_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_19()).runTest("CWE190_Integer_Overflow__console_readLine_add_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_31()).runTest("CWE190_Integer_Overflow__console_readLine_add_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_41()).runTest("CWE190_Integer_Overflow__console_readLine_add_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_42()).runTest("CWE190_Integer_Overflow__console_readLine_add_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_45()).runTest("CWE190_Integer_Overflow__console_readLine_add_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_51a()).runTest("CWE190_Integer_Overflow__console_readLine_add_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_52a()).runTest("CWE190_Integer_Overflow__console_readLine_add_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_53a()).runTest("CWE190_Integer_Overflow__console_readLine_add_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_54a()).runTest("CWE190_Integer_Overflow__console_readLine_add_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_61a()).runTest("CWE190_Integer_Overflow__console_readLine_add_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_66a()).runTest("CWE190_Integer_Overflow__console_readLine_add_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_67a()).runTest("CWE190_Integer_Overflow__console_readLine_add_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_add_68a()).runTest("CWE190_Integer_Overflow__console_readLine_add_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_01()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_02()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_03()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_04()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_05()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_06()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_07()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_08()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_09()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_10()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_11()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_12()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_13()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_14()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_15()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_16()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_17()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_19()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_31()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_41()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_42()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_45()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_51a()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_52a()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_53a()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_54a()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_61a()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_66a()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_67a()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_multiply_68a()).runTest("CWE190_Integer_Overflow__console_readLine_multiply_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_01()).runTest("CWE190_Integer_Overflow__console_readLine_square_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_02()).runTest("CWE190_Integer_Overflow__console_readLine_square_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_03()).runTest("CWE190_Integer_Overflow__console_readLine_square_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_04()).runTest("CWE190_Integer_Overflow__console_readLine_square_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_05()).runTest("CWE190_Integer_Overflow__console_readLine_square_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_06()).runTest("CWE190_Integer_Overflow__console_readLine_square_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_07()).runTest("CWE190_Integer_Overflow__console_readLine_square_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_08()).runTest("CWE190_Integer_Overflow__console_readLine_square_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_09()).runTest("CWE190_Integer_Overflow__console_readLine_square_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_10()).runTest("CWE190_Integer_Overflow__console_readLine_square_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_11()).runTest("CWE190_Integer_Overflow__console_readLine_square_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_12()).runTest("CWE190_Integer_Overflow__console_readLine_square_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_13()).runTest("CWE190_Integer_Overflow__console_readLine_square_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_14()).runTest("CWE190_Integer_Overflow__console_readLine_square_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_15()).runTest("CWE190_Integer_Overflow__console_readLine_square_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_16()).runTest("CWE190_Integer_Overflow__console_readLine_square_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_17()).runTest("CWE190_Integer_Overflow__console_readLine_square_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_19()).runTest("CWE190_Integer_Overflow__console_readLine_square_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_31()).runTest("CWE190_Integer_Overflow__console_readLine_square_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_41()).runTest("CWE190_Integer_Overflow__console_readLine_square_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_42()).runTest("CWE190_Integer_Overflow__console_readLine_square_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_45()).runTest("CWE190_Integer_Overflow__console_readLine_square_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_51a()).runTest("CWE190_Integer_Overflow__console_readLine_square_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_52a()).runTest("CWE190_Integer_Overflow__console_readLine_square_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_53a()).runTest("CWE190_Integer_Overflow__console_readLine_square_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_54a()).runTest("CWE190_Integer_Overflow__console_readLine_square_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_61a()).runTest("CWE190_Integer_Overflow__console_readLine_square_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_66a()).runTest("CWE190_Integer_Overflow__console_readLine_square_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_67a()).runTest("CWE190_Integer_Overflow__console_readLine_square_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__console_readLine_square_68a()).runTest("CWE190_Integer_Overflow__console_readLine_square_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_01()).runTest("CWE190_Integer_Overflow__Environment_add_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_02()).runTest("CWE190_Integer_Overflow__Environment_add_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_03()).runTest("CWE190_Integer_Overflow__Environment_add_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_04()).runTest("CWE190_Integer_Overflow__Environment_add_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_05()).runTest("CWE190_Integer_Overflow__Environment_add_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_06()).runTest("CWE190_Integer_Overflow__Environment_add_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_07()).runTest("CWE190_Integer_Overflow__Environment_add_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_08()).runTest("CWE190_Integer_Overflow__Environment_add_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_09()).runTest("CWE190_Integer_Overflow__Environment_add_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_10()).runTest("CWE190_Integer_Overflow__Environment_add_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_11()).runTest("CWE190_Integer_Overflow__Environment_add_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_12()).runTest("CWE190_Integer_Overflow__Environment_add_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_13()).runTest("CWE190_Integer_Overflow__Environment_add_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_14()).runTest("CWE190_Integer_Overflow__Environment_add_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_15()).runTest("CWE190_Integer_Overflow__Environment_add_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_16()).runTest("CWE190_Integer_Overflow__Environment_add_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_17()).runTest("CWE190_Integer_Overflow__Environment_add_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_19()).runTest("CWE190_Integer_Overflow__Environment_add_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_31()).runTest("CWE190_Integer_Overflow__Environment_add_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_41()).runTest("CWE190_Integer_Overflow__Environment_add_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_42()).runTest("CWE190_Integer_Overflow__Environment_add_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_45()).runTest("CWE190_Integer_Overflow__Environment_add_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_51a()).runTest("CWE190_Integer_Overflow__Environment_add_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_52a()).runTest("CWE190_Integer_Overflow__Environment_add_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_53a()).runTest("CWE190_Integer_Overflow__Environment_add_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_54a()).runTest("CWE190_Integer_Overflow__Environment_add_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_61a()).runTest("CWE190_Integer_Overflow__Environment_add_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_66a()).runTest("CWE190_Integer_Overflow__Environment_add_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_67a()).runTest("CWE190_Integer_Overflow__Environment_add_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_add_68a()).runTest("CWE190_Integer_Overflow__Environment_add_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_01()).runTest("CWE190_Integer_Overflow__Environment_multiply_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_02()).runTest("CWE190_Integer_Overflow__Environment_multiply_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_03()).runTest("CWE190_Integer_Overflow__Environment_multiply_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_04()).runTest("CWE190_Integer_Overflow__Environment_multiply_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_05()).runTest("CWE190_Integer_Overflow__Environment_multiply_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_06()).runTest("CWE190_Integer_Overflow__Environment_multiply_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_07()).runTest("CWE190_Integer_Overflow__Environment_multiply_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_08()).runTest("CWE190_Integer_Overflow__Environment_multiply_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_09()).runTest("CWE190_Integer_Overflow__Environment_multiply_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_10()).runTest("CWE190_Integer_Overflow__Environment_multiply_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_11()).runTest("CWE190_Integer_Overflow__Environment_multiply_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_12()).runTest("CWE190_Integer_Overflow__Environment_multiply_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_13()).runTest("CWE190_Integer_Overflow__Environment_multiply_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_14()).runTest("CWE190_Integer_Overflow__Environment_multiply_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_15()).runTest("CWE190_Integer_Overflow__Environment_multiply_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_16()).runTest("CWE190_Integer_Overflow__Environment_multiply_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_17()).runTest("CWE190_Integer_Overflow__Environment_multiply_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_19()).runTest("CWE190_Integer_Overflow__Environment_multiply_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_31()).runTest("CWE190_Integer_Overflow__Environment_multiply_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_41()).runTest("CWE190_Integer_Overflow__Environment_multiply_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_42()).runTest("CWE190_Integer_Overflow__Environment_multiply_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_45()).runTest("CWE190_Integer_Overflow__Environment_multiply_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_51a()).runTest("CWE190_Integer_Overflow__Environment_multiply_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_52a()).runTest("CWE190_Integer_Overflow__Environment_multiply_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_53a()).runTest("CWE190_Integer_Overflow__Environment_multiply_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_54a()).runTest("CWE190_Integer_Overflow__Environment_multiply_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_61a()).runTest("CWE190_Integer_Overflow__Environment_multiply_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_66a()).runTest("CWE190_Integer_Overflow__Environment_multiply_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_67a()).runTest("CWE190_Integer_Overflow__Environment_multiply_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_multiply_68a()).runTest("CWE190_Integer_Overflow__Environment_multiply_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_01()).runTest("CWE190_Integer_Overflow__Environment_square_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_02()).runTest("CWE190_Integer_Overflow__Environment_square_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_03()).runTest("CWE190_Integer_Overflow__Environment_square_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_04()).runTest("CWE190_Integer_Overflow__Environment_square_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_05()).runTest("CWE190_Integer_Overflow__Environment_square_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_06()).runTest("CWE190_Integer_Overflow__Environment_square_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_07()).runTest("CWE190_Integer_Overflow__Environment_square_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_08()).runTest("CWE190_Integer_Overflow__Environment_square_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_09()).runTest("CWE190_Integer_Overflow__Environment_square_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_10()).runTest("CWE190_Integer_Overflow__Environment_square_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_11()).runTest("CWE190_Integer_Overflow__Environment_square_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_12()).runTest("CWE190_Integer_Overflow__Environment_square_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_13()).runTest("CWE190_Integer_Overflow__Environment_square_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_14()).runTest("CWE190_Integer_Overflow__Environment_square_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_15()).runTest("CWE190_Integer_Overflow__Environment_square_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_16()).runTest("CWE190_Integer_Overflow__Environment_square_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_17()).runTest("CWE190_Integer_Overflow__Environment_square_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_19()).runTest("CWE190_Integer_Overflow__Environment_square_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_31()).runTest("CWE190_Integer_Overflow__Environment_square_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_41()).runTest("CWE190_Integer_Overflow__Environment_square_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_42()).runTest("CWE190_Integer_Overflow__Environment_square_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_45()).runTest("CWE190_Integer_Overflow__Environment_square_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_51a()).runTest("CWE190_Integer_Overflow__Environment_square_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_52a()).runTest("CWE190_Integer_Overflow__Environment_square_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_53a()).runTest("CWE190_Integer_Overflow__Environment_square_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_54a()).runTest("CWE190_Integer_Overflow__Environment_square_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_61a()).runTest("CWE190_Integer_Overflow__Environment_square_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_66a()).runTest("CWE190_Integer_Overflow__Environment_square_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_67a()).runTest("CWE190_Integer_Overflow__Environment_square_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__Environment_square_68a()).runTest("CWE190_Integer_Overflow__Environment_square_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_01()).runTest("CWE190_Integer_Overflow__fromDB_add_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_02()).runTest("CWE190_Integer_Overflow__fromDB_add_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_03()).runTest("CWE190_Integer_Overflow__fromDB_add_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_04()).runTest("CWE190_Integer_Overflow__fromDB_add_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_05()).runTest("CWE190_Integer_Overflow__fromDB_add_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_06()).runTest("CWE190_Integer_Overflow__fromDB_add_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_07()).runTest("CWE190_Integer_Overflow__fromDB_add_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_08()).runTest("CWE190_Integer_Overflow__fromDB_add_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_09()).runTest("CWE190_Integer_Overflow__fromDB_add_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_10()).runTest("CWE190_Integer_Overflow__fromDB_add_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_11()).runTest("CWE190_Integer_Overflow__fromDB_add_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_12()).runTest("CWE190_Integer_Overflow__fromDB_add_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_13()).runTest("CWE190_Integer_Overflow__fromDB_add_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_14()).runTest("CWE190_Integer_Overflow__fromDB_add_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_15()).runTest("CWE190_Integer_Overflow__fromDB_add_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_16()).runTest("CWE190_Integer_Overflow__fromDB_add_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_17()).runTest("CWE190_Integer_Overflow__fromDB_add_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_19()).runTest("CWE190_Integer_Overflow__fromDB_add_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_31()).runTest("CWE190_Integer_Overflow__fromDB_add_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_41()).runTest("CWE190_Integer_Overflow__fromDB_add_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_42()).runTest("CWE190_Integer_Overflow__fromDB_add_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_45()).runTest("CWE190_Integer_Overflow__fromDB_add_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_51a()).runTest("CWE190_Integer_Overflow__fromDB_add_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_52a()).runTest("CWE190_Integer_Overflow__fromDB_add_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_53a()).runTest("CWE190_Integer_Overflow__fromDB_add_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_54a()).runTest("CWE190_Integer_Overflow__fromDB_add_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_61a()).runTest("CWE190_Integer_Overflow__fromDB_add_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_66a()).runTest("CWE190_Integer_Overflow__fromDB_add_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_67a()).runTest("CWE190_Integer_Overflow__fromDB_add_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_add_68a()).runTest("CWE190_Integer_Overflow__fromDB_add_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_01()).runTest("CWE190_Integer_Overflow__fromDB_multiply_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_02()).runTest("CWE190_Integer_Overflow__fromDB_multiply_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_03()).runTest("CWE190_Integer_Overflow__fromDB_multiply_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_04()).runTest("CWE190_Integer_Overflow__fromDB_multiply_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_05()).runTest("CWE190_Integer_Overflow__fromDB_multiply_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_06()).runTest("CWE190_Integer_Overflow__fromDB_multiply_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_07()).runTest("CWE190_Integer_Overflow__fromDB_multiply_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_08()).runTest("CWE190_Integer_Overflow__fromDB_multiply_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_09()).runTest("CWE190_Integer_Overflow__fromDB_multiply_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_10()).runTest("CWE190_Integer_Overflow__fromDB_multiply_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_11()).runTest("CWE190_Integer_Overflow__fromDB_multiply_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_12()).runTest("CWE190_Integer_Overflow__fromDB_multiply_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_13()).runTest("CWE190_Integer_Overflow__fromDB_multiply_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_14()).runTest("CWE190_Integer_Overflow__fromDB_multiply_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_15()).runTest("CWE190_Integer_Overflow__fromDB_multiply_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_16()).runTest("CWE190_Integer_Overflow__fromDB_multiply_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_17()).runTest("CWE190_Integer_Overflow__fromDB_multiply_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_19()).runTest("CWE190_Integer_Overflow__fromDB_multiply_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_31()).runTest("CWE190_Integer_Overflow__fromDB_multiply_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_41()).runTest("CWE190_Integer_Overflow__fromDB_multiply_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_42()).runTest("CWE190_Integer_Overflow__fromDB_multiply_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_45()).runTest("CWE190_Integer_Overflow__fromDB_multiply_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_51a()).runTest("CWE190_Integer_Overflow__fromDB_multiply_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_52a()).runTest("CWE190_Integer_Overflow__fromDB_multiply_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_53a()).runTest("CWE190_Integer_Overflow__fromDB_multiply_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_54a()).runTest("CWE190_Integer_Overflow__fromDB_multiply_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_61a()).runTest("CWE190_Integer_Overflow__fromDB_multiply_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_66a()).runTest("CWE190_Integer_Overflow__fromDB_multiply_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_67a()).runTest("CWE190_Integer_Overflow__fromDB_multiply_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_multiply_68a()).runTest("CWE190_Integer_Overflow__fromDB_multiply_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_01()).runTest("CWE190_Integer_Overflow__fromDB_square_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_02()).runTest("CWE190_Integer_Overflow__fromDB_square_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_03()).runTest("CWE190_Integer_Overflow__fromDB_square_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_04()).runTest("CWE190_Integer_Overflow__fromDB_square_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_05()).runTest("CWE190_Integer_Overflow__fromDB_square_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_06()).runTest("CWE190_Integer_Overflow__fromDB_square_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_07()).runTest("CWE190_Integer_Overflow__fromDB_square_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_08()).runTest("CWE190_Integer_Overflow__fromDB_square_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_09()).runTest("CWE190_Integer_Overflow__fromDB_square_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_10()).runTest("CWE190_Integer_Overflow__fromDB_square_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_11()).runTest("CWE190_Integer_Overflow__fromDB_square_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_12()).runTest("CWE190_Integer_Overflow__fromDB_square_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_13()).runTest("CWE190_Integer_Overflow__fromDB_square_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_14()).runTest("CWE190_Integer_Overflow__fromDB_square_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_15()).runTest("CWE190_Integer_Overflow__fromDB_square_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_16()).runTest("CWE190_Integer_Overflow__fromDB_square_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_17()).runTest("CWE190_Integer_Overflow__fromDB_square_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_19()).runTest("CWE190_Integer_Overflow__fromDB_square_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_31()).runTest("CWE190_Integer_Overflow__fromDB_square_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_41()).runTest("CWE190_Integer_Overflow__fromDB_square_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_42()).runTest("CWE190_Integer_Overflow__fromDB_square_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_45()).runTest("CWE190_Integer_Overflow__fromDB_square_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_51a()).runTest("CWE190_Integer_Overflow__fromDB_square_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_52a()).runTest("CWE190_Integer_Overflow__fromDB_square_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_53a()).runTest("CWE190_Integer_Overflow__fromDB_square_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_54a()).runTest("CWE190_Integer_Overflow__fromDB_square_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_61a()).runTest("CWE190_Integer_Overflow__fromDB_square_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_66a()).runTest("CWE190_Integer_Overflow__fromDB_square_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_67a()).runTest("CWE190_Integer_Overflow__fromDB_square_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromDB_square_68a()).runTest("CWE190_Integer_Overflow__fromDB_square_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_01()).runTest("CWE190_Integer_Overflow__fromFile_add_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_02()).runTest("CWE190_Integer_Overflow__fromFile_add_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_03()).runTest("CWE190_Integer_Overflow__fromFile_add_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_04()).runTest("CWE190_Integer_Overflow__fromFile_add_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_05()).runTest("CWE190_Integer_Overflow__fromFile_add_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_06()).runTest("CWE190_Integer_Overflow__fromFile_add_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_07()).runTest("CWE190_Integer_Overflow__fromFile_add_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_08()).runTest("CWE190_Integer_Overflow__fromFile_add_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_09()).runTest("CWE190_Integer_Overflow__fromFile_add_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_10()).runTest("CWE190_Integer_Overflow__fromFile_add_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_11()).runTest("CWE190_Integer_Overflow__fromFile_add_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_12()).runTest("CWE190_Integer_Overflow__fromFile_add_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_13()).runTest("CWE190_Integer_Overflow__fromFile_add_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_14()).runTest("CWE190_Integer_Overflow__fromFile_add_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_15()).runTest("CWE190_Integer_Overflow__fromFile_add_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_16()).runTest("CWE190_Integer_Overflow__fromFile_add_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_17()).runTest("CWE190_Integer_Overflow__fromFile_add_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_19()).runTest("CWE190_Integer_Overflow__fromFile_add_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_31()).runTest("CWE190_Integer_Overflow__fromFile_add_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_41()).runTest("CWE190_Integer_Overflow__fromFile_add_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_42()).runTest("CWE190_Integer_Overflow__fromFile_add_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_45()).runTest("CWE190_Integer_Overflow__fromFile_add_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_51a()).runTest("CWE190_Integer_Overflow__fromFile_add_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_52a()).runTest("CWE190_Integer_Overflow__fromFile_add_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_53a()).runTest("CWE190_Integer_Overflow__fromFile_add_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_54a()).runTest("CWE190_Integer_Overflow__fromFile_add_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_61a()).runTest("CWE190_Integer_Overflow__fromFile_add_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_66a()).runTest("CWE190_Integer_Overflow__fromFile_add_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_67a()).runTest("CWE190_Integer_Overflow__fromFile_add_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_add_68a()).runTest("CWE190_Integer_Overflow__fromFile_add_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_01()).runTest("CWE190_Integer_Overflow__fromFile_multiply_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_02()).runTest("CWE190_Integer_Overflow__fromFile_multiply_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_03()).runTest("CWE190_Integer_Overflow__fromFile_multiply_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_04()).runTest("CWE190_Integer_Overflow__fromFile_multiply_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_05()).runTest("CWE190_Integer_Overflow__fromFile_multiply_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_06()).runTest("CWE190_Integer_Overflow__fromFile_multiply_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_07()).runTest("CWE190_Integer_Overflow__fromFile_multiply_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_08()).runTest("CWE190_Integer_Overflow__fromFile_multiply_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_09()).runTest("CWE190_Integer_Overflow__fromFile_multiply_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_10()).runTest("CWE190_Integer_Overflow__fromFile_multiply_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_11()).runTest("CWE190_Integer_Overflow__fromFile_multiply_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_12()).runTest("CWE190_Integer_Overflow__fromFile_multiply_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_13()).runTest("CWE190_Integer_Overflow__fromFile_multiply_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_14()).runTest("CWE190_Integer_Overflow__fromFile_multiply_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_15()).runTest("CWE190_Integer_Overflow__fromFile_multiply_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_16()).runTest("CWE190_Integer_Overflow__fromFile_multiply_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_17()).runTest("CWE190_Integer_Overflow__fromFile_multiply_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_19()).runTest("CWE190_Integer_Overflow__fromFile_multiply_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_31()).runTest("CWE190_Integer_Overflow__fromFile_multiply_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_41()).runTest("CWE190_Integer_Overflow__fromFile_multiply_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_42()).runTest("CWE190_Integer_Overflow__fromFile_multiply_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_45()).runTest("CWE190_Integer_Overflow__fromFile_multiply_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_51a()).runTest("CWE190_Integer_Overflow__fromFile_multiply_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_52a()).runTest("CWE190_Integer_Overflow__fromFile_multiply_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_53a()).runTest("CWE190_Integer_Overflow__fromFile_multiply_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_54a()).runTest("CWE190_Integer_Overflow__fromFile_multiply_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_61a()).runTest("CWE190_Integer_Overflow__fromFile_multiply_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_66a()).runTest("CWE190_Integer_Overflow__fromFile_multiply_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_67a()).runTest("CWE190_Integer_Overflow__fromFile_multiply_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_multiply_68a()).runTest("CWE190_Integer_Overflow__fromFile_multiply_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_01()).runTest("CWE190_Integer_Overflow__fromFile_square_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_02()).runTest("CWE190_Integer_Overflow__fromFile_square_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_03()).runTest("CWE190_Integer_Overflow__fromFile_square_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_04()).runTest("CWE190_Integer_Overflow__fromFile_square_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_05()).runTest("CWE190_Integer_Overflow__fromFile_square_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_06()).runTest("CWE190_Integer_Overflow__fromFile_square_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_07()).runTest("CWE190_Integer_Overflow__fromFile_square_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_08()).runTest("CWE190_Integer_Overflow__fromFile_square_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_09()).runTest("CWE190_Integer_Overflow__fromFile_square_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_10()).runTest("CWE190_Integer_Overflow__fromFile_square_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_11()).runTest("CWE190_Integer_Overflow__fromFile_square_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_12()).runTest("CWE190_Integer_Overflow__fromFile_square_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_13()).runTest("CWE190_Integer_Overflow__fromFile_square_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_14()).runTest("CWE190_Integer_Overflow__fromFile_square_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_15()).runTest("CWE190_Integer_Overflow__fromFile_square_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_16()).runTest("CWE190_Integer_Overflow__fromFile_square_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_17()).runTest("CWE190_Integer_Overflow__fromFile_square_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_19()).runTest("CWE190_Integer_Overflow__fromFile_square_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_31()).runTest("CWE190_Integer_Overflow__fromFile_square_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_41()).runTest("CWE190_Integer_Overflow__fromFile_square_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_42()).runTest("CWE190_Integer_Overflow__fromFile_square_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_45()).runTest("CWE190_Integer_Overflow__fromFile_square_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_51a()).runTest("CWE190_Integer_Overflow__fromFile_square_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_52a()).runTest("CWE190_Integer_Overflow__fromFile_square_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_53a()).runTest("CWE190_Integer_Overflow__fromFile_square_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_54a()).runTest("CWE190_Integer_Overflow__fromFile_square_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_61a()).runTest("CWE190_Integer_Overflow__fromFile_square_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_66a()).runTest("CWE190_Integer_Overflow__fromFile_square_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_67a()).runTest("CWE190_Integer_Overflow__fromFile_square_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__fromFile_square_68a()).runTest("CWE190_Integer_Overflow__fromFile_square_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_01()).runTest("CWE190_Integer_Overflow__listen_tcp_add_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_02()).runTest("CWE190_Integer_Overflow__listen_tcp_add_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_03()).runTest("CWE190_Integer_Overflow__listen_tcp_add_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_04()).runTest("CWE190_Integer_Overflow__listen_tcp_add_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_05()).runTest("CWE190_Integer_Overflow__listen_tcp_add_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_06()).runTest("CWE190_Integer_Overflow__listen_tcp_add_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_07()).runTest("CWE190_Integer_Overflow__listen_tcp_add_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_08()).runTest("CWE190_Integer_Overflow__listen_tcp_add_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_09()).runTest("CWE190_Integer_Overflow__listen_tcp_add_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_10()).runTest("CWE190_Integer_Overflow__listen_tcp_add_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_11()).runTest("CWE190_Integer_Overflow__listen_tcp_add_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_12()).runTest("CWE190_Integer_Overflow__listen_tcp_add_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_13()).runTest("CWE190_Integer_Overflow__listen_tcp_add_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_14()).runTest("CWE190_Integer_Overflow__listen_tcp_add_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_15()).runTest("CWE190_Integer_Overflow__listen_tcp_add_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_16()).runTest("CWE190_Integer_Overflow__listen_tcp_add_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_17()).runTest("CWE190_Integer_Overflow__listen_tcp_add_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_19()).runTest("CWE190_Integer_Overflow__listen_tcp_add_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_31()).runTest("CWE190_Integer_Overflow__listen_tcp_add_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_41()).runTest("CWE190_Integer_Overflow__listen_tcp_add_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_42()).runTest("CWE190_Integer_Overflow__listen_tcp_add_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_45()).runTest("CWE190_Integer_Overflow__listen_tcp_add_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_51a()).runTest("CWE190_Integer_Overflow__listen_tcp_add_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_52a()).runTest("CWE190_Integer_Overflow__listen_tcp_add_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_53a()).runTest("CWE190_Integer_Overflow__listen_tcp_add_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_54a()).runTest("CWE190_Integer_Overflow__listen_tcp_add_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_61a()).runTest("CWE190_Integer_Overflow__listen_tcp_add_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_66a()).runTest("CWE190_Integer_Overflow__listen_tcp_add_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_67a()).runTest("CWE190_Integer_Overflow__listen_tcp_add_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_add_68a()).runTest("CWE190_Integer_Overflow__listen_tcp_add_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_01()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_02()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_03()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_04()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_05()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_06()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_07()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_08()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_09()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_10()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_11()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_12()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_13()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_14()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_15()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_16()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_17()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_19()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_31()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_41()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_42()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_45()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_51a()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_52a()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_53a()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_54a()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_61a()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_66a()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_67a()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_multiply_68a()).runTest("CWE190_Integer_Overflow__listen_tcp_multiply_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_01()).runTest("CWE190_Integer_Overflow__listen_tcp_square_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_02()).runTest("CWE190_Integer_Overflow__listen_tcp_square_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_03()).runTest("CWE190_Integer_Overflow__listen_tcp_square_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_04()).runTest("CWE190_Integer_Overflow__listen_tcp_square_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_05()).runTest("CWE190_Integer_Overflow__listen_tcp_square_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_06()).runTest("CWE190_Integer_Overflow__listen_tcp_square_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_07()).runTest("CWE190_Integer_Overflow__listen_tcp_square_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_08()).runTest("CWE190_Integer_Overflow__listen_tcp_square_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_09()).runTest("CWE190_Integer_Overflow__listen_tcp_square_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_10()).runTest("CWE190_Integer_Overflow__listen_tcp_square_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_11()).runTest("CWE190_Integer_Overflow__listen_tcp_square_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_12()).runTest("CWE190_Integer_Overflow__listen_tcp_square_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_13()).runTest("CWE190_Integer_Overflow__listen_tcp_square_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_14()).runTest("CWE190_Integer_Overflow__listen_tcp_square_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_15()).runTest("CWE190_Integer_Overflow__listen_tcp_square_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_16()).runTest("CWE190_Integer_Overflow__listen_tcp_square_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_17()).runTest("CWE190_Integer_Overflow__listen_tcp_square_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_19()).runTest("CWE190_Integer_Overflow__listen_tcp_square_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_31()).runTest("CWE190_Integer_Overflow__listen_tcp_square_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_41()).runTest("CWE190_Integer_Overflow__listen_tcp_square_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_42()).runTest("CWE190_Integer_Overflow__listen_tcp_square_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_45()).runTest("CWE190_Integer_Overflow__listen_tcp_square_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_51a()).runTest("CWE190_Integer_Overflow__listen_tcp_square_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_52a()).runTest("CWE190_Integer_Overflow__listen_tcp_square_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_53a()).runTest("CWE190_Integer_Overflow__listen_tcp_square_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_54a()).runTest("CWE190_Integer_Overflow__listen_tcp_square_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_61a()).runTest("CWE190_Integer_Overflow__listen_tcp_square_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_66a()).runTest("CWE190_Integer_Overflow__listen_tcp_square_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_67a()).runTest("CWE190_Integer_Overflow__listen_tcp_square_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__listen_tcp_square_68a()).runTest("CWE190_Integer_Overflow__listen_tcp_square_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_01()).runTest("CWE190_Integer_Overflow__max_add_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_02()).runTest("CWE190_Integer_Overflow__max_add_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_03()).runTest("CWE190_Integer_Overflow__max_add_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_04()).runTest("CWE190_Integer_Overflow__max_add_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_05()).runTest("CWE190_Integer_Overflow__max_add_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_06()).runTest("CWE190_Integer_Overflow__max_add_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_07()).runTest("CWE190_Integer_Overflow__max_add_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_08()).runTest("CWE190_Integer_Overflow__max_add_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_09()).runTest("CWE190_Integer_Overflow__max_add_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_10()).runTest("CWE190_Integer_Overflow__max_add_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_11()).runTest("CWE190_Integer_Overflow__max_add_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_12()).runTest("CWE190_Integer_Overflow__max_add_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_13()).runTest("CWE190_Integer_Overflow__max_add_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_14()).runTest("CWE190_Integer_Overflow__max_add_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_15()).runTest("CWE190_Integer_Overflow__max_add_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_16()).runTest("CWE190_Integer_Overflow__max_add_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_17()).runTest("CWE190_Integer_Overflow__max_add_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_19()).runTest("CWE190_Integer_Overflow__max_add_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_31()).runTest("CWE190_Integer_Overflow__max_add_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_41()).runTest("CWE190_Integer_Overflow__max_add_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_42()).runTest("CWE190_Integer_Overflow__max_add_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_45()).runTest("CWE190_Integer_Overflow__max_add_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_51a()).runTest("CWE190_Integer_Overflow__max_add_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_52a()).runTest("CWE190_Integer_Overflow__max_add_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_53a()).runTest("CWE190_Integer_Overflow__max_add_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_54a()).runTest("CWE190_Integer_Overflow__max_add_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_61a()).runTest("CWE190_Integer_Overflow__max_add_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_66a()).runTest("CWE190_Integer_Overflow__max_add_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_67a()).runTest("CWE190_Integer_Overflow__max_add_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_add_68a()).runTest("CWE190_Integer_Overflow__max_add_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_01()).runTest("CWE190_Integer_Overflow__max_multiply_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_02()).runTest("CWE190_Integer_Overflow__max_multiply_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_03()).runTest("CWE190_Integer_Overflow__max_multiply_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_04()).runTest("CWE190_Integer_Overflow__max_multiply_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_05()).runTest("CWE190_Integer_Overflow__max_multiply_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_06()).runTest("CWE190_Integer_Overflow__max_multiply_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_07()).runTest("CWE190_Integer_Overflow__max_multiply_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_08()).runTest("CWE190_Integer_Overflow__max_multiply_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_09()).runTest("CWE190_Integer_Overflow__max_multiply_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_10()).runTest("CWE190_Integer_Overflow__max_multiply_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_11()).runTest("CWE190_Integer_Overflow__max_multiply_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_12()).runTest("CWE190_Integer_Overflow__max_multiply_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_13()).runTest("CWE190_Integer_Overflow__max_multiply_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_14()).runTest("CWE190_Integer_Overflow__max_multiply_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_15()).runTest("CWE190_Integer_Overflow__max_multiply_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_16()).runTest("CWE190_Integer_Overflow__max_multiply_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_17()).runTest("CWE190_Integer_Overflow__max_multiply_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_19()).runTest("CWE190_Integer_Overflow__max_multiply_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_31()).runTest("CWE190_Integer_Overflow__max_multiply_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_41()).runTest("CWE190_Integer_Overflow__max_multiply_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_42()).runTest("CWE190_Integer_Overflow__max_multiply_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_45()).runTest("CWE190_Integer_Overflow__max_multiply_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_51a()).runTest("CWE190_Integer_Overflow__max_multiply_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_52a()).runTest("CWE190_Integer_Overflow__max_multiply_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_53a()).runTest("CWE190_Integer_Overflow__max_multiply_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_54a()).runTest("CWE190_Integer_Overflow__max_multiply_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_61a()).runTest("CWE190_Integer_Overflow__max_multiply_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_66a()).runTest("CWE190_Integer_Overflow__max_multiply_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_67a()).runTest("CWE190_Integer_Overflow__max_multiply_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_multiply_68a()).runTest("CWE190_Integer_Overflow__max_multiply_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_01()).runTest("CWE190_Integer_Overflow__max_square_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_02()).runTest("CWE190_Integer_Overflow__max_square_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_03()).runTest("CWE190_Integer_Overflow__max_square_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_04()).runTest("CWE190_Integer_Overflow__max_square_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_05()).runTest("CWE190_Integer_Overflow__max_square_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_06()).runTest("CWE190_Integer_Overflow__max_square_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_07()).runTest("CWE190_Integer_Overflow__max_square_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_08()).runTest("CWE190_Integer_Overflow__max_square_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_09()).runTest("CWE190_Integer_Overflow__max_square_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_10()).runTest("CWE190_Integer_Overflow__max_square_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_11()).runTest("CWE190_Integer_Overflow__max_square_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_12()).runTest("CWE190_Integer_Overflow__max_square_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_13()).runTest("CWE190_Integer_Overflow__max_square_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_14()).runTest("CWE190_Integer_Overflow__max_square_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_15()).runTest("CWE190_Integer_Overflow__max_square_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_16()).runTest("CWE190_Integer_Overflow__max_square_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_17()).runTest("CWE190_Integer_Overflow__max_square_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_19()).runTest("CWE190_Integer_Overflow__max_square_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_31()).runTest("CWE190_Integer_Overflow__max_square_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_41()).runTest("CWE190_Integer_Overflow__max_square_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_42()).runTest("CWE190_Integer_Overflow__max_square_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_45()).runTest("CWE190_Integer_Overflow__max_square_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_51a()).runTest("CWE190_Integer_Overflow__max_square_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_52a()).runTest("CWE190_Integer_Overflow__max_square_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_53a()).runTest("CWE190_Integer_Overflow__max_square_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_54a()).runTest("CWE190_Integer_Overflow__max_square_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_61a()).runTest("CWE190_Integer_Overflow__max_square_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_66a()).runTest("CWE190_Integer_Overflow__max_square_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_67a()).runTest("CWE190_Integer_Overflow__max_square_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__max_square_68a()).runTest("CWE190_Integer_Overflow__max_square_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_01()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_02()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_03()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_04()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_05()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_06()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_07()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_08()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_09()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_10()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_11()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_12()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_13()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_14()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_15()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_16()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_17()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_19()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_31()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_41()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_42()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_45()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_51a()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_52a()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_53a()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_54a()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_61a()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_66a()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_67a()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_add_68a()).runTest("CWE190_Integer_Overflow__PropertiesFile_add_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_01()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_02()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_03()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_04()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_05()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_06()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_07()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_08()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_09()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_10()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_11()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_12()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_13()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_14()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_15()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_16()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_17()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_19()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_31()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_41()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_42()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_45()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_51a()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_52a()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_53a()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_54a()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_61a()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_66a()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_67a()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_multiply_68a()).runTest("CWE190_Integer_Overflow__PropertiesFile_multiply_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_01()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_02()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_03()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_04()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_05()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_06()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_07()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_08()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_09()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_10()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_11()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_12()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_13()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_14()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_15()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_16()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_17()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_19()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_31()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_41()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_42()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_45()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_51a()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_52a()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_53a()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_54a()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_61a()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_66a()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_67a()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__PropertiesFile_square_68a()).runTest("CWE190_Integer_Overflow__PropertiesFile_square_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_01()).runTest("CWE190_Integer_Overflow__random_add_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_02()).runTest("CWE190_Integer_Overflow__random_add_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_03()).runTest("CWE190_Integer_Overflow__random_add_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_04()).runTest("CWE190_Integer_Overflow__random_add_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_05()).runTest("CWE190_Integer_Overflow__random_add_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_06()).runTest("CWE190_Integer_Overflow__random_add_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_07()).runTest("CWE190_Integer_Overflow__random_add_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_08()).runTest("CWE190_Integer_Overflow__random_add_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_09()).runTest("CWE190_Integer_Overflow__random_add_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_10()).runTest("CWE190_Integer_Overflow__random_add_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_11()).runTest("CWE190_Integer_Overflow__random_add_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_12()).runTest("CWE190_Integer_Overflow__random_add_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_13()).runTest("CWE190_Integer_Overflow__random_add_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_14()).runTest("CWE190_Integer_Overflow__random_add_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_15()).runTest("CWE190_Integer_Overflow__random_add_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_16()).runTest("CWE190_Integer_Overflow__random_add_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_17()).runTest("CWE190_Integer_Overflow__random_add_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_19()).runTest("CWE190_Integer_Overflow__random_add_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_31()).runTest("CWE190_Integer_Overflow__random_add_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_41()).runTest("CWE190_Integer_Overflow__random_add_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_42()).runTest("CWE190_Integer_Overflow__random_add_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_45()).runTest("CWE190_Integer_Overflow__random_add_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_51a()).runTest("CWE190_Integer_Overflow__random_add_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_52a()).runTest("CWE190_Integer_Overflow__random_add_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_53a()).runTest("CWE190_Integer_Overflow__random_add_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_54a()).runTest("CWE190_Integer_Overflow__random_add_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_61a()).runTest("CWE190_Integer_Overflow__random_add_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_66a()).runTest("CWE190_Integer_Overflow__random_add_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_67a()).runTest("CWE190_Integer_Overflow__random_add_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_add_68a()).runTest("CWE190_Integer_Overflow__random_add_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_01()).runTest("CWE190_Integer_Overflow__random_multiply_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_02()).runTest("CWE190_Integer_Overflow__random_multiply_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_03()).runTest("CWE190_Integer_Overflow__random_multiply_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_04()).runTest("CWE190_Integer_Overflow__random_multiply_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_05()).runTest("CWE190_Integer_Overflow__random_multiply_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_06()).runTest("CWE190_Integer_Overflow__random_multiply_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_07()).runTest("CWE190_Integer_Overflow__random_multiply_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_08()).runTest("CWE190_Integer_Overflow__random_multiply_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_09()).runTest("CWE190_Integer_Overflow__random_multiply_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_10()).runTest("CWE190_Integer_Overflow__random_multiply_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_11()).runTest("CWE190_Integer_Overflow__random_multiply_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_12()).runTest("CWE190_Integer_Overflow__random_multiply_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_13()).runTest("CWE190_Integer_Overflow__random_multiply_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_14()).runTest("CWE190_Integer_Overflow__random_multiply_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_15()).runTest("CWE190_Integer_Overflow__random_multiply_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_16()).runTest("CWE190_Integer_Overflow__random_multiply_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_17()).runTest("CWE190_Integer_Overflow__random_multiply_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_19()).runTest("CWE190_Integer_Overflow__random_multiply_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_31()).runTest("CWE190_Integer_Overflow__random_multiply_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_41()).runTest("CWE190_Integer_Overflow__random_multiply_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_42()).runTest("CWE190_Integer_Overflow__random_multiply_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_45()).runTest("CWE190_Integer_Overflow__random_multiply_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_51a()).runTest("CWE190_Integer_Overflow__random_multiply_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_52a()).runTest("CWE190_Integer_Overflow__random_multiply_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_53a()).runTest("CWE190_Integer_Overflow__random_multiply_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_54a()).runTest("CWE190_Integer_Overflow__random_multiply_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_61a()).runTest("CWE190_Integer_Overflow__random_multiply_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_66a()).runTest("CWE190_Integer_Overflow__random_multiply_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_67a()).runTest("CWE190_Integer_Overflow__random_multiply_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_multiply_68a()).runTest("CWE190_Integer_Overflow__random_multiply_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_01()).runTest("CWE190_Integer_Overflow__random_square_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_02()).runTest("CWE190_Integer_Overflow__random_square_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_03()).runTest("CWE190_Integer_Overflow__random_square_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_04()).runTest("CWE190_Integer_Overflow__random_square_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_05()).runTest("CWE190_Integer_Overflow__random_square_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_06()).runTest("CWE190_Integer_Overflow__random_square_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_07()).runTest("CWE190_Integer_Overflow__random_square_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_08()).runTest("CWE190_Integer_Overflow__random_square_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_09()).runTest("CWE190_Integer_Overflow__random_square_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_10()).runTest("CWE190_Integer_Overflow__random_square_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_11()).runTest("CWE190_Integer_Overflow__random_square_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_12()).runTest("CWE190_Integer_Overflow__random_square_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_13()).runTest("CWE190_Integer_Overflow__random_square_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_14()).runTest("CWE190_Integer_Overflow__random_square_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_15()).runTest("CWE190_Integer_Overflow__random_square_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_16()).runTest("CWE190_Integer_Overflow__random_square_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_17()).runTest("CWE190_Integer_Overflow__random_square_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_19()).runTest("CWE190_Integer_Overflow__random_square_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_31()).runTest("CWE190_Integer_Overflow__random_square_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_41()).runTest("CWE190_Integer_Overflow__random_square_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_42()).runTest("CWE190_Integer_Overflow__random_square_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_45()).runTest("CWE190_Integer_Overflow__random_square_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_51a()).runTest("CWE190_Integer_Overflow__random_square_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_52a()).runTest("CWE190_Integer_Overflow__random_square_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_53a()).runTest("CWE190_Integer_Overflow__random_square_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_54a()).runTest("CWE190_Integer_Overflow__random_square_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_61a()).runTest("CWE190_Integer_Overflow__random_square_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_66a()).runTest("CWE190_Integer_Overflow__random_square_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_67a()).runTest("CWE190_Integer_Overflow__random_square_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__random_square_68a()).runTest("CWE190_Integer_Overflow__random_square_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_01()).runTest("CWE190_Integer_Overflow__URLConnection_add_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_02()).runTest("CWE190_Integer_Overflow__URLConnection_add_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_03()).runTest("CWE190_Integer_Overflow__URLConnection_add_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_04()).runTest("CWE190_Integer_Overflow__URLConnection_add_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_05()).runTest("CWE190_Integer_Overflow__URLConnection_add_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_06()).runTest("CWE190_Integer_Overflow__URLConnection_add_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_07()).runTest("CWE190_Integer_Overflow__URLConnection_add_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_08()).runTest("CWE190_Integer_Overflow__URLConnection_add_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_09()).runTest("CWE190_Integer_Overflow__URLConnection_add_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_10()).runTest("CWE190_Integer_Overflow__URLConnection_add_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_11()).runTest("CWE190_Integer_Overflow__URLConnection_add_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_12()).runTest("CWE190_Integer_Overflow__URLConnection_add_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_13()).runTest("CWE190_Integer_Overflow__URLConnection_add_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_14()).runTest("CWE190_Integer_Overflow__URLConnection_add_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_15()).runTest("CWE190_Integer_Overflow__URLConnection_add_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_16()).runTest("CWE190_Integer_Overflow__URLConnection_add_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_17()).runTest("CWE190_Integer_Overflow__URLConnection_add_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_19()).runTest("CWE190_Integer_Overflow__URLConnection_add_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_31()).runTest("CWE190_Integer_Overflow__URLConnection_add_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_41()).runTest("CWE190_Integer_Overflow__URLConnection_add_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_42()).runTest("CWE190_Integer_Overflow__URLConnection_add_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_45()).runTest("CWE190_Integer_Overflow__URLConnection_add_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_51a()).runTest("CWE190_Integer_Overflow__URLConnection_add_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_52a()).runTest("CWE190_Integer_Overflow__URLConnection_add_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_53a()).runTest("CWE190_Integer_Overflow__URLConnection_add_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_54a()).runTest("CWE190_Integer_Overflow__URLConnection_add_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_61a()).runTest("CWE190_Integer_Overflow__URLConnection_add_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_66a()).runTest("CWE190_Integer_Overflow__URLConnection_add_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_67a()).runTest("CWE190_Integer_Overflow__URLConnection_add_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_add_68a()).runTest("CWE190_Integer_Overflow__URLConnection_add_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_01()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_02()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_03()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_04()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_05()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_06()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_07()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_08()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_09()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_10()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_11()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_12()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_13()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_14()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_15()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_16()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_17()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_19()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_31()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_41()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_42()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_45()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_51a()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_52a()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_53a()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_54a()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_61a()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_66a()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_67a()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_multiply_68a()).runTest("CWE190_Integer_Overflow__URLConnection_multiply_68a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_01()).runTest("CWE190_Integer_Overflow__URLConnection_square_01");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_02()).runTest("CWE190_Integer_Overflow__URLConnection_square_02");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_03()).runTest("CWE190_Integer_Overflow__URLConnection_square_03");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_04()).runTest("CWE190_Integer_Overflow__URLConnection_square_04");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_05()).runTest("CWE190_Integer_Overflow__URLConnection_square_05");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_06()).runTest("CWE190_Integer_Overflow__URLConnection_square_06");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_07()).runTest("CWE190_Integer_Overflow__URLConnection_square_07");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_08()).runTest("CWE190_Integer_Overflow__URLConnection_square_08");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_09()).runTest("CWE190_Integer_Overflow__URLConnection_square_09");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_10()).runTest("CWE190_Integer_Overflow__URLConnection_square_10");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_11()).runTest("CWE190_Integer_Overflow__URLConnection_square_11");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_12()).runTest("CWE190_Integer_Overflow__URLConnection_square_12");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_13()).runTest("CWE190_Integer_Overflow__URLConnection_square_13");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_14()).runTest("CWE190_Integer_Overflow__URLConnection_square_14");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_15()).runTest("CWE190_Integer_Overflow__URLConnection_square_15");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_16()).runTest("CWE190_Integer_Overflow__URLConnection_square_16");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_17()).runTest("CWE190_Integer_Overflow__URLConnection_square_17");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_19()).runTest("CWE190_Integer_Overflow__URLConnection_square_19");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_31()).runTest("CWE190_Integer_Overflow__URLConnection_square_31");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_41()).runTest("CWE190_Integer_Overflow__URLConnection_square_41");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_42()).runTest("CWE190_Integer_Overflow__URLConnection_square_42");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_45()).runTest("CWE190_Integer_Overflow__URLConnection_square_45");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_51a()).runTest("CWE190_Integer_Overflow__URLConnection_square_51a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_52a()).runTest("CWE190_Integer_Overflow__URLConnection_square_52a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_53a()).runTest("CWE190_Integer_Overflow__URLConnection_square_53a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_54a()).runTest("CWE190_Integer_Overflow__URLConnection_square_54a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_61a()).runTest("CWE190_Integer_Overflow__URLConnection_square_61a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_66a()).runTest("CWE190_Integer_Overflow__URLConnection_square_66a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_67a()).runTest("CWE190_Integer_Overflow__URLConnection_square_67a");
			(new testcases.CWE190_Integer_Overflow.CWE190_Integer_Overflow__URLConnection_square_68a()).runTest("CWE190_Integer_Overflow__URLConnection_square_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_01()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_02()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_03()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_04()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_05()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_06()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_07()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_08()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_09()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_10()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_11()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_12()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_13()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_14()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_15()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_16()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_17()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_19()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_31()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_41()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_42()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_45()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_51a()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_52a()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_53a()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_54a()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_61a()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_66a()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_67a()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_multiply_68a()).runTest("CWE191_Integer_Underflow__connect_tcp_multiply_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_01()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_02()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_03()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_04()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_05()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_06()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_07()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_08()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_09()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_10()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_11()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_12()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_13()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_14()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_15()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_16()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_17()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_19()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_31()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_41()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_42()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_45()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_51a()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_52a()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_53a()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_54a()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_61a()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_66a()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_67a()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__connect_tcp_subtract_68a()).runTest("CWE191_Integer_Underflow__connect_tcp_subtract_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_01()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_02()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_03()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_04()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_05()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_06()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_07()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_08()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_09()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_10()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_11()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_12()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_13()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_14()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_15()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_16()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_17()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_19()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_31()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_41()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_42()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_45()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_51a()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_52a()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_53a()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_54a()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_61a()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_66a()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_67a()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_multiply_68a()).runTest("CWE191_Integer_Underflow__console_readLine_multiply_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_01()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_02()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_03()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_04()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_05()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_06()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_07()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_08()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_09()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_10()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_11()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_12()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_13()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_14()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_15()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_16()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_17()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_19()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_31()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_41()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_42()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_45()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_51a()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_52a()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_53a()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_54a()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_61a()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_66a()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_67a()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__console_readLine_subtract_68a()).runTest("CWE191_Integer_Underflow__console_readLine_subtract_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_01()).runTest("CWE191_Integer_Underflow__Environment_multiply_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_02()).runTest("CWE191_Integer_Underflow__Environment_multiply_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_03()).runTest("CWE191_Integer_Underflow__Environment_multiply_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_04()).runTest("CWE191_Integer_Underflow__Environment_multiply_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_05()).runTest("CWE191_Integer_Underflow__Environment_multiply_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_06()).runTest("CWE191_Integer_Underflow__Environment_multiply_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_07()).runTest("CWE191_Integer_Underflow__Environment_multiply_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_08()).runTest("CWE191_Integer_Underflow__Environment_multiply_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_09()).runTest("CWE191_Integer_Underflow__Environment_multiply_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_10()).runTest("CWE191_Integer_Underflow__Environment_multiply_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_11()).runTest("CWE191_Integer_Underflow__Environment_multiply_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_12()).runTest("CWE191_Integer_Underflow__Environment_multiply_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_13()).runTest("CWE191_Integer_Underflow__Environment_multiply_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_14()).runTest("CWE191_Integer_Underflow__Environment_multiply_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_15()).runTest("CWE191_Integer_Underflow__Environment_multiply_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_16()).runTest("CWE191_Integer_Underflow__Environment_multiply_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_17()).runTest("CWE191_Integer_Underflow__Environment_multiply_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_19()).runTest("CWE191_Integer_Underflow__Environment_multiply_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_31()).runTest("CWE191_Integer_Underflow__Environment_multiply_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_41()).runTest("CWE191_Integer_Underflow__Environment_multiply_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_42()).runTest("CWE191_Integer_Underflow__Environment_multiply_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_45()).runTest("CWE191_Integer_Underflow__Environment_multiply_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_51a()).runTest("CWE191_Integer_Underflow__Environment_multiply_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_52a()).runTest("CWE191_Integer_Underflow__Environment_multiply_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_53a()).runTest("CWE191_Integer_Underflow__Environment_multiply_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_54a()).runTest("CWE191_Integer_Underflow__Environment_multiply_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_61a()).runTest("CWE191_Integer_Underflow__Environment_multiply_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_66a()).runTest("CWE191_Integer_Underflow__Environment_multiply_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_67a()).runTest("CWE191_Integer_Underflow__Environment_multiply_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_multiply_68a()).runTest("CWE191_Integer_Underflow__Environment_multiply_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_01()).runTest("CWE191_Integer_Underflow__Environment_subtract_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_02()).runTest("CWE191_Integer_Underflow__Environment_subtract_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_03()).runTest("CWE191_Integer_Underflow__Environment_subtract_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_04()).runTest("CWE191_Integer_Underflow__Environment_subtract_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_05()).runTest("CWE191_Integer_Underflow__Environment_subtract_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_06()).runTest("CWE191_Integer_Underflow__Environment_subtract_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_07()).runTest("CWE191_Integer_Underflow__Environment_subtract_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_08()).runTest("CWE191_Integer_Underflow__Environment_subtract_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_09()).runTest("CWE191_Integer_Underflow__Environment_subtract_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_10()).runTest("CWE191_Integer_Underflow__Environment_subtract_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_11()).runTest("CWE191_Integer_Underflow__Environment_subtract_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_12()).runTest("CWE191_Integer_Underflow__Environment_subtract_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_13()).runTest("CWE191_Integer_Underflow__Environment_subtract_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_14()).runTest("CWE191_Integer_Underflow__Environment_subtract_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_15()).runTest("CWE191_Integer_Underflow__Environment_subtract_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_16()).runTest("CWE191_Integer_Underflow__Environment_subtract_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_17()).runTest("CWE191_Integer_Underflow__Environment_subtract_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_19()).runTest("CWE191_Integer_Underflow__Environment_subtract_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_31()).runTest("CWE191_Integer_Underflow__Environment_subtract_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_41()).runTest("CWE191_Integer_Underflow__Environment_subtract_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_42()).runTest("CWE191_Integer_Underflow__Environment_subtract_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_45()).runTest("CWE191_Integer_Underflow__Environment_subtract_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_51a()).runTest("CWE191_Integer_Underflow__Environment_subtract_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_52a()).runTest("CWE191_Integer_Underflow__Environment_subtract_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_53a()).runTest("CWE191_Integer_Underflow__Environment_subtract_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_54a()).runTest("CWE191_Integer_Underflow__Environment_subtract_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_61a()).runTest("CWE191_Integer_Underflow__Environment_subtract_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_66a()).runTest("CWE191_Integer_Underflow__Environment_subtract_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_67a()).runTest("CWE191_Integer_Underflow__Environment_subtract_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__Environment_subtract_68a()).runTest("CWE191_Integer_Underflow__Environment_subtract_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_01()).runTest("CWE191_Integer_Underflow__fromDB_multiply_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_02()).runTest("CWE191_Integer_Underflow__fromDB_multiply_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_03()).runTest("CWE191_Integer_Underflow__fromDB_multiply_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_04()).runTest("CWE191_Integer_Underflow__fromDB_multiply_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_05()).runTest("CWE191_Integer_Underflow__fromDB_multiply_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_06()).runTest("CWE191_Integer_Underflow__fromDB_multiply_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_07()).runTest("CWE191_Integer_Underflow__fromDB_multiply_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_08()).runTest("CWE191_Integer_Underflow__fromDB_multiply_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_09()).runTest("CWE191_Integer_Underflow__fromDB_multiply_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_10()).runTest("CWE191_Integer_Underflow__fromDB_multiply_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_11()).runTest("CWE191_Integer_Underflow__fromDB_multiply_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_12()).runTest("CWE191_Integer_Underflow__fromDB_multiply_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_13()).runTest("CWE191_Integer_Underflow__fromDB_multiply_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_14()).runTest("CWE191_Integer_Underflow__fromDB_multiply_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_15()).runTest("CWE191_Integer_Underflow__fromDB_multiply_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_16()).runTest("CWE191_Integer_Underflow__fromDB_multiply_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_17()).runTest("CWE191_Integer_Underflow__fromDB_multiply_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_19()).runTest("CWE191_Integer_Underflow__fromDB_multiply_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_31()).runTest("CWE191_Integer_Underflow__fromDB_multiply_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_41()).runTest("CWE191_Integer_Underflow__fromDB_multiply_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_42()).runTest("CWE191_Integer_Underflow__fromDB_multiply_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_45()).runTest("CWE191_Integer_Underflow__fromDB_multiply_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_51a()).runTest("CWE191_Integer_Underflow__fromDB_multiply_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_52a()).runTest("CWE191_Integer_Underflow__fromDB_multiply_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_53a()).runTest("CWE191_Integer_Underflow__fromDB_multiply_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_54a()).runTest("CWE191_Integer_Underflow__fromDB_multiply_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_61a()).runTest("CWE191_Integer_Underflow__fromDB_multiply_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_66a()).runTest("CWE191_Integer_Underflow__fromDB_multiply_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_67a()).runTest("CWE191_Integer_Underflow__fromDB_multiply_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_multiply_68a()).runTest("CWE191_Integer_Underflow__fromDB_multiply_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_01()).runTest("CWE191_Integer_Underflow__fromDB_subtract_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_02()).runTest("CWE191_Integer_Underflow__fromDB_subtract_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_03()).runTest("CWE191_Integer_Underflow__fromDB_subtract_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_04()).runTest("CWE191_Integer_Underflow__fromDB_subtract_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_05()).runTest("CWE191_Integer_Underflow__fromDB_subtract_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_06()).runTest("CWE191_Integer_Underflow__fromDB_subtract_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_07()).runTest("CWE191_Integer_Underflow__fromDB_subtract_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_08()).runTest("CWE191_Integer_Underflow__fromDB_subtract_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_09()).runTest("CWE191_Integer_Underflow__fromDB_subtract_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_10()).runTest("CWE191_Integer_Underflow__fromDB_subtract_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_11()).runTest("CWE191_Integer_Underflow__fromDB_subtract_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_12()).runTest("CWE191_Integer_Underflow__fromDB_subtract_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_13()).runTest("CWE191_Integer_Underflow__fromDB_subtract_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_14()).runTest("CWE191_Integer_Underflow__fromDB_subtract_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_15()).runTest("CWE191_Integer_Underflow__fromDB_subtract_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_16()).runTest("CWE191_Integer_Underflow__fromDB_subtract_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_17()).runTest("CWE191_Integer_Underflow__fromDB_subtract_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_19()).runTest("CWE191_Integer_Underflow__fromDB_subtract_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_31()).runTest("CWE191_Integer_Underflow__fromDB_subtract_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_41()).runTest("CWE191_Integer_Underflow__fromDB_subtract_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_42()).runTest("CWE191_Integer_Underflow__fromDB_subtract_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_45()).runTest("CWE191_Integer_Underflow__fromDB_subtract_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_51a()).runTest("CWE191_Integer_Underflow__fromDB_subtract_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_52a()).runTest("CWE191_Integer_Underflow__fromDB_subtract_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_53a()).runTest("CWE191_Integer_Underflow__fromDB_subtract_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_54a()).runTest("CWE191_Integer_Underflow__fromDB_subtract_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_61a()).runTest("CWE191_Integer_Underflow__fromDB_subtract_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_66a()).runTest("CWE191_Integer_Underflow__fromDB_subtract_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_67a()).runTest("CWE191_Integer_Underflow__fromDB_subtract_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromDB_subtract_68a()).runTest("CWE191_Integer_Underflow__fromDB_subtract_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_01()).runTest("CWE191_Integer_Underflow__fromFile_multiply_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_02()).runTest("CWE191_Integer_Underflow__fromFile_multiply_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_03()).runTest("CWE191_Integer_Underflow__fromFile_multiply_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_04()).runTest("CWE191_Integer_Underflow__fromFile_multiply_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_05()).runTest("CWE191_Integer_Underflow__fromFile_multiply_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_06()).runTest("CWE191_Integer_Underflow__fromFile_multiply_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_07()).runTest("CWE191_Integer_Underflow__fromFile_multiply_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_08()).runTest("CWE191_Integer_Underflow__fromFile_multiply_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_09()).runTest("CWE191_Integer_Underflow__fromFile_multiply_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_10()).runTest("CWE191_Integer_Underflow__fromFile_multiply_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_11()).runTest("CWE191_Integer_Underflow__fromFile_multiply_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_12()).runTest("CWE191_Integer_Underflow__fromFile_multiply_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_13()).runTest("CWE191_Integer_Underflow__fromFile_multiply_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_14()).runTest("CWE191_Integer_Underflow__fromFile_multiply_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_15()).runTest("CWE191_Integer_Underflow__fromFile_multiply_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_16()).runTest("CWE191_Integer_Underflow__fromFile_multiply_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_17()).runTest("CWE191_Integer_Underflow__fromFile_multiply_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_19()).runTest("CWE191_Integer_Underflow__fromFile_multiply_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_31()).runTest("CWE191_Integer_Underflow__fromFile_multiply_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_41()).runTest("CWE191_Integer_Underflow__fromFile_multiply_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_42()).runTest("CWE191_Integer_Underflow__fromFile_multiply_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_45()).runTest("CWE191_Integer_Underflow__fromFile_multiply_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_51a()).runTest("CWE191_Integer_Underflow__fromFile_multiply_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_52a()).runTest("CWE191_Integer_Underflow__fromFile_multiply_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_53a()).runTest("CWE191_Integer_Underflow__fromFile_multiply_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_54a()).runTest("CWE191_Integer_Underflow__fromFile_multiply_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_61a()).runTest("CWE191_Integer_Underflow__fromFile_multiply_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_66a()).runTest("CWE191_Integer_Underflow__fromFile_multiply_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_67a()).runTest("CWE191_Integer_Underflow__fromFile_multiply_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_multiply_68a()).runTest("CWE191_Integer_Underflow__fromFile_multiply_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_01()).runTest("CWE191_Integer_Underflow__fromFile_subtract_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_02()).runTest("CWE191_Integer_Underflow__fromFile_subtract_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_03()).runTest("CWE191_Integer_Underflow__fromFile_subtract_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_04()).runTest("CWE191_Integer_Underflow__fromFile_subtract_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_05()).runTest("CWE191_Integer_Underflow__fromFile_subtract_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_06()).runTest("CWE191_Integer_Underflow__fromFile_subtract_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_07()).runTest("CWE191_Integer_Underflow__fromFile_subtract_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_08()).runTest("CWE191_Integer_Underflow__fromFile_subtract_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_09()).runTest("CWE191_Integer_Underflow__fromFile_subtract_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_10()).runTest("CWE191_Integer_Underflow__fromFile_subtract_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_11()).runTest("CWE191_Integer_Underflow__fromFile_subtract_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_12()).runTest("CWE191_Integer_Underflow__fromFile_subtract_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_13()).runTest("CWE191_Integer_Underflow__fromFile_subtract_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_14()).runTest("CWE191_Integer_Underflow__fromFile_subtract_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_15()).runTest("CWE191_Integer_Underflow__fromFile_subtract_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_16()).runTest("CWE191_Integer_Underflow__fromFile_subtract_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_17()).runTest("CWE191_Integer_Underflow__fromFile_subtract_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_19()).runTest("CWE191_Integer_Underflow__fromFile_subtract_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_31()).runTest("CWE191_Integer_Underflow__fromFile_subtract_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_41()).runTest("CWE191_Integer_Underflow__fromFile_subtract_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_42()).runTest("CWE191_Integer_Underflow__fromFile_subtract_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_45()).runTest("CWE191_Integer_Underflow__fromFile_subtract_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_51a()).runTest("CWE191_Integer_Underflow__fromFile_subtract_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_52a()).runTest("CWE191_Integer_Underflow__fromFile_subtract_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_53a()).runTest("CWE191_Integer_Underflow__fromFile_subtract_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_54a()).runTest("CWE191_Integer_Underflow__fromFile_subtract_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_61a()).runTest("CWE191_Integer_Underflow__fromFile_subtract_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_66a()).runTest("CWE191_Integer_Underflow__fromFile_subtract_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_67a()).runTest("CWE191_Integer_Underflow__fromFile_subtract_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__fromFile_subtract_68a()).runTest("CWE191_Integer_Underflow__fromFile_subtract_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_01()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_02()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_03()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_04()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_05()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_06()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_07()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_08()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_09()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_10()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_11()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_12()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_13()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_14()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_15()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_16()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_17()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_19()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_31()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_41()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_42()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_45()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_51a()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_52a()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_53a()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_54a()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_61a()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_66a()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_67a()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_multiply_68a()).runTest("CWE191_Integer_Underflow__listen_tcp_multiply_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_01()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_02()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_03()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_04()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_05()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_06()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_07()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_08()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_09()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_10()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_11()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_12()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_13()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_14()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_15()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_16()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_17()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_19()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_31()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_41()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_42()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_45()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_51a()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_52a()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_53a()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_54a()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_61a()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_66a()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_67a()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__listen_tcp_subtract_68a()).runTest("CWE191_Integer_Underflow__listen_tcp_subtract_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_01()).runTest("CWE191_Integer_Underflow__min_multiply_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_02()).runTest("CWE191_Integer_Underflow__min_multiply_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_03()).runTest("CWE191_Integer_Underflow__min_multiply_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_04()).runTest("CWE191_Integer_Underflow__min_multiply_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_05()).runTest("CWE191_Integer_Underflow__min_multiply_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_06()).runTest("CWE191_Integer_Underflow__min_multiply_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_07()).runTest("CWE191_Integer_Underflow__min_multiply_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_08()).runTest("CWE191_Integer_Underflow__min_multiply_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_09()).runTest("CWE191_Integer_Underflow__min_multiply_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_10()).runTest("CWE191_Integer_Underflow__min_multiply_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_11()).runTest("CWE191_Integer_Underflow__min_multiply_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_12()).runTest("CWE191_Integer_Underflow__min_multiply_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_13()).runTest("CWE191_Integer_Underflow__min_multiply_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_14()).runTest("CWE191_Integer_Underflow__min_multiply_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_15()).runTest("CWE191_Integer_Underflow__min_multiply_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_16()).runTest("CWE191_Integer_Underflow__min_multiply_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_17()).runTest("CWE191_Integer_Underflow__min_multiply_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_19()).runTest("CWE191_Integer_Underflow__min_multiply_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_31()).runTest("CWE191_Integer_Underflow__min_multiply_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_41()).runTest("CWE191_Integer_Underflow__min_multiply_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_42()).runTest("CWE191_Integer_Underflow__min_multiply_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_45()).runTest("CWE191_Integer_Underflow__min_multiply_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_51a()).runTest("CWE191_Integer_Underflow__min_multiply_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_52a()).runTest("CWE191_Integer_Underflow__min_multiply_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_53a()).runTest("CWE191_Integer_Underflow__min_multiply_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_54a()).runTest("CWE191_Integer_Underflow__min_multiply_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_61a()).runTest("CWE191_Integer_Underflow__min_multiply_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_66a()).runTest("CWE191_Integer_Underflow__min_multiply_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_67a()).runTest("CWE191_Integer_Underflow__min_multiply_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_multiply_68a()).runTest("CWE191_Integer_Underflow__min_multiply_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_01()).runTest("CWE191_Integer_Underflow__min_subtract_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_02()).runTest("CWE191_Integer_Underflow__min_subtract_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_03()).runTest("CWE191_Integer_Underflow__min_subtract_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_04()).runTest("CWE191_Integer_Underflow__min_subtract_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_05()).runTest("CWE191_Integer_Underflow__min_subtract_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_06()).runTest("CWE191_Integer_Underflow__min_subtract_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_07()).runTest("CWE191_Integer_Underflow__min_subtract_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_08()).runTest("CWE191_Integer_Underflow__min_subtract_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_09()).runTest("CWE191_Integer_Underflow__min_subtract_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_10()).runTest("CWE191_Integer_Underflow__min_subtract_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_11()).runTest("CWE191_Integer_Underflow__min_subtract_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_12()).runTest("CWE191_Integer_Underflow__min_subtract_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_13()).runTest("CWE191_Integer_Underflow__min_subtract_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_14()).runTest("CWE191_Integer_Underflow__min_subtract_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_15()).runTest("CWE191_Integer_Underflow__min_subtract_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_16()).runTest("CWE191_Integer_Underflow__min_subtract_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_17()).runTest("CWE191_Integer_Underflow__min_subtract_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_19()).runTest("CWE191_Integer_Underflow__min_subtract_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_31()).runTest("CWE191_Integer_Underflow__min_subtract_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_41()).runTest("CWE191_Integer_Underflow__min_subtract_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_42()).runTest("CWE191_Integer_Underflow__min_subtract_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_45()).runTest("CWE191_Integer_Underflow__min_subtract_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_51a()).runTest("CWE191_Integer_Underflow__min_subtract_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_52a()).runTest("CWE191_Integer_Underflow__min_subtract_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_53a()).runTest("CWE191_Integer_Underflow__min_subtract_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_54a()).runTest("CWE191_Integer_Underflow__min_subtract_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_61a()).runTest("CWE191_Integer_Underflow__min_subtract_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_66a()).runTest("CWE191_Integer_Underflow__min_subtract_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_67a()).runTest("CWE191_Integer_Underflow__min_subtract_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__min_subtract_68a()).runTest("CWE191_Integer_Underflow__min_subtract_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_01()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_02()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_03()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_04()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_05()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_06()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_07()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_08()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_09()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_10()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_11()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_12()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_13()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_14()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_15()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_16()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_17()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_19()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_31()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_41()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_42()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_45()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_51a()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_52a()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_53a()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_54a()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_61a()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_66a()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_67a()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_multiply_68a()).runTest("CWE191_Integer_Underflow__PropertiesFile_multiply_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_01()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_02()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_03()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_04()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_05()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_06()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_07()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_08()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_09()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_10()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_11()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_12()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_13()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_14()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_15()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_16()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_17()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_19()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_31()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_41()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_42()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_45()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_51a()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_52a()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_53a()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_54a()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_61a()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_66a()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_67a()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__PropertiesFile_subtract_68a()).runTest("CWE191_Integer_Underflow__PropertiesFile_subtract_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_01()).runTest("CWE191_Integer_Underflow__random_multiply_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_02()).runTest("CWE191_Integer_Underflow__random_multiply_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_03()).runTest("CWE191_Integer_Underflow__random_multiply_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_04()).runTest("CWE191_Integer_Underflow__random_multiply_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_05()).runTest("CWE191_Integer_Underflow__random_multiply_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_06()).runTest("CWE191_Integer_Underflow__random_multiply_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_07()).runTest("CWE191_Integer_Underflow__random_multiply_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_08()).runTest("CWE191_Integer_Underflow__random_multiply_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_09()).runTest("CWE191_Integer_Underflow__random_multiply_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_10()).runTest("CWE191_Integer_Underflow__random_multiply_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_11()).runTest("CWE191_Integer_Underflow__random_multiply_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_12()).runTest("CWE191_Integer_Underflow__random_multiply_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_13()).runTest("CWE191_Integer_Underflow__random_multiply_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_14()).runTest("CWE191_Integer_Underflow__random_multiply_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_15()).runTest("CWE191_Integer_Underflow__random_multiply_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_16()).runTest("CWE191_Integer_Underflow__random_multiply_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_17()).runTest("CWE191_Integer_Underflow__random_multiply_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_19()).runTest("CWE191_Integer_Underflow__random_multiply_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_31()).runTest("CWE191_Integer_Underflow__random_multiply_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_41()).runTest("CWE191_Integer_Underflow__random_multiply_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_42()).runTest("CWE191_Integer_Underflow__random_multiply_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_45()).runTest("CWE191_Integer_Underflow__random_multiply_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_51a()).runTest("CWE191_Integer_Underflow__random_multiply_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_52a()).runTest("CWE191_Integer_Underflow__random_multiply_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_53a()).runTest("CWE191_Integer_Underflow__random_multiply_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_54a()).runTest("CWE191_Integer_Underflow__random_multiply_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_61a()).runTest("CWE191_Integer_Underflow__random_multiply_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_66a()).runTest("CWE191_Integer_Underflow__random_multiply_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_67a()).runTest("CWE191_Integer_Underflow__random_multiply_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_multiply_68a()).runTest("CWE191_Integer_Underflow__random_multiply_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_01()).runTest("CWE191_Integer_Underflow__random_subtract_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_02()).runTest("CWE191_Integer_Underflow__random_subtract_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_03()).runTest("CWE191_Integer_Underflow__random_subtract_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_04()).runTest("CWE191_Integer_Underflow__random_subtract_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_05()).runTest("CWE191_Integer_Underflow__random_subtract_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_06()).runTest("CWE191_Integer_Underflow__random_subtract_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_07()).runTest("CWE191_Integer_Underflow__random_subtract_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_08()).runTest("CWE191_Integer_Underflow__random_subtract_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_09()).runTest("CWE191_Integer_Underflow__random_subtract_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_10()).runTest("CWE191_Integer_Underflow__random_subtract_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_11()).runTest("CWE191_Integer_Underflow__random_subtract_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_12()).runTest("CWE191_Integer_Underflow__random_subtract_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_13()).runTest("CWE191_Integer_Underflow__random_subtract_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_14()).runTest("CWE191_Integer_Underflow__random_subtract_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_15()).runTest("CWE191_Integer_Underflow__random_subtract_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_16()).runTest("CWE191_Integer_Underflow__random_subtract_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_17()).runTest("CWE191_Integer_Underflow__random_subtract_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_19()).runTest("CWE191_Integer_Underflow__random_subtract_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_31()).runTest("CWE191_Integer_Underflow__random_subtract_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_41()).runTest("CWE191_Integer_Underflow__random_subtract_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_42()).runTest("CWE191_Integer_Underflow__random_subtract_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_45()).runTest("CWE191_Integer_Underflow__random_subtract_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_51a()).runTest("CWE191_Integer_Underflow__random_subtract_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_52a()).runTest("CWE191_Integer_Underflow__random_subtract_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_53a()).runTest("CWE191_Integer_Underflow__random_subtract_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_54a()).runTest("CWE191_Integer_Underflow__random_subtract_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_61a()).runTest("CWE191_Integer_Underflow__random_subtract_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_66a()).runTest("CWE191_Integer_Underflow__random_subtract_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_67a()).runTest("CWE191_Integer_Underflow__random_subtract_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__random_subtract_68a()).runTest("CWE191_Integer_Underflow__random_subtract_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_01()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_02()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_03()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_04()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_05()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_06()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_07()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_08()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_09()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_10()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_11()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_12()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_13()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_14()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_15()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_16()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_17()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_19()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_31()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_41()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_42()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_45()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_51a()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_52a()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_53a()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_54a()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_61a()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_66a()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_67a()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_multiply_68a()).runTest("CWE191_Integer_Underflow__URLConnection_multiply_68a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_01()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_01");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_02()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_02");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_03()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_03");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_04()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_04");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_05()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_05");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_06()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_06");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_07()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_07");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_08()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_08");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_09()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_09");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_10()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_10");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_11()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_11");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_12()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_12");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_13()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_13");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_14()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_14");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_15()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_15");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_16()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_16");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_17()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_17");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_19()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_19");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_31()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_31");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_41()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_41");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_42()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_42");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_45()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_45");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_51a()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_51a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_52a()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_52a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_53a()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_53a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_54a()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_54a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_61a()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_61a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_66a()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_66a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_67a()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_67a");
			(new testcases.CWE191_Integer_Underflow.CWE191_Integer_Underflow__URLConnection_subtract_68a()).runTest("CWE191_Integer_Underflow__URLConnection_subtract_68a");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_01()).runTest("CWE193_Off_by_One_Error__basic_01");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_02()).runTest("CWE193_Off_by_One_Error__basic_02");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_03()).runTest("CWE193_Off_by_One_Error__basic_03");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_04()).runTest("CWE193_Off_by_One_Error__basic_04");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_05()).runTest("CWE193_Off_by_One_Error__basic_05");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_06()).runTest("CWE193_Off_by_One_Error__basic_06");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_07()).runTest("CWE193_Off_by_One_Error__basic_07");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_08()).runTest("CWE193_Off_by_One_Error__basic_08");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_09()).runTest("CWE193_Off_by_One_Error__basic_09");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_10()).runTest("CWE193_Off_by_One_Error__basic_10");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_11()).runTest("CWE193_Off_by_One_Error__basic_11");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_12()).runTest("CWE193_Off_by_One_Error__basic_12");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_13()).runTest("CWE193_Off_by_One_Error__basic_13");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_14()).runTest("CWE193_Off_by_One_Error__basic_14");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_15()).runTest("CWE193_Off_by_One_Error__basic_15");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_16()).runTest("CWE193_Off_by_One_Error__basic_16");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_17()).runTest("CWE193_Off_by_One_Error__basic_17");
			(new testcases.CWE193_Off_by_One_Error.CWE193_Off_by_One_Error__basic_19()).runTest("CWE193_Off_by_One_Error__basic_19");
			/* END-AUTOGENERATED-JAVA-TESTS-1 */
}

private static void runTestCWE2() {
/* BEGIN-AUTOGENERATED-JAVA-TESTS-2 */
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_01()).runTest("CWE209_Information_Leak_Error__connect_tcp_01");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_02()).runTest("CWE209_Information_Leak_Error__connect_tcp_02");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_03()).runTest("CWE209_Information_Leak_Error__connect_tcp_03");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_04()).runTest("CWE209_Information_Leak_Error__connect_tcp_04");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_05()).runTest("CWE209_Information_Leak_Error__connect_tcp_05");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_06()).runTest("CWE209_Information_Leak_Error__connect_tcp_06");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_07()).runTest("CWE209_Information_Leak_Error__connect_tcp_07");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_08()).runTest("CWE209_Information_Leak_Error__connect_tcp_08");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_09()).runTest("CWE209_Information_Leak_Error__connect_tcp_09");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_10()).runTest("CWE209_Information_Leak_Error__connect_tcp_10");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_11()).runTest("CWE209_Information_Leak_Error__connect_tcp_11");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_12()).runTest("CWE209_Information_Leak_Error__connect_tcp_12");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_13()).runTest("CWE209_Information_Leak_Error__connect_tcp_13");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_14()).runTest("CWE209_Information_Leak_Error__connect_tcp_14");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_15()).runTest("CWE209_Information_Leak_Error__connect_tcp_15");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_16()).runTest("CWE209_Information_Leak_Error__connect_tcp_16");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_17()).runTest("CWE209_Information_Leak_Error__connect_tcp_17");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_19()).runTest("CWE209_Information_Leak_Error__connect_tcp_19");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_31()).runTest("CWE209_Information_Leak_Error__connect_tcp_31");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_41()).runTest("CWE209_Information_Leak_Error__connect_tcp_41");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_42()).runTest("CWE209_Information_Leak_Error__connect_tcp_42");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_45()).runTest("CWE209_Information_Leak_Error__connect_tcp_45");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_51a()).runTest("CWE209_Information_Leak_Error__connect_tcp_51a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_52a()).runTest("CWE209_Information_Leak_Error__connect_tcp_52a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_53a()).runTest("CWE209_Information_Leak_Error__connect_tcp_53a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_54a()).runTest("CWE209_Information_Leak_Error__connect_tcp_54a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_61a()).runTest("CWE209_Information_Leak_Error__connect_tcp_61a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_66a()).runTest("CWE209_Information_Leak_Error__connect_tcp_66a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_67a()).runTest("CWE209_Information_Leak_Error__connect_tcp_67a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_68a()).runTest("CWE209_Information_Leak_Error__connect_tcp_68a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__connect_tcp_71a()).runTest("CWE209_Information_Leak_Error__connect_tcp_71a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_01()).runTest("CWE209_Information_Leak_Error__console_readLine_01");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_02()).runTest("CWE209_Information_Leak_Error__console_readLine_02");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_03()).runTest("CWE209_Information_Leak_Error__console_readLine_03");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_04()).runTest("CWE209_Information_Leak_Error__console_readLine_04");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_05()).runTest("CWE209_Information_Leak_Error__console_readLine_05");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_06()).runTest("CWE209_Information_Leak_Error__console_readLine_06");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_07()).runTest("CWE209_Information_Leak_Error__console_readLine_07");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_08()).runTest("CWE209_Information_Leak_Error__console_readLine_08");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_09()).runTest("CWE209_Information_Leak_Error__console_readLine_09");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_10()).runTest("CWE209_Information_Leak_Error__console_readLine_10");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_11()).runTest("CWE209_Information_Leak_Error__console_readLine_11");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_12()).runTest("CWE209_Information_Leak_Error__console_readLine_12");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_13()).runTest("CWE209_Information_Leak_Error__console_readLine_13");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_14()).runTest("CWE209_Information_Leak_Error__console_readLine_14");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_15()).runTest("CWE209_Information_Leak_Error__console_readLine_15");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_16()).runTest("CWE209_Information_Leak_Error__console_readLine_16");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_17()).runTest("CWE209_Information_Leak_Error__console_readLine_17");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_19()).runTest("CWE209_Information_Leak_Error__console_readLine_19");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_31()).runTest("CWE209_Information_Leak_Error__console_readLine_31");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_41()).runTest("CWE209_Information_Leak_Error__console_readLine_41");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_42()).runTest("CWE209_Information_Leak_Error__console_readLine_42");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_45()).runTest("CWE209_Information_Leak_Error__console_readLine_45");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_51a()).runTest("CWE209_Information_Leak_Error__console_readLine_51a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_52a()).runTest("CWE209_Information_Leak_Error__console_readLine_52a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_53a()).runTest("CWE209_Information_Leak_Error__console_readLine_53a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_54a()).runTest("CWE209_Information_Leak_Error__console_readLine_54a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_61a()).runTest("CWE209_Information_Leak_Error__console_readLine_61a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_66a()).runTest("CWE209_Information_Leak_Error__console_readLine_66a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_67a()).runTest("CWE209_Information_Leak_Error__console_readLine_67a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_68a()).runTest("CWE209_Information_Leak_Error__console_readLine_68a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__console_readLine_71a()).runTest("CWE209_Information_Leak_Error__console_readLine_71a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_01()).runTest("CWE209_Information_Leak_Error__Environment_01");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_02()).runTest("CWE209_Information_Leak_Error__Environment_02");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_03()).runTest("CWE209_Information_Leak_Error__Environment_03");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_04()).runTest("CWE209_Information_Leak_Error__Environment_04");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_05()).runTest("CWE209_Information_Leak_Error__Environment_05");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_06()).runTest("CWE209_Information_Leak_Error__Environment_06");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_07()).runTest("CWE209_Information_Leak_Error__Environment_07");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_08()).runTest("CWE209_Information_Leak_Error__Environment_08");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_09()).runTest("CWE209_Information_Leak_Error__Environment_09");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_10()).runTest("CWE209_Information_Leak_Error__Environment_10");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_11()).runTest("CWE209_Information_Leak_Error__Environment_11");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_12()).runTest("CWE209_Information_Leak_Error__Environment_12");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_13()).runTest("CWE209_Information_Leak_Error__Environment_13");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_14()).runTest("CWE209_Information_Leak_Error__Environment_14");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_15()).runTest("CWE209_Information_Leak_Error__Environment_15");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_16()).runTest("CWE209_Information_Leak_Error__Environment_16");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_17()).runTest("CWE209_Information_Leak_Error__Environment_17");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_19()).runTest("CWE209_Information_Leak_Error__Environment_19");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_31()).runTest("CWE209_Information_Leak_Error__Environment_31");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_41()).runTest("CWE209_Information_Leak_Error__Environment_41");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_42()).runTest("CWE209_Information_Leak_Error__Environment_42");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_45()).runTest("CWE209_Information_Leak_Error__Environment_45");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_51a()).runTest("CWE209_Information_Leak_Error__Environment_51a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_52a()).runTest("CWE209_Information_Leak_Error__Environment_52a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_53a()).runTest("CWE209_Information_Leak_Error__Environment_53a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_54a()).runTest("CWE209_Information_Leak_Error__Environment_54a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_61a()).runTest("CWE209_Information_Leak_Error__Environment_61a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_66a()).runTest("CWE209_Information_Leak_Error__Environment_66a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_67a()).runTest("CWE209_Information_Leak_Error__Environment_67a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_68a()).runTest("CWE209_Information_Leak_Error__Environment_68a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__Environment_71a()).runTest("CWE209_Information_Leak_Error__Environment_71a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_01()).runTest("CWE209_Information_Leak_Error__fromDB_01");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_02()).runTest("CWE209_Information_Leak_Error__fromDB_02");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_03()).runTest("CWE209_Information_Leak_Error__fromDB_03");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_04()).runTest("CWE209_Information_Leak_Error__fromDB_04");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_05()).runTest("CWE209_Information_Leak_Error__fromDB_05");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_06()).runTest("CWE209_Information_Leak_Error__fromDB_06");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_07()).runTest("CWE209_Information_Leak_Error__fromDB_07");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_08()).runTest("CWE209_Information_Leak_Error__fromDB_08");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_09()).runTest("CWE209_Information_Leak_Error__fromDB_09");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_10()).runTest("CWE209_Information_Leak_Error__fromDB_10");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_11()).runTest("CWE209_Information_Leak_Error__fromDB_11");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_12()).runTest("CWE209_Information_Leak_Error__fromDB_12");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_13()).runTest("CWE209_Information_Leak_Error__fromDB_13");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_14()).runTest("CWE209_Information_Leak_Error__fromDB_14");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_15()).runTest("CWE209_Information_Leak_Error__fromDB_15");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_16()).runTest("CWE209_Information_Leak_Error__fromDB_16");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_17()).runTest("CWE209_Information_Leak_Error__fromDB_17");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_19()).runTest("CWE209_Information_Leak_Error__fromDB_19");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_31()).runTest("CWE209_Information_Leak_Error__fromDB_31");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_41()).runTest("CWE209_Information_Leak_Error__fromDB_41");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_42()).runTest("CWE209_Information_Leak_Error__fromDB_42");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_45()).runTest("CWE209_Information_Leak_Error__fromDB_45");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_51a()).runTest("CWE209_Information_Leak_Error__fromDB_51a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_52a()).runTest("CWE209_Information_Leak_Error__fromDB_52a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_53a()).runTest("CWE209_Information_Leak_Error__fromDB_53a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_54a()).runTest("CWE209_Information_Leak_Error__fromDB_54a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_61a()).runTest("CWE209_Information_Leak_Error__fromDB_61a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_66a()).runTest("CWE209_Information_Leak_Error__fromDB_66a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_67a()).runTest("CWE209_Information_Leak_Error__fromDB_67a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_68a()).runTest("CWE209_Information_Leak_Error__fromDB_68a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromDB_71a()).runTest("CWE209_Information_Leak_Error__fromDB_71a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_01()).runTest("CWE209_Information_Leak_Error__fromFile_01");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_02()).runTest("CWE209_Information_Leak_Error__fromFile_02");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_03()).runTest("CWE209_Information_Leak_Error__fromFile_03");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_04()).runTest("CWE209_Information_Leak_Error__fromFile_04");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_05()).runTest("CWE209_Information_Leak_Error__fromFile_05");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_06()).runTest("CWE209_Information_Leak_Error__fromFile_06");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_07()).runTest("CWE209_Information_Leak_Error__fromFile_07");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_08()).runTest("CWE209_Information_Leak_Error__fromFile_08");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_09()).runTest("CWE209_Information_Leak_Error__fromFile_09");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_10()).runTest("CWE209_Information_Leak_Error__fromFile_10");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_11()).runTest("CWE209_Information_Leak_Error__fromFile_11");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_12()).runTest("CWE209_Information_Leak_Error__fromFile_12");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_13()).runTest("CWE209_Information_Leak_Error__fromFile_13");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_14()).runTest("CWE209_Information_Leak_Error__fromFile_14");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_15()).runTest("CWE209_Information_Leak_Error__fromFile_15");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_16()).runTest("CWE209_Information_Leak_Error__fromFile_16");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_17()).runTest("CWE209_Information_Leak_Error__fromFile_17");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_19()).runTest("CWE209_Information_Leak_Error__fromFile_19");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_31()).runTest("CWE209_Information_Leak_Error__fromFile_31");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_41()).runTest("CWE209_Information_Leak_Error__fromFile_41");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_42()).runTest("CWE209_Information_Leak_Error__fromFile_42");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_45()).runTest("CWE209_Information_Leak_Error__fromFile_45");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_51a()).runTest("CWE209_Information_Leak_Error__fromFile_51a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_52a()).runTest("CWE209_Information_Leak_Error__fromFile_52a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_53a()).runTest("CWE209_Information_Leak_Error__fromFile_53a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_54a()).runTest("CWE209_Information_Leak_Error__fromFile_54a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_61a()).runTest("CWE209_Information_Leak_Error__fromFile_61a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_66a()).runTest("CWE209_Information_Leak_Error__fromFile_66a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_67a()).runTest("CWE209_Information_Leak_Error__fromFile_67a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_68a()).runTest("CWE209_Information_Leak_Error__fromFile_68a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__fromFile_71a()).runTest("CWE209_Information_Leak_Error__fromFile_71a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_01()).runTest("CWE209_Information_Leak_Error__listen_tcp_01");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_02()).runTest("CWE209_Information_Leak_Error__listen_tcp_02");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_03()).runTest("CWE209_Information_Leak_Error__listen_tcp_03");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_04()).runTest("CWE209_Information_Leak_Error__listen_tcp_04");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_05()).runTest("CWE209_Information_Leak_Error__listen_tcp_05");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_06()).runTest("CWE209_Information_Leak_Error__listen_tcp_06");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_07()).runTest("CWE209_Information_Leak_Error__listen_tcp_07");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_08()).runTest("CWE209_Information_Leak_Error__listen_tcp_08");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_09()).runTest("CWE209_Information_Leak_Error__listen_tcp_09");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_10()).runTest("CWE209_Information_Leak_Error__listen_tcp_10");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_11()).runTest("CWE209_Information_Leak_Error__listen_tcp_11");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_12()).runTest("CWE209_Information_Leak_Error__listen_tcp_12");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_13()).runTest("CWE209_Information_Leak_Error__listen_tcp_13");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_14()).runTest("CWE209_Information_Leak_Error__listen_tcp_14");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_15()).runTest("CWE209_Information_Leak_Error__listen_tcp_15");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_16()).runTest("CWE209_Information_Leak_Error__listen_tcp_16");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_17()).runTest("CWE209_Information_Leak_Error__listen_tcp_17");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_19()).runTest("CWE209_Information_Leak_Error__listen_tcp_19");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_31()).runTest("CWE209_Information_Leak_Error__listen_tcp_31");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_41()).runTest("CWE209_Information_Leak_Error__listen_tcp_41");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_42()).runTest("CWE209_Information_Leak_Error__listen_tcp_42");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_45()).runTest("CWE209_Information_Leak_Error__listen_tcp_45");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_51a()).runTest("CWE209_Information_Leak_Error__listen_tcp_51a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_52a()).runTest("CWE209_Information_Leak_Error__listen_tcp_52a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_53a()).runTest("CWE209_Information_Leak_Error__listen_tcp_53a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_54a()).runTest("CWE209_Information_Leak_Error__listen_tcp_54a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_61a()).runTest("CWE209_Information_Leak_Error__listen_tcp_61a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_66a()).runTest("CWE209_Information_Leak_Error__listen_tcp_66a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_67a()).runTest("CWE209_Information_Leak_Error__listen_tcp_67a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_68a()).runTest("CWE209_Information_Leak_Error__listen_tcp_68a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__listen_tcp_71a()).runTest("CWE209_Information_Leak_Error__listen_tcp_71a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_01()).runTest("CWE209_Information_Leak_Error__PropertiesFile_01");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_02()).runTest("CWE209_Information_Leak_Error__PropertiesFile_02");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_03()).runTest("CWE209_Information_Leak_Error__PropertiesFile_03");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_04()).runTest("CWE209_Information_Leak_Error__PropertiesFile_04");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_05()).runTest("CWE209_Information_Leak_Error__PropertiesFile_05");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_06()).runTest("CWE209_Information_Leak_Error__PropertiesFile_06");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_07()).runTest("CWE209_Information_Leak_Error__PropertiesFile_07");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_08()).runTest("CWE209_Information_Leak_Error__PropertiesFile_08");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_09()).runTest("CWE209_Information_Leak_Error__PropertiesFile_09");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_10()).runTest("CWE209_Information_Leak_Error__PropertiesFile_10");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_11()).runTest("CWE209_Information_Leak_Error__PropertiesFile_11");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_12()).runTest("CWE209_Information_Leak_Error__PropertiesFile_12");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_13()).runTest("CWE209_Information_Leak_Error__PropertiesFile_13");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_14()).runTest("CWE209_Information_Leak_Error__PropertiesFile_14");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_15()).runTest("CWE209_Information_Leak_Error__PropertiesFile_15");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_16()).runTest("CWE209_Information_Leak_Error__PropertiesFile_16");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_17()).runTest("CWE209_Information_Leak_Error__PropertiesFile_17");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_19()).runTest("CWE209_Information_Leak_Error__PropertiesFile_19");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_31()).runTest("CWE209_Information_Leak_Error__PropertiesFile_31");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_41()).runTest("CWE209_Information_Leak_Error__PropertiesFile_41");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_42()).runTest("CWE209_Information_Leak_Error__PropertiesFile_42");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_45()).runTest("CWE209_Information_Leak_Error__PropertiesFile_45");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_51a()).runTest("CWE209_Information_Leak_Error__PropertiesFile_51a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_52a()).runTest("CWE209_Information_Leak_Error__PropertiesFile_52a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_53a()).runTest("CWE209_Information_Leak_Error__PropertiesFile_53a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_54a()).runTest("CWE209_Information_Leak_Error__PropertiesFile_54a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_61a()).runTest("CWE209_Information_Leak_Error__PropertiesFile_61a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_66a()).runTest("CWE209_Information_Leak_Error__PropertiesFile_66a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_67a()).runTest("CWE209_Information_Leak_Error__PropertiesFile_67a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_68a()).runTest("CWE209_Information_Leak_Error__PropertiesFile_68a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__PropertiesFile_71a()).runTest("CWE209_Information_Leak_Error__PropertiesFile_71a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_01()).runTest("CWE209_Information_Leak_Error__URLConnection_01");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_02()).runTest("CWE209_Information_Leak_Error__URLConnection_02");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_03()).runTest("CWE209_Information_Leak_Error__URLConnection_03");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_04()).runTest("CWE209_Information_Leak_Error__URLConnection_04");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_05()).runTest("CWE209_Information_Leak_Error__URLConnection_05");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_06()).runTest("CWE209_Information_Leak_Error__URLConnection_06");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_07()).runTest("CWE209_Information_Leak_Error__URLConnection_07");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_08()).runTest("CWE209_Information_Leak_Error__URLConnection_08");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_09()).runTest("CWE209_Information_Leak_Error__URLConnection_09");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_10()).runTest("CWE209_Information_Leak_Error__URLConnection_10");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_11()).runTest("CWE209_Information_Leak_Error__URLConnection_11");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_12()).runTest("CWE209_Information_Leak_Error__URLConnection_12");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_13()).runTest("CWE209_Information_Leak_Error__URLConnection_13");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_14()).runTest("CWE209_Information_Leak_Error__URLConnection_14");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_15()).runTest("CWE209_Information_Leak_Error__URLConnection_15");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_16()).runTest("CWE209_Information_Leak_Error__URLConnection_16");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_17()).runTest("CWE209_Information_Leak_Error__URLConnection_17");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_19()).runTest("CWE209_Information_Leak_Error__URLConnection_19");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_31()).runTest("CWE209_Information_Leak_Error__URLConnection_31");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_41()).runTest("CWE209_Information_Leak_Error__URLConnection_41");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_42()).runTest("CWE209_Information_Leak_Error__URLConnection_42");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_45()).runTest("CWE209_Information_Leak_Error__URLConnection_45");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_51a()).runTest("CWE209_Information_Leak_Error__URLConnection_51a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_52a()).runTest("CWE209_Information_Leak_Error__URLConnection_52a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_53a()).runTest("CWE209_Information_Leak_Error__URLConnection_53a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_54a()).runTest("CWE209_Information_Leak_Error__URLConnection_54a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_61a()).runTest("CWE209_Information_Leak_Error__URLConnection_61a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_66a()).runTest("CWE209_Information_Leak_Error__URLConnection_66a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_67a()).runTest("CWE209_Information_Leak_Error__URLConnection_67a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_68a()).runTest("CWE209_Information_Leak_Error__URLConnection_68a");
			(new testcases.CWE209_Information_Leak_Error.CWE209_Information_Leak_Error__URLConnection_71a()).runTest("CWE209_Information_Leak_Error__URLConnection_71a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_01()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_01");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_02()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_02");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_03()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_03");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_04()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_04");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_05()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_05");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_06()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_06");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_07()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_07");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_08()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_08");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_09()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_09");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_10()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_10");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_11()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_11");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_12()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_12");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_13()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_13");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_14()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_14");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_15()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_15");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_16()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_16");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_17()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_17");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_19()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_19");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_31()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_31");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_41()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_41");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_42()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_42");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_45()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_45");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_51a()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_51a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_52a()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_52a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_53a()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_53a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_54a()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_54a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_61a()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_61a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_66a()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_66a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_67a()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_67a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_68a()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_68a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__connect_tcp_71a()).runTest("CWE23_Relative_Path_Traversal__connect_tcp_71a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_01()).runTest("CWE23_Relative_Path_Traversal__console_readLine_01");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_02()).runTest("CWE23_Relative_Path_Traversal__console_readLine_02");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_03()).runTest("CWE23_Relative_Path_Traversal__console_readLine_03");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_04()).runTest("CWE23_Relative_Path_Traversal__console_readLine_04");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_05()).runTest("CWE23_Relative_Path_Traversal__console_readLine_05");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_06()).runTest("CWE23_Relative_Path_Traversal__console_readLine_06");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_07()).runTest("CWE23_Relative_Path_Traversal__console_readLine_07");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_08()).runTest("CWE23_Relative_Path_Traversal__console_readLine_08");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_09()).runTest("CWE23_Relative_Path_Traversal__console_readLine_09");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_10()).runTest("CWE23_Relative_Path_Traversal__console_readLine_10");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_11()).runTest("CWE23_Relative_Path_Traversal__console_readLine_11");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_12()).runTest("CWE23_Relative_Path_Traversal__console_readLine_12");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_13()).runTest("CWE23_Relative_Path_Traversal__console_readLine_13");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_14()).runTest("CWE23_Relative_Path_Traversal__console_readLine_14");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_15()).runTest("CWE23_Relative_Path_Traversal__console_readLine_15");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_16()).runTest("CWE23_Relative_Path_Traversal__console_readLine_16");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_17()).runTest("CWE23_Relative_Path_Traversal__console_readLine_17");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_19()).runTest("CWE23_Relative_Path_Traversal__console_readLine_19");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_31()).runTest("CWE23_Relative_Path_Traversal__console_readLine_31");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_41()).runTest("CWE23_Relative_Path_Traversal__console_readLine_41");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_42()).runTest("CWE23_Relative_Path_Traversal__console_readLine_42");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_45()).runTest("CWE23_Relative_Path_Traversal__console_readLine_45");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_51a()).runTest("CWE23_Relative_Path_Traversal__console_readLine_51a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_52a()).runTest("CWE23_Relative_Path_Traversal__console_readLine_52a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_53a()).runTest("CWE23_Relative_Path_Traversal__console_readLine_53a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_54a()).runTest("CWE23_Relative_Path_Traversal__console_readLine_54a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_61a()).runTest("CWE23_Relative_Path_Traversal__console_readLine_61a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_66a()).runTest("CWE23_Relative_Path_Traversal__console_readLine_66a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_67a()).runTest("CWE23_Relative_Path_Traversal__console_readLine_67a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_68a()).runTest("CWE23_Relative_Path_Traversal__console_readLine_68a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__console_readLine_71a()).runTest("CWE23_Relative_Path_Traversal__console_readLine_71a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_01()).runTest("CWE23_Relative_Path_Traversal__Environment_01");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_02()).runTest("CWE23_Relative_Path_Traversal__Environment_02");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_03()).runTest("CWE23_Relative_Path_Traversal__Environment_03");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_04()).runTest("CWE23_Relative_Path_Traversal__Environment_04");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_05()).runTest("CWE23_Relative_Path_Traversal__Environment_05");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_06()).runTest("CWE23_Relative_Path_Traversal__Environment_06");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_07()).runTest("CWE23_Relative_Path_Traversal__Environment_07");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_08()).runTest("CWE23_Relative_Path_Traversal__Environment_08");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_09()).runTest("CWE23_Relative_Path_Traversal__Environment_09");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_10()).runTest("CWE23_Relative_Path_Traversal__Environment_10");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_11()).runTest("CWE23_Relative_Path_Traversal__Environment_11");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_12()).runTest("CWE23_Relative_Path_Traversal__Environment_12");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_13()).runTest("CWE23_Relative_Path_Traversal__Environment_13");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_14()).runTest("CWE23_Relative_Path_Traversal__Environment_14");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_15()).runTest("CWE23_Relative_Path_Traversal__Environment_15");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_16()).runTest("CWE23_Relative_Path_Traversal__Environment_16");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_17()).runTest("CWE23_Relative_Path_Traversal__Environment_17");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_19()).runTest("CWE23_Relative_Path_Traversal__Environment_19");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_31()).runTest("CWE23_Relative_Path_Traversal__Environment_31");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_41()).runTest("CWE23_Relative_Path_Traversal__Environment_41");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_42()).runTest("CWE23_Relative_Path_Traversal__Environment_42");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_45()).runTest("CWE23_Relative_Path_Traversal__Environment_45");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_51a()).runTest("CWE23_Relative_Path_Traversal__Environment_51a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_52a()).runTest("CWE23_Relative_Path_Traversal__Environment_52a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_53a()).runTest("CWE23_Relative_Path_Traversal__Environment_53a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_54a()).runTest("CWE23_Relative_Path_Traversal__Environment_54a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_61a()).runTest("CWE23_Relative_Path_Traversal__Environment_61a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_66a()).runTest("CWE23_Relative_Path_Traversal__Environment_66a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_67a()).runTest("CWE23_Relative_Path_Traversal__Environment_67a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_68a()).runTest("CWE23_Relative_Path_Traversal__Environment_68a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__Environment_71a()).runTest("CWE23_Relative_Path_Traversal__Environment_71a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_01()).runTest("CWE23_Relative_Path_Traversal__fromDB_01");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_02()).runTest("CWE23_Relative_Path_Traversal__fromDB_02");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_03()).runTest("CWE23_Relative_Path_Traversal__fromDB_03");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_04()).runTest("CWE23_Relative_Path_Traversal__fromDB_04");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_05()).runTest("CWE23_Relative_Path_Traversal__fromDB_05");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_06()).runTest("CWE23_Relative_Path_Traversal__fromDB_06");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_07()).runTest("CWE23_Relative_Path_Traversal__fromDB_07");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_08()).runTest("CWE23_Relative_Path_Traversal__fromDB_08");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_09()).runTest("CWE23_Relative_Path_Traversal__fromDB_09");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_10()).runTest("CWE23_Relative_Path_Traversal__fromDB_10");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_11()).runTest("CWE23_Relative_Path_Traversal__fromDB_11");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_12()).runTest("CWE23_Relative_Path_Traversal__fromDB_12");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_13()).runTest("CWE23_Relative_Path_Traversal__fromDB_13");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_14()).runTest("CWE23_Relative_Path_Traversal__fromDB_14");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_15()).runTest("CWE23_Relative_Path_Traversal__fromDB_15");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_16()).runTest("CWE23_Relative_Path_Traversal__fromDB_16");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_17()).runTest("CWE23_Relative_Path_Traversal__fromDB_17");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_19()).runTest("CWE23_Relative_Path_Traversal__fromDB_19");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_31()).runTest("CWE23_Relative_Path_Traversal__fromDB_31");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_41()).runTest("CWE23_Relative_Path_Traversal__fromDB_41");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_42()).runTest("CWE23_Relative_Path_Traversal__fromDB_42");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_45()).runTest("CWE23_Relative_Path_Traversal__fromDB_45");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_51a()).runTest("CWE23_Relative_Path_Traversal__fromDB_51a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_52a()).runTest("CWE23_Relative_Path_Traversal__fromDB_52a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_53a()).runTest("CWE23_Relative_Path_Traversal__fromDB_53a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_54a()).runTest("CWE23_Relative_Path_Traversal__fromDB_54a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_61a()).runTest("CWE23_Relative_Path_Traversal__fromDB_61a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_66a()).runTest("CWE23_Relative_Path_Traversal__fromDB_66a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_67a()).runTest("CWE23_Relative_Path_Traversal__fromDB_67a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_68a()).runTest("CWE23_Relative_Path_Traversal__fromDB_68a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromDB_71a()).runTest("CWE23_Relative_Path_Traversal__fromDB_71a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_01()).runTest("CWE23_Relative_Path_Traversal__fromFile_01");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_02()).runTest("CWE23_Relative_Path_Traversal__fromFile_02");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_03()).runTest("CWE23_Relative_Path_Traversal__fromFile_03");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_04()).runTest("CWE23_Relative_Path_Traversal__fromFile_04");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_05()).runTest("CWE23_Relative_Path_Traversal__fromFile_05");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_06()).runTest("CWE23_Relative_Path_Traversal__fromFile_06");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_07()).runTest("CWE23_Relative_Path_Traversal__fromFile_07");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_08()).runTest("CWE23_Relative_Path_Traversal__fromFile_08");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_09()).runTest("CWE23_Relative_Path_Traversal__fromFile_09");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_10()).runTest("CWE23_Relative_Path_Traversal__fromFile_10");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_11()).runTest("CWE23_Relative_Path_Traversal__fromFile_11");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_12()).runTest("CWE23_Relative_Path_Traversal__fromFile_12");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_13()).runTest("CWE23_Relative_Path_Traversal__fromFile_13");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_14()).runTest("CWE23_Relative_Path_Traversal__fromFile_14");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_15()).runTest("CWE23_Relative_Path_Traversal__fromFile_15");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_16()).runTest("CWE23_Relative_Path_Traversal__fromFile_16");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_17()).runTest("CWE23_Relative_Path_Traversal__fromFile_17");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_19()).runTest("CWE23_Relative_Path_Traversal__fromFile_19");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_31()).runTest("CWE23_Relative_Path_Traversal__fromFile_31");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_41()).runTest("CWE23_Relative_Path_Traversal__fromFile_41");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_42()).runTest("CWE23_Relative_Path_Traversal__fromFile_42");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_45()).runTest("CWE23_Relative_Path_Traversal__fromFile_45");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_51a()).runTest("CWE23_Relative_Path_Traversal__fromFile_51a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_52a()).runTest("CWE23_Relative_Path_Traversal__fromFile_52a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_53a()).runTest("CWE23_Relative_Path_Traversal__fromFile_53a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_54a()).runTest("CWE23_Relative_Path_Traversal__fromFile_54a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_61a()).runTest("CWE23_Relative_Path_Traversal__fromFile_61a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_66a()).runTest("CWE23_Relative_Path_Traversal__fromFile_66a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_67a()).runTest("CWE23_Relative_Path_Traversal__fromFile_67a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_68a()).runTest("CWE23_Relative_Path_Traversal__fromFile_68a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__fromFile_71a()).runTest("CWE23_Relative_Path_Traversal__fromFile_71a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_01()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_01");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_02()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_02");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_03()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_03");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_04()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_04");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_05()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_05");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_06()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_06");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_07()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_07");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_08()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_08");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_09()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_09");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_10()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_10");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_11()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_11");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_12()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_12");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_13()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_13");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_14()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_14");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_15()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_15");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_16()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_16");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_17()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_17");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_19()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_19");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_31()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_31");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_41()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_41");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_42()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_42");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_45()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_45");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_51a()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_51a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_52a()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_52a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_53a()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_53a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_54a()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_54a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_61a()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_61a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_66a()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_66a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_67a()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_67a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_68a()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_68a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__listen_tcp_71a()).runTest("CWE23_Relative_Path_Traversal__listen_tcp_71a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_01()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_01");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_02()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_02");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_03()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_03");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_04()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_04");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_05()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_05");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_06()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_06");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_07()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_07");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_08()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_08");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_09()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_09");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_10()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_10");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_11()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_11");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_12()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_12");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_13()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_13");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_14()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_14");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_15()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_15");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_16()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_16");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_17()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_17");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_19()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_19");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_31()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_31");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_41()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_41");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_42()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_42");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_45()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_45");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_51a()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_51a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_52a()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_52a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_53a()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_53a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_54a()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_54a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_61a()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_61a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_66a()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_66a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_67a()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_67a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_68a()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_68a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__PropertiesFile_71a()).runTest("CWE23_Relative_Path_Traversal__PropertiesFile_71a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_01()).runTest("CWE23_Relative_Path_Traversal__URLConnection_01");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_02()).runTest("CWE23_Relative_Path_Traversal__URLConnection_02");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_03()).runTest("CWE23_Relative_Path_Traversal__URLConnection_03");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_04()).runTest("CWE23_Relative_Path_Traversal__URLConnection_04");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_05()).runTest("CWE23_Relative_Path_Traversal__URLConnection_05");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_06()).runTest("CWE23_Relative_Path_Traversal__URLConnection_06");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_07()).runTest("CWE23_Relative_Path_Traversal__URLConnection_07");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_08()).runTest("CWE23_Relative_Path_Traversal__URLConnection_08");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_09()).runTest("CWE23_Relative_Path_Traversal__URLConnection_09");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_10()).runTest("CWE23_Relative_Path_Traversal__URLConnection_10");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_11()).runTest("CWE23_Relative_Path_Traversal__URLConnection_11");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_12()).runTest("CWE23_Relative_Path_Traversal__URLConnection_12");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_13()).runTest("CWE23_Relative_Path_Traversal__URLConnection_13");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_14()).runTest("CWE23_Relative_Path_Traversal__URLConnection_14");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_15()).runTest("CWE23_Relative_Path_Traversal__URLConnection_15");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_16()).runTest("CWE23_Relative_Path_Traversal__URLConnection_16");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_17()).runTest("CWE23_Relative_Path_Traversal__URLConnection_17");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_19()).runTest("CWE23_Relative_Path_Traversal__URLConnection_19");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_31()).runTest("CWE23_Relative_Path_Traversal__URLConnection_31");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_41()).runTest("CWE23_Relative_Path_Traversal__URLConnection_41");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_42()).runTest("CWE23_Relative_Path_Traversal__URLConnection_42");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_45()).runTest("CWE23_Relative_Path_Traversal__URLConnection_45");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_51a()).runTest("CWE23_Relative_Path_Traversal__URLConnection_51a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_52a()).runTest("CWE23_Relative_Path_Traversal__URLConnection_52a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_53a()).runTest("CWE23_Relative_Path_Traversal__URLConnection_53a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_54a()).runTest("CWE23_Relative_Path_Traversal__URLConnection_54a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_61a()).runTest("CWE23_Relative_Path_Traversal__URLConnection_61a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_66a()).runTest("CWE23_Relative_Path_Traversal__URLConnection_66a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_67a()).runTest("CWE23_Relative_Path_Traversal__URLConnection_67a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_68a()).runTest("CWE23_Relative_Path_Traversal__URLConnection_68a");
			(new testcases.CWE23_Relative_Path_Traversal.CWE23_Relative_Path_Traversal__URLConnection_71a()).runTest("CWE23_Relative_Path_Traversal__URLConnection_71a");
			(new testcases.CWE248_Uncaught_Exception.CWE248_Uncaught_Exception__Error_01()).runTest("CWE248_Uncaught_Exception__Error_01");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_01()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_01");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_02()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_02");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_03()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_03");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_04()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_04");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_05()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_05");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_06()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_06");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_07()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_07");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_08()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_08");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_09()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_09");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_10()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_10");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_11()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_11");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_12()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_12");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_13()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_13");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_14()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_14");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_15()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_15");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_16()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_16");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_17()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_17");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_19()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_19");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_31()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_31");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_41()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_41");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_42()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_42");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_45()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_45");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_51a()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_51a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_52a()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_52a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_53a()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_53a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_54a()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_54a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_61a()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_61a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_66a()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_66a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_67a()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_67a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_68a()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_68a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__fromFile_71a()).runTest("CWE256_Plaintext_Storage_of_Password__fromFile_71a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_01()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_01");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_02()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_02");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_03()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_03");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_04()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_04");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_05()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_05");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_06()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_06");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_07()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_07");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_08()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_08");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_09()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_09");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_10()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_10");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_11()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_11");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_12()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_12");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_13()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_13");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_14()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_14");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_15()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_15");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_16()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_16");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_17()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_17");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_19()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_19");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_31()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_31");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_41()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_41");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_42()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_42");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_45()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_45");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_51a()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_51a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_52a()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_52a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_53a()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_53a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_54a()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_54a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_61a()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_61a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_66a()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_66a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_67a()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_67a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_68a()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_68a");
			(new testcases.CWE256_Plaintext_Storage_of_Password.CWE256_Plaintext_Storage_of_Password__PropertiesFile_71a()).runTest("CWE256_Plaintext_Storage_of_Password__PropertiesFile_71a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_01()).runTest("CWE259_Hard_Coded_Password__driverManager_01");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_02()).runTest("CWE259_Hard_Coded_Password__driverManager_02");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_03()).runTest("CWE259_Hard_Coded_Password__driverManager_03");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_04()).runTest("CWE259_Hard_Coded_Password__driverManager_04");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_05()).runTest("CWE259_Hard_Coded_Password__driverManager_05");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_06()).runTest("CWE259_Hard_Coded_Password__driverManager_06");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_07()).runTest("CWE259_Hard_Coded_Password__driverManager_07");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_08()).runTest("CWE259_Hard_Coded_Password__driverManager_08");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_09()).runTest("CWE259_Hard_Coded_Password__driverManager_09");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_10()).runTest("CWE259_Hard_Coded_Password__driverManager_10");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_11()).runTest("CWE259_Hard_Coded_Password__driverManager_11");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_12()).runTest("CWE259_Hard_Coded_Password__driverManager_12");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_13()).runTest("CWE259_Hard_Coded_Password__driverManager_13");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_14()).runTest("CWE259_Hard_Coded_Password__driverManager_14");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_15()).runTest("CWE259_Hard_Coded_Password__driverManager_15");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_16()).runTest("CWE259_Hard_Coded_Password__driverManager_16");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_17()).runTest("CWE259_Hard_Coded_Password__driverManager_17");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_19()).runTest("CWE259_Hard_Coded_Password__driverManager_19");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_31()).runTest("CWE259_Hard_Coded_Password__driverManager_31");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_41()).runTest("CWE259_Hard_Coded_Password__driverManager_41");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_42()).runTest("CWE259_Hard_Coded_Password__driverManager_42");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_45()).runTest("CWE259_Hard_Coded_Password__driverManager_45");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_51a()).runTest("CWE259_Hard_Coded_Password__driverManager_51a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_52a()).runTest("CWE259_Hard_Coded_Password__driverManager_52a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_53a()).runTest("CWE259_Hard_Coded_Password__driverManager_53a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_54a()).runTest("CWE259_Hard_Coded_Password__driverManager_54a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_61a()).runTest("CWE259_Hard_Coded_Password__driverManager_61a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_66a()).runTest("CWE259_Hard_Coded_Password__driverManager_66a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_67a()).runTest("CWE259_Hard_Coded_Password__driverManager_67a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_68a()).runTest("CWE259_Hard_Coded_Password__driverManager_68a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__driverManager_71a()).runTest("CWE259_Hard_Coded_Password__driverManager_71a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_01()).runTest("CWE259_Hard_Coded_Password__kerberosKey_01");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_02()).runTest("CWE259_Hard_Coded_Password__kerberosKey_02");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_03()).runTest("CWE259_Hard_Coded_Password__kerberosKey_03");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_04()).runTest("CWE259_Hard_Coded_Password__kerberosKey_04");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_05()).runTest("CWE259_Hard_Coded_Password__kerberosKey_05");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_06()).runTest("CWE259_Hard_Coded_Password__kerberosKey_06");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_07()).runTest("CWE259_Hard_Coded_Password__kerberosKey_07");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_08()).runTest("CWE259_Hard_Coded_Password__kerberosKey_08");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_09()).runTest("CWE259_Hard_Coded_Password__kerberosKey_09");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_10()).runTest("CWE259_Hard_Coded_Password__kerberosKey_10");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_11()).runTest("CWE259_Hard_Coded_Password__kerberosKey_11");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_12()).runTest("CWE259_Hard_Coded_Password__kerberosKey_12");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_13()).runTest("CWE259_Hard_Coded_Password__kerberosKey_13");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_14()).runTest("CWE259_Hard_Coded_Password__kerberosKey_14");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_15()).runTest("CWE259_Hard_Coded_Password__kerberosKey_15");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_16()).runTest("CWE259_Hard_Coded_Password__kerberosKey_16");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_17()).runTest("CWE259_Hard_Coded_Password__kerberosKey_17");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_19()).runTest("CWE259_Hard_Coded_Password__kerberosKey_19");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_31()).runTest("CWE259_Hard_Coded_Password__kerberosKey_31");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_41()).runTest("CWE259_Hard_Coded_Password__kerberosKey_41");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_42()).runTest("CWE259_Hard_Coded_Password__kerberosKey_42");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_45()).runTest("CWE259_Hard_Coded_Password__kerberosKey_45");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_51a()).runTest("CWE259_Hard_Coded_Password__kerberosKey_51a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_52a()).runTest("CWE259_Hard_Coded_Password__kerberosKey_52a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_53a()).runTest("CWE259_Hard_Coded_Password__kerberosKey_53a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_54a()).runTest("CWE259_Hard_Coded_Password__kerberosKey_54a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_61a()).runTest("CWE259_Hard_Coded_Password__kerberosKey_61a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_66a()).runTest("CWE259_Hard_Coded_Password__kerberosKey_66a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_67a()).runTest("CWE259_Hard_Coded_Password__kerberosKey_67a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_68a()).runTest("CWE259_Hard_Coded_Password__kerberosKey_68a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__kerberosKey_71a()).runTest("CWE259_Hard_Coded_Password__kerberosKey_71a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_01()).runTest("CWE259_Hard_Coded_Password__passwordAuth_01");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_02()).runTest("CWE259_Hard_Coded_Password__passwordAuth_02");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_03()).runTest("CWE259_Hard_Coded_Password__passwordAuth_03");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_04()).runTest("CWE259_Hard_Coded_Password__passwordAuth_04");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_05()).runTest("CWE259_Hard_Coded_Password__passwordAuth_05");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_06()).runTest("CWE259_Hard_Coded_Password__passwordAuth_06");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_07()).runTest("CWE259_Hard_Coded_Password__passwordAuth_07");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_08()).runTest("CWE259_Hard_Coded_Password__passwordAuth_08");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_09()).runTest("CWE259_Hard_Coded_Password__passwordAuth_09");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_10()).runTest("CWE259_Hard_Coded_Password__passwordAuth_10");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_11()).runTest("CWE259_Hard_Coded_Password__passwordAuth_11");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_12()).runTest("CWE259_Hard_Coded_Password__passwordAuth_12");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_13()).runTest("CWE259_Hard_Coded_Password__passwordAuth_13");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_14()).runTest("CWE259_Hard_Coded_Password__passwordAuth_14");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_15()).runTest("CWE259_Hard_Coded_Password__passwordAuth_15");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_16()).runTest("CWE259_Hard_Coded_Password__passwordAuth_16");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_17()).runTest("CWE259_Hard_Coded_Password__passwordAuth_17");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_19()).runTest("CWE259_Hard_Coded_Password__passwordAuth_19");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_31()).runTest("CWE259_Hard_Coded_Password__passwordAuth_31");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_41()).runTest("CWE259_Hard_Coded_Password__passwordAuth_41");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_42()).runTest("CWE259_Hard_Coded_Password__passwordAuth_42");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_45()).runTest("CWE259_Hard_Coded_Password__passwordAuth_45");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_51a()).runTest("CWE259_Hard_Coded_Password__passwordAuth_51a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_52a()).runTest("CWE259_Hard_Coded_Password__passwordAuth_52a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_53a()).runTest("CWE259_Hard_Coded_Password__passwordAuth_53a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_54a()).runTest("CWE259_Hard_Coded_Password__passwordAuth_54a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_61a()).runTest("CWE259_Hard_Coded_Password__passwordAuth_61a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_66a()).runTest("CWE259_Hard_Coded_Password__passwordAuth_66a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_67a()).runTest("CWE259_Hard_Coded_Password__passwordAuth_67a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_68a()).runTest("CWE259_Hard_Coded_Password__passwordAuth_68a");
			(new testcases.CWE259_Hard_Coded_Password.CWE259_Hard_Coded_Password__passwordAuth_71a()).runTest("CWE259_Hard_Coded_Password__passwordAuth_71a");
			/* END-AUTOGENERATED-JAVA-TESTS-2 */
}

private static void runTestCWE3() {
/* BEGIN-AUTOGENERATED-JAVA-TESTS-3 */
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_01()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_01");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_02()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_02");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_03()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_03");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_04()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_04");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_05()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_05");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_06()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_06");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_07()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_07");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_08()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_08");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_09()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_09");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_10()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_10");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_11()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_11");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_12()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_12");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_13()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_13");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_14()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_14");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_15()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_15");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_16()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_16");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_17()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_17");
			(new testcases.CWE300_Channel_Accessible_by_NonEndpoint.CWE300_Channel_Accessible_by_NonEndpoint__basic_19()).runTest("CWE300_Channel_Accessible_by_NonEndpoint__basic_19");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_01()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_01");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_02()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_02");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_03()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_03");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_04()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_04");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_05()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_05");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_06()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_06");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_07()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_07");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_08()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_08");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_09()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_09");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_10()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_10");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_11()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_11");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_12()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_12");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_13()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_13");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_14()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_14");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_15()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_15");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_16()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_16");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_17()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_17");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_19()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_19");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_31()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_31");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_41()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_41");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_42()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_42");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_45()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_45");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_51a()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_51a");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_52a()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_52a");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_53a()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_53a");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_54a()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_54a");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_61a()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_61a");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_66a()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_66a");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_67a()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_67a");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_68a()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_68a");
			(new testcases.CWE321_Hard_Coded_Cryptographic_Key.CWE321_Hard_Coded_Cryptographic_Key__basic_71a()).runTest("CWE321_Hard_Coded_Cryptographic_Key__basic_71a");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_01()).runTest("CWE327_Use_Broken_Crypto__basic_01");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_02()).runTest("CWE327_Use_Broken_Crypto__basic_02");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_03()).runTest("CWE327_Use_Broken_Crypto__basic_03");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_04()).runTest("CWE327_Use_Broken_Crypto__basic_04");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_05()).runTest("CWE327_Use_Broken_Crypto__basic_05");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_06()).runTest("CWE327_Use_Broken_Crypto__basic_06");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_07()).runTest("CWE327_Use_Broken_Crypto__basic_07");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_08()).runTest("CWE327_Use_Broken_Crypto__basic_08");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_09()).runTest("CWE327_Use_Broken_Crypto__basic_09");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_10()).runTest("CWE327_Use_Broken_Crypto__basic_10");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_11()).runTest("CWE327_Use_Broken_Crypto__basic_11");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_12()).runTest("CWE327_Use_Broken_Crypto__basic_12");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_13()).runTest("CWE327_Use_Broken_Crypto__basic_13");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_14()).runTest("CWE327_Use_Broken_Crypto__basic_14");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_15()).runTest("CWE327_Use_Broken_Crypto__basic_15");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_16()).runTest("CWE327_Use_Broken_Crypto__basic_16");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_17()).runTest("CWE327_Use_Broken_Crypto__basic_17");
			(new testcases.CWE327_Use_Broken_Crypto.CWE327_Use_Broken_Crypto__basic_19()).runTest("CWE327_Use_Broken_Crypto__basic_19");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_01()).runTest("CWE328_Reversible_One_Way_Hash__basic_01");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_02()).runTest("CWE328_Reversible_One_Way_Hash__basic_02");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_03()).runTest("CWE328_Reversible_One_Way_Hash__basic_03");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_04()).runTest("CWE328_Reversible_One_Way_Hash__basic_04");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_05()).runTest("CWE328_Reversible_One_Way_Hash__basic_05");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_06()).runTest("CWE328_Reversible_One_Way_Hash__basic_06");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_07()).runTest("CWE328_Reversible_One_Way_Hash__basic_07");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_08()).runTest("CWE328_Reversible_One_Way_Hash__basic_08");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_09()).runTest("CWE328_Reversible_One_Way_Hash__basic_09");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_10()).runTest("CWE328_Reversible_One_Way_Hash__basic_10");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_11()).runTest("CWE328_Reversible_One_Way_Hash__basic_11");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_12()).runTest("CWE328_Reversible_One_Way_Hash__basic_12");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_13()).runTest("CWE328_Reversible_One_Way_Hash__basic_13");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_14()).runTest("CWE328_Reversible_One_Way_Hash__basic_14");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_15()).runTest("CWE328_Reversible_One_Way_Hash__basic_15");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_16()).runTest("CWE328_Reversible_One_Way_Hash__basic_16");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_17()).runTest("CWE328_Reversible_One_Way_Hash__basic_17");
			(new testcases.CWE328_Reversible_One_Way_Hash.CWE328_Reversible_One_Way_Hash__basic_19()).runTest("CWE328_Reversible_One_Way_Hash__basic_19");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_01()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_01");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_02()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_02");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_03()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_03");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_04()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_04");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_05()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_05");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_06()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_06");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_07()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_07");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_08()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_08");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_09()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_09");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_10()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_10");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_11()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_11");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_12()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_12");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_13()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_13");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_14()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_14");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_15()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_15");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_16()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_16");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_17()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_17");
			(new testcases.CWE329_Not_Using_Random_IV_with_CBC_Mode.CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_19()).runTest("CWE329_Not_Using_Random_IV_with_CBC_Mode__basic_19");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_01()).runTest("CWE330_Insufficiently_Random_Values__basic_01");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_02()).runTest("CWE330_Insufficiently_Random_Values__basic_02");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_03()).runTest("CWE330_Insufficiently_Random_Values__basic_03");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_04()).runTest("CWE330_Insufficiently_Random_Values__basic_04");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_05()).runTest("CWE330_Insufficiently_Random_Values__basic_05");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_06()).runTest("CWE330_Insufficiently_Random_Values__basic_06");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_07()).runTest("CWE330_Insufficiently_Random_Values__basic_07");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_08()).runTest("CWE330_Insufficiently_Random_Values__basic_08");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_09()).runTest("CWE330_Insufficiently_Random_Values__basic_09");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_10()).runTest("CWE330_Insufficiently_Random_Values__basic_10");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_11()).runTest("CWE330_Insufficiently_Random_Values__basic_11");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_12()).runTest("CWE330_Insufficiently_Random_Values__basic_12");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_13()).runTest("CWE330_Insufficiently_Random_Values__basic_13");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_14()).runTest("CWE330_Insufficiently_Random_Values__basic_14");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_15()).runTest("CWE330_Insufficiently_Random_Values__basic_15");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_16()).runTest("CWE330_Insufficiently_Random_Values__basic_16");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_17()).runTest("CWE330_Insufficiently_Random_Values__basic_17");
			(new testcases.CWE330_Insufficiently_Random_Values.CWE330_Insufficiently_Random_Values__basic_19()).runTest("CWE330_Insufficiently_Random_Values__basic_19");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_01()).runTest("CWE336_Same_Seed_in_PRNG__basic_01");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_02()).runTest("CWE336_Same_Seed_in_PRNG__basic_02");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_03()).runTest("CWE336_Same_Seed_in_PRNG__basic_03");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_04()).runTest("CWE336_Same_Seed_in_PRNG__basic_04");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_05()).runTest("CWE336_Same_Seed_in_PRNG__basic_05");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_06()).runTest("CWE336_Same_Seed_in_PRNG__basic_06");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_07()).runTest("CWE336_Same_Seed_in_PRNG__basic_07");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_08()).runTest("CWE336_Same_Seed_in_PRNG__basic_08");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_09()).runTest("CWE336_Same_Seed_in_PRNG__basic_09");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_10()).runTest("CWE336_Same_Seed_in_PRNG__basic_10");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_11()).runTest("CWE336_Same_Seed_in_PRNG__basic_11");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_12()).runTest("CWE336_Same_Seed_in_PRNG__basic_12");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_13()).runTest("CWE336_Same_Seed_in_PRNG__basic_13");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_14()).runTest("CWE336_Same_Seed_in_PRNG__basic_14");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_15()).runTest("CWE336_Same_Seed_in_PRNG__basic_15");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_16()).runTest("CWE336_Same_Seed_in_PRNG__basic_16");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_17()).runTest("CWE336_Same_Seed_in_PRNG__basic_17");
			(new testcases.CWE336_Same_Seed_in_PRNG.CWE336_Same_Seed_in_PRNG__basic_19()).runTest("CWE336_Same_Seed_in_PRNG__basic_19");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_01()).runTest("CWE338_Weak_PRNG__math_01");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_02()).runTest("CWE338_Weak_PRNG__math_02");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_03()).runTest("CWE338_Weak_PRNG__math_03");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_04()).runTest("CWE338_Weak_PRNG__math_04");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_05()).runTest("CWE338_Weak_PRNG__math_05");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_06()).runTest("CWE338_Weak_PRNG__math_06");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_07()).runTest("CWE338_Weak_PRNG__math_07");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_08()).runTest("CWE338_Weak_PRNG__math_08");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_09()).runTest("CWE338_Weak_PRNG__math_09");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_10()).runTest("CWE338_Weak_PRNG__math_10");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_11()).runTest("CWE338_Weak_PRNG__math_11");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_12()).runTest("CWE338_Weak_PRNG__math_12");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_13()).runTest("CWE338_Weak_PRNG__math_13");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_14()).runTest("CWE338_Weak_PRNG__math_14");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_15()).runTest("CWE338_Weak_PRNG__math_15");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_16()).runTest("CWE338_Weak_PRNG__math_16");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_17()).runTest("CWE338_Weak_PRNG__math_17");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__math_19()).runTest("CWE338_Weak_PRNG__math_19");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_01()).runTest("CWE338_Weak_PRNG__util_01");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_02()).runTest("CWE338_Weak_PRNG__util_02");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_03()).runTest("CWE338_Weak_PRNG__util_03");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_04()).runTest("CWE338_Weak_PRNG__util_04");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_05()).runTest("CWE338_Weak_PRNG__util_05");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_06()).runTest("CWE338_Weak_PRNG__util_06");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_07()).runTest("CWE338_Weak_PRNG__util_07");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_08()).runTest("CWE338_Weak_PRNG__util_08");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_09()).runTest("CWE338_Weak_PRNG__util_09");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_10()).runTest("CWE338_Weak_PRNG__util_10");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_11()).runTest("CWE338_Weak_PRNG__util_11");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_12()).runTest("CWE338_Weak_PRNG__util_12");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_13()).runTest("CWE338_Weak_PRNG__util_13");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_14()).runTest("CWE338_Weak_PRNG__util_14");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_15()).runTest("CWE338_Weak_PRNG__util_15");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_16()).runTest("CWE338_Weak_PRNG__util_16");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_17()).runTest("CWE338_Weak_PRNG__util_17");
			(new testcases.CWE338_Weak_PRNG.CWE338_Weak_PRNG__util_19()).runTest("CWE338_Weak_PRNG__util_19");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_01()).runTest("CWE367_TOC_TOU__basic_01");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_02()).runTest("CWE367_TOC_TOU__basic_02");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_03()).runTest("CWE367_TOC_TOU__basic_03");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_04()).runTest("CWE367_TOC_TOU__basic_04");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_05()).runTest("CWE367_TOC_TOU__basic_05");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_06()).runTest("CWE367_TOC_TOU__basic_06");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_07()).runTest("CWE367_TOC_TOU__basic_07");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_08()).runTest("CWE367_TOC_TOU__basic_08");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_09()).runTest("CWE367_TOC_TOU__basic_09");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_10()).runTest("CWE367_TOC_TOU__basic_10");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_11()).runTest("CWE367_TOC_TOU__basic_11");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_12()).runTest("CWE367_TOC_TOU__basic_12");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_13()).runTest("CWE367_TOC_TOU__basic_13");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_14()).runTest("CWE367_TOC_TOU__basic_14");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_15()).runTest("CWE367_TOC_TOU__basic_15");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_16()).runTest("CWE367_TOC_TOU__basic_16");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_17()).runTest("CWE367_TOC_TOU__basic_17");
			(new testcases.CWE367_TOC_TOU.CWE367_TOC_TOU__basic_19()).runTest("CWE367_TOC_TOU__basic_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_01()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_02()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_03()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_04()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_05()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_06()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_07()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_08()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_09()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_10()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_11()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_12()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_13()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_14()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_15()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_16()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_17()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_19()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_31()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_41()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_42()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_45()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_51a()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_52a()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_53a()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_54a()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_61a()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_66a()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_67a()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_divide_68a()).runTest("CWE369_Divide_By_Zero__connect_tcp_divide_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_01()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_02()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_03()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_04()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_05()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_06()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_07()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_08()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_09()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_10()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_11()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_12()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_13()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_14()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_15()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_16()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_17()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_19()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_31()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_41()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_42()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_45()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_51a()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_52a()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_53a()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_54a()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_61a()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_66a()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_67a()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__connect_tcp_modulo_68a()).runTest("CWE369_Divide_By_Zero__connect_tcp_modulo_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_01()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_02()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_03()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_04()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_05()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_06()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_07()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_08()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_09()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_10()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_11()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_12()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_13()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_14()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_15()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_16()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_17()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_19()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_31()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_41()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_42()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_45()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_51a()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_52a()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_53a()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_54a()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_61a()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_66a()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_67a()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_divide_68a()).runTest("CWE369_Divide_By_Zero__console_readLine_divide_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_01()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_02()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_03()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_04()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_05()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_06()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_07()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_08()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_09()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_10()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_11()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_12()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_13()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_14()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_15()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_16()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_17()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_19()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_31()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_41()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_42()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_45()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_51a()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_52a()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_53a()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_54a()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_61a()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_66a()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_67a()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__console_readLine_modulo_68a()).runTest("CWE369_Divide_By_Zero__console_readLine_modulo_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_01()).runTest("CWE369_Divide_By_Zero__Environment_divide_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_02()).runTest("CWE369_Divide_By_Zero__Environment_divide_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_03()).runTest("CWE369_Divide_By_Zero__Environment_divide_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_04()).runTest("CWE369_Divide_By_Zero__Environment_divide_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_05()).runTest("CWE369_Divide_By_Zero__Environment_divide_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_06()).runTest("CWE369_Divide_By_Zero__Environment_divide_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_07()).runTest("CWE369_Divide_By_Zero__Environment_divide_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_08()).runTest("CWE369_Divide_By_Zero__Environment_divide_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_09()).runTest("CWE369_Divide_By_Zero__Environment_divide_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_10()).runTest("CWE369_Divide_By_Zero__Environment_divide_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_11()).runTest("CWE369_Divide_By_Zero__Environment_divide_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_12()).runTest("CWE369_Divide_By_Zero__Environment_divide_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_13()).runTest("CWE369_Divide_By_Zero__Environment_divide_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_14()).runTest("CWE369_Divide_By_Zero__Environment_divide_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_15()).runTest("CWE369_Divide_By_Zero__Environment_divide_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_16()).runTest("CWE369_Divide_By_Zero__Environment_divide_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_17()).runTest("CWE369_Divide_By_Zero__Environment_divide_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_19()).runTest("CWE369_Divide_By_Zero__Environment_divide_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_31()).runTest("CWE369_Divide_By_Zero__Environment_divide_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_41()).runTest("CWE369_Divide_By_Zero__Environment_divide_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_42()).runTest("CWE369_Divide_By_Zero__Environment_divide_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_45()).runTest("CWE369_Divide_By_Zero__Environment_divide_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_51a()).runTest("CWE369_Divide_By_Zero__Environment_divide_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_52a()).runTest("CWE369_Divide_By_Zero__Environment_divide_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_53a()).runTest("CWE369_Divide_By_Zero__Environment_divide_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_54a()).runTest("CWE369_Divide_By_Zero__Environment_divide_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_61a()).runTest("CWE369_Divide_By_Zero__Environment_divide_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_66a()).runTest("CWE369_Divide_By_Zero__Environment_divide_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_67a()).runTest("CWE369_Divide_By_Zero__Environment_divide_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_divide_68a()).runTest("CWE369_Divide_By_Zero__Environment_divide_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_01()).runTest("CWE369_Divide_By_Zero__Environment_modulo_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_02()).runTest("CWE369_Divide_By_Zero__Environment_modulo_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_03()).runTest("CWE369_Divide_By_Zero__Environment_modulo_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_04()).runTest("CWE369_Divide_By_Zero__Environment_modulo_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_05()).runTest("CWE369_Divide_By_Zero__Environment_modulo_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_06()).runTest("CWE369_Divide_By_Zero__Environment_modulo_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_07()).runTest("CWE369_Divide_By_Zero__Environment_modulo_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_08()).runTest("CWE369_Divide_By_Zero__Environment_modulo_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_09()).runTest("CWE369_Divide_By_Zero__Environment_modulo_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_10()).runTest("CWE369_Divide_By_Zero__Environment_modulo_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_11()).runTest("CWE369_Divide_By_Zero__Environment_modulo_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_12()).runTest("CWE369_Divide_By_Zero__Environment_modulo_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_13()).runTest("CWE369_Divide_By_Zero__Environment_modulo_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_14()).runTest("CWE369_Divide_By_Zero__Environment_modulo_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_15()).runTest("CWE369_Divide_By_Zero__Environment_modulo_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_16()).runTest("CWE369_Divide_By_Zero__Environment_modulo_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_17()).runTest("CWE369_Divide_By_Zero__Environment_modulo_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_19()).runTest("CWE369_Divide_By_Zero__Environment_modulo_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_31()).runTest("CWE369_Divide_By_Zero__Environment_modulo_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_41()).runTest("CWE369_Divide_By_Zero__Environment_modulo_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_42()).runTest("CWE369_Divide_By_Zero__Environment_modulo_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_45()).runTest("CWE369_Divide_By_Zero__Environment_modulo_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_51a()).runTest("CWE369_Divide_By_Zero__Environment_modulo_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_52a()).runTest("CWE369_Divide_By_Zero__Environment_modulo_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_53a()).runTest("CWE369_Divide_By_Zero__Environment_modulo_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_54a()).runTest("CWE369_Divide_By_Zero__Environment_modulo_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_61a()).runTest("CWE369_Divide_By_Zero__Environment_modulo_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_66a()).runTest("CWE369_Divide_By_Zero__Environment_modulo_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_67a()).runTest("CWE369_Divide_By_Zero__Environment_modulo_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__Environment_modulo_68a()).runTest("CWE369_Divide_By_Zero__Environment_modulo_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_01()).runTest("CWE369_Divide_By_Zero__fromDB_divide_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_02()).runTest("CWE369_Divide_By_Zero__fromDB_divide_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_03()).runTest("CWE369_Divide_By_Zero__fromDB_divide_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_04()).runTest("CWE369_Divide_By_Zero__fromDB_divide_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_05()).runTest("CWE369_Divide_By_Zero__fromDB_divide_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_06()).runTest("CWE369_Divide_By_Zero__fromDB_divide_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_07()).runTest("CWE369_Divide_By_Zero__fromDB_divide_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_08()).runTest("CWE369_Divide_By_Zero__fromDB_divide_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_09()).runTest("CWE369_Divide_By_Zero__fromDB_divide_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_10()).runTest("CWE369_Divide_By_Zero__fromDB_divide_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_11()).runTest("CWE369_Divide_By_Zero__fromDB_divide_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_12()).runTest("CWE369_Divide_By_Zero__fromDB_divide_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_13()).runTest("CWE369_Divide_By_Zero__fromDB_divide_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_14()).runTest("CWE369_Divide_By_Zero__fromDB_divide_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_15()).runTest("CWE369_Divide_By_Zero__fromDB_divide_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_16()).runTest("CWE369_Divide_By_Zero__fromDB_divide_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_17()).runTest("CWE369_Divide_By_Zero__fromDB_divide_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_19()).runTest("CWE369_Divide_By_Zero__fromDB_divide_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_31()).runTest("CWE369_Divide_By_Zero__fromDB_divide_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_41()).runTest("CWE369_Divide_By_Zero__fromDB_divide_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_42()).runTest("CWE369_Divide_By_Zero__fromDB_divide_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_45()).runTest("CWE369_Divide_By_Zero__fromDB_divide_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_51a()).runTest("CWE369_Divide_By_Zero__fromDB_divide_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_52a()).runTest("CWE369_Divide_By_Zero__fromDB_divide_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_53a()).runTest("CWE369_Divide_By_Zero__fromDB_divide_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_54a()).runTest("CWE369_Divide_By_Zero__fromDB_divide_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_61a()).runTest("CWE369_Divide_By_Zero__fromDB_divide_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_66a()).runTest("CWE369_Divide_By_Zero__fromDB_divide_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_67a()).runTest("CWE369_Divide_By_Zero__fromDB_divide_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_divide_68a()).runTest("CWE369_Divide_By_Zero__fromDB_divide_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_01()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_02()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_03()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_04()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_05()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_06()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_07()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_08()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_09()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_10()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_11()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_12()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_13()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_14()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_15()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_16()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_17()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_19()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_31()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_41()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_42()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_45()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_51a()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_52a()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_53a()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_54a()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_61a()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_66a()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_67a()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromDB_modulo_68a()).runTest("CWE369_Divide_By_Zero__fromDB_modulo_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_01()).runTest("CWE369_Divide_By_Zero__fromFile_divide_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_02()).runTest("CWE369_Divide_By_Zero__fromFile_divide_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_03()).runTest("CWE369_Divide_By_Zero__fromFile_divide_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_04()).runTest("CWE369_Divide_By_Zero__fromFile_divide_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_05()).runTest("CWE369_Divide_By_Zero__fromFile_divide_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_06()).runTest("CWE369_Divide_By_Zero__fromFile_divide_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_07()).runTest("CWE369_Divide_By_Zero__fromFile_divide_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_08()).runTest("CWE369_Divide_By_Zero__fromFile_divide_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_09()).runTest("CWE369_Divide_By_Zero__fromFile_divide_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_10()).runTest("CWE369_Divide_By_Zero__fromFile_divide_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_11()).runTest("CWE369_Divide_By_Zero__fromFile_divide_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_12()).runTest("CWE369_Divide_By_Zero__fromFile_divide_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_13()).runTest("CWE369_Divide_By_Zero__fromFile_divide_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_14()).runTest("CWE369_Divide_By_Zero__fromFile_divide_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_15()).runTest("CWE369_Divide_By_Zero__fromFile_divide_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_16()).runTest("CWE369_Divide_By_Zero__fromFile_divide_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_17()).runTest("CWE369_Divide_By_Zero__fromFile_divide_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_19()).runTest("CWE369_Divide_By_Zero__fromFile_divide_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_31()).runTest("CWE369_Divide_By_Zero__fromFile_divide_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_41()).runTest("CWE369_Divide_By_Zero__fromFile_divide_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_42()).runTest("CWE369_Divide_By_Zero__fromFile_divide_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_45()).runTest("CWE369_Divide_By_Zero__fromFile_divide_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_51a()).runTest("CWE369_Divide_By_Zero__fromFile_divide_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_52a()).runTest("CWE369_Divide_By_Zero__fromFile_divide_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_53a()).runTest("CWE369_Divide_By_Zero__fromFile_divide_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_54a()).runTest("CWE369_Divide_By_Zero__fromFile_divide_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_61a()).runTest("CWE369_Divide_By_Zero__fromFile_divide_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_66a()).runTest("CWE369_Divide_By_Zero__fromFile_divide_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_67a()).runTest("CWE369_Divide_By_Zero__fromFile_divide_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_divide_68a()).runTest("CWE369_Divide_By_Zero__fromFile_divide_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_01()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_02()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_03()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_04()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_05()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_06()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_07()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_08()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_09()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_10()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_11()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_12()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_13()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_14()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_15()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_16()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_17()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_19()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_31()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_41()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_42()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_45()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_51a()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_52a()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_53a()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_54a()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_61a()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_66a()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_67a()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__fromFile_modulo_68a()).runTest("CWE369_Divide_By_Zero__fromFile_modulo_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_01()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_02()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_03()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_04()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_05()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_06()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_07()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_08()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_09()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_10()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_11()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_12()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_13()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_14()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_15()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_16()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_17()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_19()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_31()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_41()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_42()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_45()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_51a()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_52a()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_53a()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_54a()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_61a()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_66a()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_67a()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_divide_68a()).runTest("CWE369_Divide_By_Zero__listen_tcp_divide_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_01()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_02()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_03()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_04()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_05()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_06()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_07()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_08()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_09()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_10()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_11()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_12()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_13()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_14()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_15()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_16()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_17()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_19()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_31()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_41()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_42()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_45()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_51a()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_52a()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_53a()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_54a()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_61a()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_66a()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_67a()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__listen_tcp_modulo_68a()).runTest("CWE369_Divide_By_Zero__listen_tcp_modulo_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_01()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_02()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_03()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_04()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_05()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_06()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_07()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_08()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_09()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_10()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_11()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_12()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_13()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_14()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_15()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_16()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_17()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_19()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_31()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_41()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_42()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_45()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_51a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_52a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_53a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_54a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_61a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_66a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_67a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_divide_68a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_divide_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_01()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_02()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_03()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_04()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_05()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_06()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_07()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_08()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_09()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_10()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_11()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_12()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_13()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_14()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_15()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_16()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_17()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_19()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_31()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_41()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_42()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_45()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_51a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_52a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_53a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_54a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_61a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_66a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_67a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__PropertiesFile_modulo_68a()).runTest("CWE369_Divide_By_Zero__PropertiesFile_modulo_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_01()).runTest("CWE369_Divide_By_Zero__random_divide_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_02()).runTest("CWE369_Divide_By_Zero__random_divide_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_03()).runTest("CWE369_Divide_By_Zero__random_divide_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_04()).runTest("CWE369_Divide_By_Zero__random_divide_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_05()).runTest("CWE369_Divide_By_Zero__random_divide_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_06()).runTest("CWE369_Divide_By_Zero__random_divide_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_07()).runTest("CWE369_Divide_By_Zero__random_divide_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_08()).runTest("CWE369_Divide_By_Zero__random_divide_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_09()).runTest("CWE369_Divide_By_Zero__random_divide_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_10()).runTest("CWE369_Divide_By_Zero__random_divide_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_11()).runTest("CWE369_Divide_By_Zero__random_divide_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_12()).runTest("CWE369_Divide_By_Zero__random_divide_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_13()).runTest("CWE369_Divide_By_Zero__random_divide_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_14()).runTest("CWE369_Divide_By_Zero__random_divide_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_15()).runTest("CWE369_Divide_By_Zero__random_divide_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_16()).runTest("CWE369_Divide_By_Zero__random_divide_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_17()).runTest("CWE369_Divide_By_Zero__random_divide_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_19()).runTest("CWE369_Divide_By_Zero__random_divide_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_31()).runTest("CWE369_Divide_By_Zero__random_divide_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_41()).runTest("CWE369_Divide_By_Zero__random_divide_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_42()).runTest("CWE369_Divide_By_Zero__random_divide_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_45()).runTest("CWE369_Divide_By_Zero__random_divide_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_51a()).runTest("CWE369_Divide_By_Zero__random_divide_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_52a()).runTest("CWE369_Divide_By_Zero__random_divide_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_53a()).runTest("CWE369_Divide_By_Zero__random_divide_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_54a()).runTest("CWE369_Divide_By_Zero__random_divide_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_61a()).runTest("CWE369_Divide_By_Zero__random_divide_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_66a()).runTest("CWE369_Divide_By_Zero__random_divide_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_67a()).runTest("CWE369_Divide_By_Zero__random_divide_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_divide_68a()).runTest("CWE369_Divide_By_Zero__random_divide_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_01()).runTest("CWE369_Divide_By_Zero__random_modulo_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_02()).runTest("CWE369_Divide_By_Zero__random_modulo_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_03()).runTest("CWE369_Divide_By_Zero__random_modulo_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_04()).runTest("CWE369_Divide_By_Zero__random_modulo_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_05()).runTest("CWE369_Divide_By_Zero__random_modulo_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_06()).runTest("CWE369_Divide_By_Zero__random_modulo_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_07()).runTest("CWE369_Divide_By_Zero__random_modulo_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_08()).runTest("CWE369_Divide_By_Zero__random_modulo_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_09()).runTest("CWE369_Divide_By_Zero__random_modulo_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_10()).runTest("CWE369_Divide_By_Zero__random_modulo_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_11()).runTest("CWE369_Divide_By_Zero__random_modulo_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_12()).runTest("CWE369_Divide_By_Zero__random_modulo_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_13()).runTest("CWE369_Divide_By_Zero__random_modulo_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_14()).runTest("CWE369_Divide_By_Zero__random_modulo_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_15()).runTest("CWE369_Divide_By_Zero__random_modulo_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_16()).runTest("CWE369_Divide_By_Zero__random_modulo_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_17()).runTest("CWE369_Divide_By_Zero__random_modulo_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_19()).runTest("CWE369_Divide_By_Zero__random_modulo_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_31()).runTest("CWE369_Divide_By_Zero__random_modulo_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_41()).runTest("CWE369_Divide_By_Zero__random_modulo_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_42()).runTest("CWE369_Divide_By_Zero__random_modulo_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_45()).runTest("CWE369_Divide_By_Zero__random_modulo_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_51a()).runTest("CWE369_Divide_By_Zero__random_modulo_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_52a()).runTest("CWE369_Divide_By_Zero__random_modulo_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_53a()).runTest("CWE369_Divide_By_Zero__random_modulo_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_54a()).runTest("CWE369_Divide_By_Zero__random_modulo_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_61a()).runTest("CWE369_Divide_By_Zero__random_modulo_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_66a()).runTest("CWE369_Divide_By_Zero__random_modulo_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_67a()).runTest("CWE369_Divide_By_Zero__random_modulo_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__random_modulo_68a()).runTest("CWE369_Divide_By_Zero__random_modulo_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_01()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_02()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_03()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_04()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_05()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_06()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_07()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_08()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_09()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_10()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_11()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_12()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_13()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_14()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_15()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_16()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_17()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_19()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_31()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_41()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_42()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_45()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_51a()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_52a()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_53a()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_54a()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_61a()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_66a()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_67a()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_divide_68a()).runTest("CWE369_Divide_By_Zero__URLConnection_divide_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_01()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_02()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_03()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_04()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_05()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_06()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_07()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_08()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_09()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_10()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_11()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_12()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_13()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_14()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_15()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_16()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_17()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_19()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_31()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_41()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_42()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_45()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_51a()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_52a()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_53a()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_54a()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_61a()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_66a()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_67a()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__URLConnection_modulo_68a()).runTest("CWE369_Divide_By_Zero__URLConnection_modulo_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_01()).runTest("CWE369_Divide_By_Zero__zero_divide_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_02()).runTest("CWE369_Divide_By_Zero__zero_divide_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_03()).runTest("CWE369_Divide_By_Zero__zero_divide_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_04()).runTest("CWE369_Divide_By_Zero__zero_divide_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_05()).runTest("CWE369_Divide_By_Zero__zero_divide_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_06()).runTest("CWE369_Divide_By_Zero__zero_divide_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_07()).runTest("CWE369_Divide_By_Zero__zero_divide_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_08()).runTest("CWE369_Divide_By_Zero__zero_divide_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_09()).runTest("CWE369_Divide_By_Zero__zero_divide_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_10()).runTest("CWE369_Divide_By_Zero__zero_divide_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_11()).runTest("CWE369_Divide_By_Zero__zero_divide_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_12()).runTest("CWE369_Divide_By_Zero__zero_divide_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_13()).runTest("CWE369_Divide_By_Zero__zero_divide_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_14()).runTest("CWE369_Divide_By_Zero__zero_divide_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_15()).runTest("CWE369_Divide_By_Zero__zero_divide_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_16()).runTest("CWE369_Divide_By_Zero__zero_divide_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_17()).runTest("CWE369_Divide_By_Zero__zero_divide_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_19()).runTest("CWE369_Divide_By_Zero__zero_divide_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_31()).runTest("CWE369_Divide_By_Zero__zero_divide_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_41()).runTest("CWE369_Divide_By_Zero__zero_divide_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_42()).runTest("CWE369_Divide_By_Zero__zero_divide_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_45()).runTest("CWE369_Divide_By_Zero__zero_divide_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_51a()).runTest("CWE369_Divide_By_Zero__zero_divide_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_52a()).runTest("CWE369_Divide_By_Zero__zero_divide_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_53a()).runTest("CWE369_Divide_By_Zero__zero_divide_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_54a()).runTest("CWE369_Divide_By_Zero__zero_divide_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_61a()).runTest("CWE369_Divide_By_Zero__zero_divide_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_66a()).runTest("CWE369_Divide_By_Zero__zero_divide_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_67a()).runTest("CWE369_Divide_By_Zero__zero_divide_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_divide_68a()).runTest("CWE369_Divide_By_Zero__zero_divide_68a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_01()).runTest("CWE369_Divide_By_Zero__zero_modulo_01");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_02()).runTest("CWE369_Divide_By_Zero__zero_modulo_02");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_03()).runTest("CWE369_Divide_By_Zero__zero_modulo_03");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_04()).runTest("CWE369_Divide_By_Zero__zero_modulo_04");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_05()).runTest("CWE369_Divide_By_Zero__zero_modulo_05");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_06()).runTest("CWE369_Divide_By_Zero__zero_modulo_06");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_07()).runTest("CWE369_Divide_By_Zero__zero_modulo_07");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_08()).runTest("CWE369_Divide_By_Zero__zero_modulo_08");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_09()).runTest("CWE369_Divide_By_Zero__zero_modulo_09");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_10()).runTest("CWE369_Divide_By_Zero__zero_modulo_10");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_11()).runTest("CWE369_Divide_By_Zero__zero_modulo_11");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_12()).runTest("CWE369_Divide_By_Zero__zero_modulo_12");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_13()).runTest("CWE369_Divide_By_Zero__zero_modulo_13");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_14()).runTest("CWE369_Divide_By_Zero__zero_modulo_14");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_15()).runTest("CWE369_Divide_By_Zero__zero_modulo_15");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_16()).runTest("CWE369_Divide_By_Zero__zero_modulo_16");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_17()).runTest("CWE369_Divide_By_Zero__zero_modulo_17");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_19()).runTest("CWE369_Divide_By_Zero__zero_modulo_19");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_31()).runTest("CWE369_Divide_By_Zero__zero_modulo_31");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_41()).runTest("CWE369_Divide_By_Zero__zero_modulo_41");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_42()).runTest("CWE369_Divide_By_Zero__zero_modulo_42");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_45()).runTest("CWE369_Divide_By_Zero__zero_modulo_45");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_51a()).runTest("CWE369_Divide_By_Zero__zero_modulo_51a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_52a()).runTest("CWE369_Divide_By_Zero__zero_modulo_52a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_53a()).runTest("CWE369_Divide_By_Zero__zero_modulo_53a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_54a()).runTest("CWE369_Divide_By_Zero__zero_modulo_54a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_61a()).runTest("CWE369_Divide_By_Zero__zero_modulo_61a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_66a()).runTest("CWE369_Divide_By_Zero__zero_modulo_66a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_67a()).runTest("CWE369_Divide_By_Zero__zero_modulo_67a");
			(new testcases.CWE369_Divide_By_Zero.CWE369_Divide_By_Zero__zero_modulo_68a()).runTest("CWE369_Divide_By_Zero__zero_modulo_68a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_01()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_01");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_02()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_02");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_03()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_03");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_04()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_04");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_05()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_05");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_06()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_06");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_07()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_07");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_08()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_08");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_09()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_09");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_10()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_10");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_11()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_11");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_12()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_12");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_13()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_13");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_14()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_14");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_15()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_15");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_16()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_16");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_17()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_17");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_19()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_19");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_31()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_31");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_41()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_41");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_42()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_42");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_45()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_45");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_51a()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_51a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_52a()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_52a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_53a()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_53a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_54a()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_54a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_61a()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_61a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_66a()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_66a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_67a()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_67a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_68a()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_68a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__connect_tcp_71a()).runTest("CWE36_Absolute_Path_Traversal__connect_tcp_71a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_01()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_01");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_02()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_02");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_03()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_03");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_04()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_04");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_05()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_05");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_06()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_06");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_07()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_07");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_08()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_08");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_09()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_09");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_10()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_10");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_11()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_11");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_12()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_12");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_13()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_13");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_14()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_14");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_15()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_15");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_16()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_16");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_17()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_17");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_19()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_19");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_31()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_31");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_41()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_41");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_42()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_42");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_45()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_45");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_51a()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_51a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_52a()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_52a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_53a()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_53a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_54a()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_54a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_61a()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_61a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_66a()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_66a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_67a()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_67a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_68a()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_68a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__console_readLine_71a()).runTest("CWE36_Absolute_Path_Traversal__console_readLine_71a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_01()).runTest("CWE36_Absolute_Path_Traversal__Environment_01");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_02()).runTest("CWE36_Absolute_Path_Traversal__Environment_02");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_03()).runTest("CWE36_Absolute_Path_Traversal__Environment_03");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_04()).runTest("CWE36_Absolute_Path_Traversal__Environment_04");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_05()).runTest("CWE36_Absolute_Path_Traversal__Environment_05");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_06()).runTest("CWE36_Absolute_Path_Traversal__Environment_06");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_07()).runTest("CWE36_Absolute_Path_Traversal__Environment_07");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_08()).runTest("CWE36_Absolute_Path_Traversal__Environment_08");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_09()).runTest("CWE36_Absolute_Path_Traversal__Environment_09");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_10()).runTest("CWE36_Absolute_Path_Traversal__Environment_10");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_11()).runTest("CWE36_Absolute_Path_Traversal__Environment_11");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_12()).runTest("CWE36_Absolute_Path_Traversal__Environment_12");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_13()).runTest("CWE36_Absolute_Path_Traversal__Environment_13");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_14()).runTest("CWE36_Absolute_Path_Traversal__Environment_14");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_15()).runTest("CWE36_Absolute_Path_Traversal__Environment_15");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_16()).runTest("CWE36_Absolute_Path_Traversal__Environment_16");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_17()).runTest("CWE36_Absolute_Path_Traversal__Environment_17");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_19()).runTest("CWE36_Absolute_Path_Traversal__Environment_19");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_31()).runTest("CWE36_Absolute_Path_Traversal__Environment_31");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_41()).runTest("CWE36_Absolute_Path_Traversal__Environment_41");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_42()).runTest("CWE36_Absolute_Path_Traversal__Environment_42");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_45()).runTest("CWE36_Absolute_Path_Traversal__Environment_45");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_51a()).runTest("CWE36_Absolute_Path_Traversal__Environment_51a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_52a()).runTest("CWE36_Absolute_Path_Traversal__Environment_52a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_53a()).runTest("CWE36_Absolute_Path_Traversal__Environment_53a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_54a()).runTest("CWE36_Absolute_Path_Traversal__Environment_54a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_61a()).runTest("CWE36_Absolute_Path_Traversal__Environment_61a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_66a()).runTest("CWE36_Absolute_Path_Traversal__Environment_66a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_67a()).runTest("CWE36_Absolute_Path_Traversal__Environment_67a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_68a()).runTest("CWE36_Absolute_Path_Traversal__Environment_68a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__Environment_71a()).runTest("CWE36_Absolute_Path_Traversal__Environment_71a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_01()).runTest("CWE36_Absolute_Path_Traversal__fromDB_01");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_02()).runTest("CWE36_Absolute_Path_Traversal__fromDB_02");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_03()).runTest("CWE36_Absolute_Path_Traversal__fromDB_03");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_04()).runTest("CWE36_Absolute_Path_Traversal__fromDB_04");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_05()).runTest("CWE36_Absolute_Path_Traversal__fromDB_05");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_06()).runTest("CWE36_Absolute_Path_Traversal__fromDB_06");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_07()).runTest("CWE36_Absolute_Path_Traversal__fromDB_07");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_08()).runTest("CWE36_Absolute_Path_Traversal__fromDB_08");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_09()).runTest("CWE36_Absolute_Path_Traversal__fromDB_09");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_10()).runTest("CWE36_Absolute_Path_Traversal__fromDB_10");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_11()).runTest("CWE36_Absolute_Path_Traversal__fromDB_11");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_12()).runTest("CWE36_Absolute_Path_Traversal__fromDB_12");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_13()).runTest("CWE36_Absolute_Path_Traversal__fromDB_13");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_14()).runTest("CWE36_Absolute_Path_Traversal__fromDB_14");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_15()).runTest("CWE36_Absolute_Path_Traversal__fromDB_15");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_16()).runTest("CWE36_Absolute_Path_Traversal__fromDB_16");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_17()).runTest("CWE36_Absolute_Path_Traversal__fromDB_17");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_19()).runTest("CWE36_Absolute_Path_Traversal__fromDB_19");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_31()).runTest("CWE36_Absolute_Path_Traversal__fromDB_31");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_41()).runTest("CWE36_Absolute_Path_Traversal__fromDB_41");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_42()).runTest("CWE36_Absolute_Path_Traversal__fromDB_42");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_45()).runTest("CWE36_Absolute_Path_Traversal__fromDB_45");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_51a()).runTest("CWE36_Absolute_Path_Traversal__fromDB_51a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_52a()).runTest("CWE36_Absolute_Path_Traversal__fromDB_52a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_53a()).runTest("CWE36_Absolute_Path_Traversal__fromDB_53a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_54a()).runTest("CWE36_Absolute_Path_Traversal__fromDB_54a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_61a()).runTest("CWE36_Absolute_Path_Traversal__fromDB_61a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_66a()).runTest("CWE36_Absolute_Path_Traversal__fromDB_66a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_67a()).runTest("CWE36_Absolute_Path_Traversal__fromDB_67a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_68a()).runTest("CWE36_Absolute_Path_Traversal__fromDB_68a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromDB_71a()).runTest("CWE36_Absolute_Path_Traversal__fromDB_71a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_01()).runTest("CWE36_Absolute_Path_Traversal__fromFile_01");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_02()).runTest("CWE36_Absolute_Path_Traversal__fromFile_02");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_03()).runTest("CWE36_Absolute_Path_Traversal__fromFile_03");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_04()).runTest("CWE36_Absolute_Path_Traversal__fromFile_04");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_05()).runTest("CWE36_Absolute_Path_Traversal__fromFile_05");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_06()).runTest("CWE36_Absolute_Path_Traversal__fromFile_06");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_07()).runTest("CWE36_Absolute_Path_Traversal__fromFile_07");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_08()).runTest("CWE36_Absolute_Path_Traversal__fromFile_08");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_09()).runTest("CWE36_Absolute_Path_Traversal__fromFile_09");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_10()).runTest("CWE36_Absolute_Path_Traversal__fromFile_10");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_11()).runTest("CWE36_Absolute_Path_Traversal__fromFile_11");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_12()).runTest("CWE36_Absolute_Path_Traversal__fromFile_12");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_13()).runTest("CWE36_Absolute_Path_Traversal__fromFile_13");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_14()).runTest("CWE36_Absolute_Path_Traversal__fromFile_14");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_15()).runTest("CWE36_Absolute_Path_Traversal__fromFile_15");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_16()).runTest("CWE36_Absolute_Path_Traversal__fromFile_16");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_17()).runTest("CWE36_Absolute_Path_Traversal__fromFile_17");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_19()).runTest("CWE36_Absolute_Path_Traversal__fromFile_19");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_31()).runTest("CWE36_Absolute_Path_Traversal__fromFile_31");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_41()).runTest("CWE36_Absolute_Path_Traversal__fromFile_41");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_42()).runTest("CWE36_Absolute_Path_Traversal__fromFile_42");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_45()).runTest("CWE36_Absolute_Path_Traversal__fromFile_45");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_51a()).runTest("CWE36_Absolute_Path_Traversal__fromFile_51a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_52a()).runTest("CWE36_Absolute_Path_Traversal__fromFile_52a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_53a()).runTest("CWE36_Absolute_Path_Traversal__fromFile_53a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_54a()).runTest("CWE36_Absolute_Path_Traversal__fromFile_54a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_61a()).runTest("CWE36_Absolute_Path_Traversal__fromFile_61a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_66a()).runTest("CWE36_Absolute_Path_Traversal__fromFile_66a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_67a()).runTest("CWE36_Absolute_Path_Traversal__fromFile_67a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_68a()).runTest("CWE36_Absolute_Path_Traversal__fromFile_68a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__fromFile_71a()).runTest("CWE36_Absolute_Path_Traversal__fromFile_71a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_01()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_01");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_02()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_02");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_03()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_03");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_04()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_04");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_05()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_05");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_06()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_06");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_07()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_07");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_08()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_08");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_09()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_09");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_10()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_10");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_11()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_11");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_12()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_12");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_13()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_13");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_14()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_14");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_15()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_15");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_16()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_16");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_17()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_17");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_19()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_19");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_31()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_31");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_41()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_41");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_42()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_42");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_45()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_45");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_51a()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_51a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_52a()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_52a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_53a()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_53a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_54a()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_54a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_61a()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_61a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_66a()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_66a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_67a()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_67a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_68a()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_68a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__listen_tcp_71a()).runTest("CWE36_Absolute_Path_Traversal__listen_tcp_71a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_01()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_01");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_02()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_02");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_03()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_03");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_04()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_04");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_05()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_05");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_06()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_06");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_07()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_07");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_08()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_08");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_09()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_09");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_10()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_10");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_11()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_11");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_12()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_12");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_13()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_13");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_14()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_14");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_15()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_15");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_16()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_16");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_17()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_17");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_19()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_19");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_31()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_31");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_41()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_41");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_42()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_42");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_45()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_45");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_51a()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_51a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_52a()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_52a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_53a()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_53a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_54a()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_54a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_61a()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_61a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_66a()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_66a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_67a()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_67a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_68a()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_68a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__PropertiesFile_71a()).runTest("CWE36_Absolute_Path_Traversal__PropertiesFile_71a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_01()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_01");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_02()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_02");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_03()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_03");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_04()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_04");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_05()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_05");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_06()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_06");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_07()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_07");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_08()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_08");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_09()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_09");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_10()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_10");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_11()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_11");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_12()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_12");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_13()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_13");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_14()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_14");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_15()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_15");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_16()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_16");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_17()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_17");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_19()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_19");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_31()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_31");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_41()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_41");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_42()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_42");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_45()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_45");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_51a()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_51a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_52a()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_52a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_53a()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_53a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_54a()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_54a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_61a()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_61a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_66a()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_66a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_67a()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_67a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_68a()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_68a");
			(new testcases.CWE36_Absolute_Path_Traversal.CWE36_Absolute_Path_Traversal__URLConnection_71a()).runTest("CWE36_Absolute_Path_Traversal__URLConnection_71a");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_01()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_01");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_02()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_02");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_03()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_03");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_04()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_04");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_05()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_05");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_06()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_06");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_07()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_07");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_08()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_08");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_09()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_09");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_10()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_10");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_11()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_11");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_12()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_12");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_13()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_13");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_14()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_14");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_15()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_15");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_16()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_16");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_17()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_17");
			(new testcases.CWE378_Creation_of_File_with_Insecure_Perms.CWE378_Creation_of_File_with_Insecure_Perms__basic_19()).runTest("CWE378_Creation_of_File_with_Insecure_Perms__basic_19");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_01()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_01");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_02()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_02");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_03()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_03");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_04()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_04");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_05()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_05");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_06()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_06");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_07()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_07");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_08()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_08");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_09()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_09");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_10()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_10");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_11()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_11");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_12()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_12");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_13()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_13");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_14()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_14");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_15()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_15");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_16()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_16");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_17()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_17");
			(new testcases.CWE379_File_Creation_in_Insecure_Dir.CWE379_File_Creation_in_Insecure_Dir__basic_19()).runTest("CWE379_File_Creation_in_Insecure_Dir__basic_19");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_01()).runTest("CWE390_Error_Without_Action__mkdirs_01");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_02()).runTest("CWE390_Error_Without_Action__mkdirs_02");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_03()).runTest("CWE390_Error_Without_Action__mkdirs_03");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_04()).runTest("CWE390_Error_Without_Action__mkdirs_04");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_05()).runTest("CWE390_Error_Without_Action__mkdirs_05");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_06()).runTest("CWE390_Error_Without_Action__mkdirs_06");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_07()).runTest("CWE390_Error_Without_Action__mkdirs_07");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_08()).runTest("CWE390_Error_Without_Action__mkdirs_08");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_09()).runTest("CWE390_Error_Without_Action__mkdirs_09");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_10()).runTest("CWE390_Error_Without_Action__mkdirs_10");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_11()).runTest("CWE390_Error_Without_Action__mkdirs_11");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_12()).runTest("CWE390_Error_Without_Action__mkdirs_12");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_13()).runTest("CWE390_Error_Without_Action__mkdirs_13");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_14()).runTest("CWE390_Error_Without_Action__mkdirs_14");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_15()).runTest("CWE390_Error_Without_Action__mkdirs_15");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_16()).runTest("CWE390_Error_Without_Action__mkdirs_16");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_17()).runTest("CWE390_Error_Without_Action__mkdirs_17");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__mkdirs_19()).runTest("CWE390_Error_Without_Action__mkdirs_19");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_01()).runTest("CWE390_Error_Without_Action__npe_01");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_02()).runTest("CWE390_Error_Without_Action__npe_02");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_03()).runTest("CWE390_Error_Without_Action__npe_03");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_04()).runTest("CWE390_Error_Without_Action__npe_04");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_05()).runTest("CWE390_Error_Without_Action__npe_05");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_06()).runTest("CWE390_Error_Without_Action__npe_06");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_07()).runTest("CWE390_Error_Without_Action__npe_07");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_08()).runTest("CWE390_Error_Without_Action__npe_08");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_09()).runTest("CWE390_Error_Without_Action__npe_09");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_10()).runTest("CWE390_Error_Without_Action__npe_10");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_11()).runTest("CWE390_Error_Without_Action__npe_11");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_12()).runTest("CWE390_Error_Without_Action__npe_12");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_13()).runTest("CWE390_Error_Without_Action__npe_13");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_14()).runTest("CWE390_Error_Without_Action__npe_14");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_15()).runTest("CWE390_Error_Without_Action__npe_15");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_16()).runTest("CWE390_Error_Without_Action__npe_16");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_17()).runTest("CWE390_Error_Without_Action__npe_17");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__npe_19()).runTest("CWE390_Error_Without_Action__npe_19");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_01()).runTest("CWE390_Error_Without_Action__reader_01");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_02()).runTest("CWE390_Error_Without_Action__reader_02");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_03()).runTest("CWE390_Error_Without_Action__reader_03");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_04()).runTest("CWE390_Error_Without_Action__reader_04");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_05()).runTest("CWE390_Error_Without_Action__reader_05");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_06()).runTest("CWE390_Error_Without_Action__reader_06");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_07()).runTest("CWE390_Error_Without_Action__reader_07");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_08()).runTest("CWE390_Error_Without_Action__reader_08");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_09()).runTest("CWE390_Error_Without_Action__reader_09");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_10()).runTest("CWE390_Error_Without_Action__reader_10");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_11()).runTest("CWE390_Error_Without_Action__reader_11");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_12()).runTest("CWE390_Error_Without_Action__reader_12");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_13()).runTest("CWE390_Error_Without_Action__reader_13");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_14()).runTest("CWE390_Error_Without_Action__reader_14");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_15()).runTest("CWE390_Error_Without_Action__reader_15");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_16()).runTest("CWE390_Error_Without_Action__reader_16");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_17()).runTest("CWE390_Error_Without_Action__reader_17");
			(new testcases.CWE390_Error_Without_Action.CWE390_Error_Without_Action__reader_19()).runTest("CWE390_Error_Without_Action__reader_19");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_01()).runTest("CWE395_NullPointerException_Catch__basic_01");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_02()).runTest("CWE395_NullPointerException_Catch__basic_02");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_03()).runTest("CWE395_NullPointerException_Catch__basic_03");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_04()).runTest("CWE395_NullPointerException_Catch__basic_04");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_05()).runTest("CWE395_NullPointerException_Catch__basic_05");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_06()).runTest("CWE395_NullPointerException_Catch__basic_06");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_07()).runTest("CWE395_NullPointerException_Catch__basic_07");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_08()).runTest("CWE395_NullPointerException_Catch__basic_08");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_09()).runTest("CWE395_NullPointerException_Catch__basic_09");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_10()).runTest("CWE395_NullPointerException_Catch__basic_10");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_11()).runTest("CWE395_NullPointerException_Catch__basic_11");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_12()).runTest("CWE395_NullPointerException_Catch__basic_12");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_13()).runTest("CWE395_NullPointerException_Catch__basic_13");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_14()).runTest("CWE395_NullPointerException_Catch__basic_14");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_15()).runTest("CWE395_NullPointerException_Catch__basic_15");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_16()).runTest("CWE395_NullPointerException_Catch__basic_16");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_17()).runTest("CWE395_NullPointerException_Catch__basic_17");
			(new testcases.CWE395_NullPointerException_Catch.CWE395_NullPointerException_Catch__basic_19()).runTest("CWE395_NullPointerException_Catch__basic_19");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_01()).runTest("CWE396_Catch_Generic_Exception__Exception_01");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_02()).runTest("CWE396_Catch_Generic_Exception__Exception_02");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_03()).runTest("CWE396_Catch_Generic_Exception__Exception_03");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_04()).runTest("CWE396_Catch_Generic_Exception__Exception_04");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_05()).runTest("CWE396_Catch_Generic_Exception__Exception_05");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_06()).runTest("CWE396_Catch_Generic_Exception__Exception_06");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_07()).runTest("CWE396_Catch_Generic_Exception__Exception_07");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_08()).runTest("CWE396_Catch_Generic_Exception__Exception_08");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_09()).runTest("CWE396_Catch_Generic_Exception__Exception_09");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_10()).runTest("CWE396_Catch_Generic_Exception__Exception_10");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_11()).runTest("CWE396_Catch_Generic_Exception__Exception_11");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_12()).runTest("CWE396_Catch_Generic_Exception__Exception_12");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_13()).runTest("CWE396_Catch_Generic_Exception__Exception_13");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_14()).runTest("CWE396_Catch_Generic_Exception__Exception_14");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_15()).runTest("CWE396_Catch_Generic_Exception__Exception_15");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_16()).runTest("CWE396_Catch_Generic_Exception__Exception_16");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_17()).runTest("CWE396_Catch_Generic_Exception__Exception_17");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Exception_19()).runTest("CWE396_Catch_Generic_Exception__Exception_19");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_01()).runTest("CWE396_Catch_Generic_Exception__Throwable_01");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_02()).runTest("CWE396_Catch_Generic_Exception__Throwable_02");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_03()).runTest("CWE396_Catch_Generic_Exception__Throwable_03");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_04()).runTest("CWE396_Catch_Generic_Exception__Throwable_04");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_05()).runTest("CWE396_Catch_Generic_Exception__Throwable_05");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_06()).runTest("CWE396_Catch_Generic_Exception__Throwable_06");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_07()).runTest("CWE396_Catch_Generic_Exception__Throwable_07");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_08()).runTest("CWE396_Catch_Generic_Exception__Throwable_08");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_09()).runTest("CWE396_Catch_Generic_Exception__Throwable_09");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_10()).runTest("CWE396_Catch_Generic_Exception__Throwable_10");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_11()).runTest("CWE396_Catch_Generic_Exception__Throwable_11");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_12()).runTest("CWE396_Catch_Generic_Exception__Throwable_12");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_13()).runTest("CWE396_Catch_Generic_Exception__Throwable_13");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_14()).runTest("CWE396_Catch_Generic_Exception__Throwable_14");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_15()).runTest("CWE396_Catch_Generic_Exception__Throwable_15");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_16()).runTest("CWE396_Catch_Generic_Exception__Throwable_16");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_17()).runTest("CWE396_Catch_Generic_Exception__Throwable_17");
			(new testcases.CWE396_Catch_Generic_Exception.CWE396_Catch_Generic_Exception__Throwable_19()).runTest("CWE396_Catch_Generic_Exception__Throwable_19");
			(new testcases.CWE397_Throw_Generic_Exception.CWE397_Throw_Generic_Exception__declare_Exception_01()).runTest("CWE397_Throw_Generic_Exception__declare_Exception_01");
			(new testcases.CWE397_Throw_Generic_Exception.CWE397_Throw_Generic_Exception__declare_Throwable_01()).runTest("CWE397_Throw_Generic_Exception__declare_Throwable_01");
			(new testcases.CWE397_Throw_Generic_Exception.CWE397_Throw_Generic_Exception__throw_Exception_01()).runTest("CWE397_Throw_Generic_Exception__throw_Exception_01");
			(new testcases.CWE397_Throw_Generic_Exception.CWE397_Throw_Generic_Exception__throw_Throwable_01()).runTest("CWE397_Throw_Generic_Exception__throw_Throwable_01");
			/* END-AUTOGENERATED-JAVA-TESTS-3 */
}

private static void runTestCWE4() {
/* BEGIN-AUTOGENERATED-JAVA-TESTS-4 */
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_01()).runTest("CWE400_Resource_Exhaustion__heap_01");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_02()).runTest("CWE400_Resource_Exhaustion__heap_02");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_03()).runTest("CWE400_Resource_Exhaustion__heap_03");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_04()).runTest("CWE400_Resource_Exhaustion__heap_04");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_05()).runTest("CWE400_Resource_Exhaustion__heap_05");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_06()).runTest("CWE400_Resource_Exhaustion__heap_06");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_07()).runTest("CWE400_Resource_Exhaustion__heap_07");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_08()).runTest("CWE400_Resource_Exhaustion__heap_08");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_09()).runTest("CWE400_Resource_Exhaustion__heap_09");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_10()).runTest("CWE400_Resource_Exhaustion__heap_10");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_11()).runTest("CWE400_Resource_Exhaustion__heap_11");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_12()).runTest("CWE400_Resource_Exhaustion__heap_12");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_13()).runTest("CWE400_Resource_Exhaustion__heap_13");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_14()).runTest("CWE400_Resource_Exhaustion__heap_14");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_15()).runTest("CWE400_Resource_Exhaustion__heap_15");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_16()).runTest("CWE400_Resource_Exhaustion__heap_16");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_17()).runTest("CWE400_Resource_Exhaustion__heap_17");
			(new testcases.CWE400_Resource_Exhaustion.CWE400_Resource_Exhaustion__heap_19()).runTest("CWE400_Resource_Exhaustion__heap_19");
			(new testcases.CWE404_Improper_Resource_Shutdown.CWE404_Improper_Resource_Shutdown__console_InputStreamReader_01()).runTest("CWE404_Improper_Resource_Shutdown__console_InputStreamReader_01");
			(new testcases.CWE404_Improper_Resource_Shutdown.CWE404_Improper_Resource_Shutdown__db_Connection_01()).runTest("CWE404_Improper_Resource_Shutdown__db_Connection_01");
			(new testcases.CWE404_Improper_Resource_Shutdown.CWE404_Improper_Resource_Shutdown__FileReader_01()).runTest("CWE404_Improper_Resource_Shutdown__FileReader_01");
			(new testcases.CWE413_Insufficient_Resource_Locking.CWE413_Insufficient_Resource_Locking__console_reentrant_function_unsync_01_bad()).runTest("CWE413_Insufficient_Resource_Locking__console_reentrant_function_unsync_01_bad");
			(new testcases.CWE413_Insufficient_Resource_Locking.CWE413_Insufficient_Resource_Locking__console_reentrant_function_unsync_01_good1()).runTest("CWE413_Insufficient_Resource_Locking__console_reentrant_function_unsync_01_good1");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_01()).runTest("CWE470_Unsafe_Reflection__connect_tcp_01");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_02()).runTest("CWE470_Unsafe_Reflection__connect_tcp_02");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_03()).runTest("CWE470_Unsafe_Reflection__connect_tcp_03");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_04()).runTest("CWE470_Unsafe_Reflection__connect_tcp_04");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_05()).runTest("CWE470_Unsafe_Reflection__connect_tcp_05");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_06()).runTest("CWE470_Unsafe_Reflection__connect_tcp_06");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_07()).runTest("CWE470_Unsafe_Reflection__connect_tcp_07");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_08()).runTest("CWE470_Unsafe_Reflection__connect_tcp_08");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_09()).runTest("CWE470_Unsafe_Reflection__connect_tcp_09");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_10()).runTest("CWE470_Unsafe_Reflection__connect_tcp_10");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_11()).runTest("CWE470_Unsafe_Reflection__connect_tcp_11");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_12()).runTest("CWE470_Unsafe_Reflection__connect_tcp_12");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_13()).runTest("CWE470_Unsafe_Reflection__connect_tcp_13");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_14()).runTest("CWE470_Unsafe_Reflection__connect_tcp_14");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_15()).runTest("CWE470_Unsafe_Reflection__connect_tcp_15");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_16()).runTest("CWE470_Unsafe_Reflection__connect_tcp_16");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_17()).runTest("CWE470_Unsafe_Reflection__connect_tcp_17");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_19()).runTest("CWE470_Unsafe_Reflection__connect_tcp_19");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_31()).runTest("CWE470_Unsafe_Reflection__connect_tcp_31");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_41()).runTest("CWE470_Unsafe_Reflection__connect_tcp_41");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_42()).runTest("CWE470_Unsafe_Reflection__connect_tcp_42");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_45()).runTest("CWE470_Unsafe_Reflection__connect_tcp_45");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_51a()).runTest("CWE470_Unsafe_Reflection__connect_tcp_51a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_52a()).runTest("CWE470_Unsafe_Reflection__connect_tcp_52a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_53a()).runTest("CWE470_Unsafe_Reflection__connect_tcp_53a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_54a()).runTest("CWE470_Unsafe_Reflection__connect_tcp_54a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_61a()).runTest("CWE470_Unsafe_Reflection__connect_tcp_61a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_66a()).runTest("CWE470_Unsafe_Reflection__connect_tcp_66a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_67a()).runTest("CWE470_Unsafe_Reflection__connect_tcp_67a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_68a()).runTest("CWE470_Unsafe_Reflection__connect_tcp_68a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__connect_tcp_71a()).runTest("CWE470_Unsafe_Reflection__connect_tcp_71a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_01()).runTest("CWE470_Unsafe_Reflection__console_readLine_01");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_02()).runTest("CWE470_Unsafe_Reflection__console_readLine_02");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_03()).runTest("CWE470_Unsafe_Reflection__console_readLine_03");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_04()).runTest("CWE470_Unsafe_Reflection__console_readLine_04");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_05()).runTest("CWE470_Unsafe_Reflection__console_readLine_05");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_06()).runTest("CWE470_Unsafe_Reflection__console_readLine_06");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_07()).runTest("CWE470_Unsafe_Reflection__console_readLine_07");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_08()).runTest("CWE470_Unsafe_Reflection__console_readLine_08");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_09()).runTest("CWE470_Unsafe_Reflection__console_readLine_09");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_10()).runTest("CWE470_Unsafe_Reflection__console_readLine_10");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_11()).runTest("CWE470_Unsafe_Reflection__console_readLine_11");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_12()).runTest("CWE470_Unsafe_Reflection__console_readLine_12");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_13()).runTest("CWE470_Unsafe_Reflection__console_readLine_13");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_14()).runTest("CWE470_Unsafe_Reflection__console_readLine_14");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_15()).runTest("CWE470_Unsafe_Reflection__console_readLine_15");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_16()).runTest("CWE470_Unsafe_Reflection__console_readLine_16");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_17()).runTest("CWE470_Unsafe_Reflection__console_readLine_17");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_19()).runTest("CWE470_Unsafe_Reflection__console_readLine_19");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_31()).runTest("CWE470_Unsafe_Reflection__console_readLine_31");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_41()).runTest("CWE470_Unsafe_Reflection__console_readLine_41");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_42()).runTest("CWE470_Unsafe_Reflection__console_readLine_42");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_45()).runTest("CWE470_Unsafe_Reflection__console_readLine_45");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_51a()).runTest("CWE470_Unsafe_Reflection__console_readLine_51a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_52a()).runTest("CWE470_Unsafe_Reflection__console_readLine_52a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_53a()).runTest("CWE470_Unsafe_Reflection__console_readLine_53a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_54a()).runTest("CWE470_Unsafe_Reflection__console_readLine_54a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_61a()).runTest("CWE470_Unsafe_Reflection__console_readLine_61a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_66a()).runTest("CWE470_Unsafe_Reflection__console_readLine_66a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_67a()).runTest("CWE470_Unsafe_Reflection__console_readLine_67a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_68a()).runTest("CWE470_Unsafe_Reflection__console_readLine_68a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__console_readLine_71a()).runTest("CWE470_Unsafe_Reflection__console_readLine_71a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_01()).runTest("CWE470_Unsafe_Reflection__Environment_01");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_02()).runTest("CWE470_Unsafe_Reflection__Environment_02");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_03()).runTest("CWE470_Unsafe_Reflection__Environment_03");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_04()).runTest("CWE470_Unsafe_Reflection__Environment_04");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_05()).runTest("CWE470_Unsafe_Reflection__Environment_05");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_06()).runTest("CWE470_Unsafe_Reflection__Environment_06");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_07()).runTest("CWE470_Unsafe_Reflection__Environment_07");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_08()).runTest("CWE470_Unsafe_Reflection__Environment_08");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_09()).runTest("CWE470_Unsafe_Reflection__Environment_09");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_10()).runTest("CWE470_Unsafe_Reflection__Environment_10");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_11()).runTest("CWE470_Unsafe_Reflection__Environment_11");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_12()).runTest("CWE470_Unsafe_Reflection__Environment_12");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_13()).runTest("CWE470_Unsafe_Reflection__Environment_13");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_14()).runTest("CWE470_Unsafe_Reflection__Environment_14");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_15()).runTest("CWE470_Unsafe_Reflection__Environment_15");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_16()).runTest("CWE470_Unsafe_Reflection__Environment_16");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_17()).runTest("CWE470_Unsafe_Reflection__Environment_17");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_19()).runTest("CWE470_Unsafe_Reflection__Environment_19");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_31()).runTest("CWE470_Unsafe_Reflection__Environment_31");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_41()).runTest("CWE470_Unsafe_Reflection__Environment_41");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_42()).runTest("CWE470_Unsafe_Reflection__Environment_42");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_45()).runTest("CWE470_Unsafe_Reflection__Environment_45");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_51a()).runTest("CWE470_Unsafe_Reflection__Environment_51a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_52a()).runTest("CWE470_Unsafe_Reflection__Environment_52a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_53a()).runTest("CWE470_Unsafe_Reflection__Environment_53a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_54a()).runTest("CWE470_Unsafe_Reflection__Environment_54a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_61a()).runTest("CWE470_Unsafe_Reflection__Environment_61a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_66a()).runTest("CWE470_Unsafe_Reflection__Environment_66a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_67a()).runTest("CWE470_Unsafe_Reflection__Environment_67a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_68a()).runTest("CWE470_Unsafe_Reflection__Environment_68a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__Environment_71a()).runTest("CWE470_Unsafe_Reflection__Environment_71a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_01()).runTest("CWE470_Unsafe_Reflection__fromDB_01");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_02()).runTest("CWE470_Unsafe_Reflection__fromDB_02");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_03()).runTest("CWE470_Unsafe_Reflection__fromDB_03");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_04()).runTest("CWE470_Unsafe_Reflection__fromDB_04");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_05()).runTest("CWE470_Unsafe_Reflection__fromDB_05");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_06()).runTest("CWE470_Unsafe_Reflection__fromDB_06");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_07()).runTest("CWE470_Unsafe_Reflection__fromDB_07");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_08()).runTest("CWE470_Unsafe_Reflection__fromDB_08");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_09()).runTest("CWE470_Unsafe_Reflection__fromDB_09");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_10()).runTest("CWE470_Unsafe_Reflection__fromDB_10");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_11()).runTest("CWE470_Unsafe_Reflection__fromDB_11");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_12()).runTest("CWE470_Unsafe_Reflection__fromDB_12");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_13()).runTest("CWE470_Unsafe_Reflection__fromDB_13");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_14()).runTest("CWE470_Unsafe_Reflection__fromDB_14");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_15()).runTest("CWE470_Unsafe_Reflection__fromDB_15");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_16()).runTest("CWE470_Unsafe_Reflection__fromDB_16");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_17()).runTest("CWE470_Unsafe_Reflection__fromDB_17");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_19()).runTest("CWE470_Unsafe_Reflection__fromDB_19");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_31()).runTest("CWE470_Unsafe_Reflection__fromDB_31");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_41()).runTest("CWE470_Unsafe_Reflection__fromDB_41");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_42()).runTest("CWE470_Unsafe_Reflection__fromDB_42");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_45()).runTest("CWE470_Unsafe_Reflection__fromDB_45");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_51a()).runTest("CWE470_Unsafe_Reflection__fromDB_51a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_52a()).runTest("CWE470_Unsafe_Reflection__fromDB_52a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_53a()).runTest("CWE470_Unsafe_Reflection__fromDB_53a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_54a()).runTest("CWE470_Unsafe_Reflection__fromDB_54a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_61a()).runTest("CWE470_Unsafe_Reflection__fromDB_61a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_66a()).runTest("CWE470_Unsafe_Reflection__fromDB_66a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_67a()).runTest("CWE470_Unsafe_Reflection__fromDB_67a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_68a()).runTest("CWE470_Unsafe_Reflection__fromDB_68a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromDB_71a()).runTest("CWE470_Unsafe_Reflection__fromDB_71a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_01()).runTest("CWE470_Unsafe_Reflection__fromFile_01");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_02()).runTest("CWE470_Unsafe_Reflection__fromFile_02");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_03()).runTest("CWE470_Unsafe_Reflection__fromFile_03");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_04()).runTest("CWE470_Unsafe_Reflection__fromFile_04");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_05()).runTest("CWE470_Unsafe_Reflection__fromFile_05");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_06()).runTest("CWE470_Unsafe_Reflection__fromFile_06");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_07()).runTest("CWE470_Unsafe_Reflection__fromFile_07");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_08()).runTest("CWE470_Unsafe_Reflection__fromFile_08");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_09()).runTest("CWE470_Unsafe_Reflection__fromFile_09");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_10()).runTest("CWE470_Unsafe_Reflection__fromFile_10");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_11()).runTest("CWE470_Unsafe_Reflection__fromFile_11");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_12()).runTest("CWE470_Unsafe_Reflection__fromFile_12");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_13()).runTest("CWE470_Unsafe_Reflection__fromFile_13");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_14()).runTest("CWE470_Unsafe_Reflection__fromFile_14");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_15()).runTest("CWE470_Unsafe_Reflection__fromFile_15");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_16()).runTest("CWE470_Unsafe_Reflection__fromFile_16");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_17()).runTest("CWE470_Unsafe_Reflection__fromFile_17");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_19()).runTest("CWE470_Unsafe_Reflection__fromFile_19");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_31()).runTest("CWE470_Unsafe_Reflection__fromFile_31");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_41()).runTest("CWE470_Unsafe_Reflection__fromFile_41");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_42()).runTest("CWE470_Unsafe_Reflection__fromFile_42");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_45()).runTest("CWE470_Unsafe_Reflection__fromFile_45");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_51a()).runTest("CWE470_Unsafe_Reflection__fromFile_51a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_52a()).runTest("CWE470_Unsafe_Reflection__fromFile_52a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_53a()).runTest("CWE470_Unsafe_Reflection__fromFile_53a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_54a()).runTest("CWE470_Unsafe_Reflection__fromFile_54a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_61a()).runTest("CWE470_Unsafe_Reflection__fromFile_61a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_66a()).runTest("CWE470_Unsafe_Reflection__fromFile_66a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_67a()).runTest("CWE470_Unsafe_Reflection__fromFile_67a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_68a()).runTest("CWE470_Unsafe_Reflection__fromFile_68a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__fromFile_71a()).runTest("CWE470_Unsafe_Reflection__fromFile_71a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_01()).runTest("CWE470_Unsafe_Reflection__listen_tcp_01");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_02()).runTest("CWE470_Unsafe_Reflection__listen_tcp_02");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_03()).runTest("CWE470_Unsafe_Reflection__listen_tcp_03");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_04()).runTest("CWE470_Unsafe_Reflection__listen_tcp_04");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_05()).runTest("CWE470_Unsafe_Reflection__listen_tcp_05");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_06()).runTest("CWE470_Unsafe_Reflection__listen_tcp_06");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_07()).runTest("CWE470_Unsafe_Reflection__listen_tcp_07");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_08()).runTest("CWE470_Unsafe_Reflection__listen_tcp_08");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_09()).runTest("CWE470_Unsafe_Reflection__listen_tcp_09");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_10()).runTest("CWE470_Unsafe_Reflection__listen_tcp_10");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_11()).runTest("CWE470_Unsafe_Reflection__listen_tcp_11");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_12()).runTest("CWE470_Unsafe_Reflection__listen_tcp_12");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_13()).runTest("CWE470_Unsafe_Reflection__listen_tcp_13");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_14()).runTest("CWE470_Unsafe_Reflection__listen_tcp_14");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_15()).runTest("CWE470_Unsafe_Reflection__listen_tcp_15");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_16()).runTest("CWE470_Unsafe_Reflection__listen_tcp_16");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_17()).runTest("CWE470_Unsafe_Reflection__listen_tcp_17");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_19()).runTest("CWE470_Unsafe_Reflection__listen_tcp_19");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_31()).runTest("CWE470_Unsafe_Reflection__listen_tcp_31");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_41()).runTest("CWE470_Unsafe_Reflection__listen_tcp_41");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_42()).runTest("CWE470_Unsafe_Reflection__listen_tcp_42");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_45()).runTest("CWE470_Unsafe_Reflection__listen_tcp_45");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_51a()).runTest("CWE470_Unsafe_Reflection__listen_tcp_51a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_52a()).runTest("CWE470_Unsafe_Reflection__listen_tcp_52a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_53a()).runTest("CWE470_Unsafe_Reflection__listen_tcp_53a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_54a()).runTest("CWE470_Unsafe_Reflection__listen_tcp_54a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_61a()).runTest("CWE470_Unsafe_Reflection__listen_tcp_61a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_66a()).runTest("CWE470_Unsafe_Reflection__listen_tcp_66a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_67a()).runTest("CWE470_Unsafe_Reflection__listen_tcp_67a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_68a()).runTest("CWE470_Unsafe_Reflection__listen_tcp_68a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__listen_tcp_71a()).runTest("CWE470_Unsafe_Reflection__listen_tcp_71a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_01()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_01");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_02()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_02");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_03()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_03");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_04()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_04");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_05()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_05");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_06()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_06");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_07()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_07");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_08()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_08");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_09()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_09");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_10()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_10");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_11()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_11");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_12()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_12");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_13()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_13");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_14()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_14");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_15()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_15");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_16()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_16");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_17()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_17");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_19()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_19");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_31()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_31");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_41()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_41");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_42()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_42");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_45()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_45");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_51a()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_51a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_52a()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_52a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_53a()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_53a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_54a()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_54a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_61a()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_61a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_66a()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_66a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_67a()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_67a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_68a()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_68a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__PropertiesFile_71a()).runTest("CWE470_Unsafe_Reflection__PropertiesFile_71a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_01()).runTest("CWE470_Unsafe_Reflection__URLConnection_01");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_02()).runTest("CWE470_Unsafe_Reflection__URLConnection_02");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_03()).runTest("CWE470_Unsafe_Reflection__URLConnection_03");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_04()).runTest("CWE470_Unsafe_Reflection__URLConnection_04");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_05()).runTest("CWE470_Unsafe_Reflection__URLConnection_05");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_06()).runTest("CWE470_Unsafe_Reflection__URLConnection_06");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_07()).runTest("CWE470_Unsafe_Reflection__URLConnection_07");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_08()).runTest("CWE470_Unsafe_Reflection__URLConnection_08");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_09()).runTest("CWE470_Unsafe_Reflection__URLConnection_09");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_10()).runTest("CWE470_Unsafe_Reflection__URLConnection_10");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_11()).runTest("CWE470_Unsafe_Reflection__URLConnection_11");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_12()).runTest("CWE470_Unsafe_Reflection__URLConnection_12");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_13()).runTest("CWE470_Unsafe_Reflection__URLConnection_13");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_14()).runTest("CWE470_Unsafe_Reflection__URLConnection_14");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_15()).runTest("CWE470_Unsafe_Reflection__URLConnection_15");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_16()).runTest("CWE470_Unsafe_Reflection__URLConnection_16");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_17()).runTest("CWE470_Unsafe_Reflection__URLConnection_17");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_19()).runTest("CWE470_Unsafe_Reflection__URLConnection_19");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_31()).runTest("CWE470_Unsafe_Reflection__URLConnection_31");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_41()).runTest("CWE470_Unsafe_Reflection__URLConnection_41");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_42()).runTest("CWE470_Unsafe_Reflection__URLConnection_42");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_45()).runTest("CWE470_Unsafe_Reflection__URLConnection_45");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_51a()).runTest("CWE470_Unsafe_Reflection__URLConnection_51a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_52a()).runTest("CWE470_Unsafe_Reflection__URLConnection_52a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_53a()).runTest("CWE470_Unsafe_Reflection__URLConnection_53a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_54a()).runTest("CWE470_Unsafe_Reflection__URLConnection_54a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_61a()).runTest("CWE470_Unsafe_Reflection__URLConnection_61a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_66a()).runTest("CWE470_Unsafe_Reflection__URLConnection_66a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_67a()).runTest("CWE470_Unsafe_Reflection__URLConnection_67a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_68a()).runTest("CWE470_Unsafe_Reflection__URLConnection_68a");
			(new testcases.CWE470_Unsafe_Reflection.CWE470_Unsafe_Reflection__URLConnection_71a()).runTest("CWE470_Unsafe_Reflection__URLConnection_71a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_01()).runTest("CWE476_NULL_Pointer_Dereference__Integer_01");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_02()).runTest("CWE476_NULL_Pointer_Dereference__Integer_02");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_03()).runTest("CWE476_NULL_Pointer_Dereference__Integer_03");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_04()).runTest("CWE476_NULL_Pointer_Dereference__Integer_04");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_05()).runTest("CWE476_NULL_Pointer_Dereference__Integer_05");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_06()).runTest("CWE476_NULL_Pointer_Dereference__Integer_06");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_07()).runTest("CWE476_NULL_Pointer_Dereference__Integer_07");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_08()).runTest("CWE476_NULL_Pointer_Dereference__Integer_08");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_09()).runTest("CWE476_NULL_Pointer_Dereference__Integer_09");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_10()).runTest("CWE476_NULL_Pointer_Dereference__Integer_10");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_11()).runTest("CWE476_NULL_Pointer_Dereference__Integer_11");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_12()).runTest("CWE476_NULL_Pointer_Dereference__Integer_12");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_13()).runTest("CWE476_NULL_Pointer_Dereference__Integer_13");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_14()).runTest("CWE476_NULL_Pointer_Dereference__Integer_14");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_15()).runTest("CWE476_NULL_Pointer_Dereference__Integer_15");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_16()).runTest("CWE476_NULL_Pointer_Dereference__Integer_16");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_17()).runTest("CWE476_NULL_Pointer_Dereference__Integer_17");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_19()).runTest("CWE476_NULL_Pointer_Dereference__Integer_19");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_31()).runTest("CWE476_NULL_Pointer_Dereference__Integer_31");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_41()).runTest("CWE476_NULL_Pointer_Dereference__Integer_41");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_42()).runTest("CWE476_NULL_Pointer_Dereference__Integer_42");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_45()).runTest("CWE476_NULL_Pointer_Dereference__Integer_45");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_51a()).runTest("CWE476_NULL_Pointer_Dereference__Integer_51a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_52a()).runTest("CWE476_NULL_Pointer_Dereference__Integer_52a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_53a()).runTest("CWE476_NULL_Pointer_Dereference__Integer_53a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_54a()).runTest("CWE476_NULL_Pointer_Dereference__Integer_54a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_61a()).runTest("CWE476_NULL_Pointer_Dereference__Integer_61a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_66a()).runTest("CWE476_NULL_Pointer_Dereference__Integer_66a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_67a()).runTest("CWE476_NULL_Pointer_Dereference__Integer_67a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_68a()).runTest("CWE476_NULL_Pointer_Dereference__Integer_68a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__Integer_71a()).runTest("CWE476_NULL_Pointer_Dereference__Integer_71a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_01()).runTest("CWE476_NULL_Pointer_Dereference__int_array_01");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_02()).runTest("CWE476_NULL_Pointer_Dereference__int_array_02");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_03()).runTest("CWE476_NULL_Pointer_Dereference__int_array_03");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_04()).runTest("CWE476_NULL_Pointer_Dereference__int_array_04");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_05()).runTest("CWE476_NULL_Pointer_Dereference__int_array_05");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_06()).runTest("CWE476_NULL_Pointer_Dereference__int_array_06");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_07()).runTest("CWE476_NULL_Pointer_Dereference__int_array_07");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_08()).runTest("CWE476_NULL_Pointer_Dereference__int_array_08");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_09()).runTest("CWE476_NULL_Pointer_Dereference__int_array_09");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_10()).runTest("CWE476_NULL_Pointer_Dereference__int_array_10");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_11()).runTest("CWE476_NULL_Pointer_Dereference__int_array_11");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_12()).runTest("CWE476_NULL_Pointer_Dereference__int_array_12");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_13()).runTest("CWE476_NULL_Pointer_Dereference__int_array_13");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_14()).runTest("CWE476_NULL_Pointer_Dereference__int_array_14");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_15()).runTest("CWE476_NULL_Pointer_Dereference__int_array_15");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_16()).runTest("CWE476_NULL_Pointer_Dereference__int_array_16");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_17()).runTest("CWE476_NULL_Pointer_Dereference__int_array_17");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_19()).runTest("CWE476_NULL_Pointer_Dereference__int_array_19");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_31()).runTest("CWE476_NULL_Pointer_Dereference__int_array_31");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_41()).runTest("CWE476_NULL_Pointer_Dereference__int_array_41");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_42()).runTest("CWE476_NULL_Pointer_Dereference__int_array_42");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_45()).runTest("CWE476_NULL_Pointer_Dereference__int_array_45");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_51a()).runTest("CWE476_NULL_Pointer_Dereference__int_array_51a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_52a()).runTest("CWE476_NULL_Pointer_Dereference__int_array_52a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_53a()).runTest("CWE476_NULL_Pointer_Dereference__int_array_53a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_54a()).runTest("CWE476_NULL_Pointer_Dereference__int_array_54a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_61a()).runTest("CWE476_NULL_Pointer_Dereference__int_array_61a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_67a()).runTest("CWE476_NULL_Pointer_Dereference__int_array_67a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_68a()).runTest("CWE476_NULL_Pointer_Dereference__int_array_68a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__int_array_71a()).runTest("CWE476_NULL_Pointer_Dereference__int_array_71a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_01()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_01");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_02()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_02");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_03()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_03");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_04()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_04");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_05()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_05");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_06()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_06");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_07()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_07");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_08()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_08");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_09()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_09");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_10()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_10");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_11()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_11");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_12()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_12");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_13()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_13");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_14()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_14");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_15()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_15");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_16()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_16");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_17()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_17");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_19()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_19");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_31()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_31");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_41()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_41");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_42()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_42");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_45()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_45");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_51a()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_51a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_52a()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_52a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_53a()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_53a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_54a()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_54a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_61a()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_61a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_66a()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_66a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_67a()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_67a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_68a()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_68a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__StringBuilder_71a()).runTest("CWE476_NULL_Pointer_Dereference__StringBuilder_71a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_01()).runTest("CWE476_NULL_Pointer_Dereference__String_01");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_02()).runTest("CWE476_NULL_Pointer_Dereference__String_02");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_03()).runTest("CWE476_NULL_Pointer_Dereference__String_03");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_04()).runTest("CWE476_NULL_Pointer_Dereference__String_04");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_05()).runTest("CWE476_NULL_Pointer_Dereference__String_05");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_06()).runTest("CWE476_NULL_Pointer_Dereference__String_06");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_07()).runTest("CWE476_NULL_Pointer_Dereference__String_07");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_08()).runTest("CWE476_NULL_Pointer_Dereference__String_08");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_09()).runTest("CWE476_NULL_Pointer_Dereference__String_09");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_10()).runTest("CWE476_NULL_Pointer_Dereference__String_10");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_11()).runTest("CWE476_NULL_Pointer_Dereference__String_11");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_12()).runTest("CWE476_NULL_Pointer_Dereference__String_12");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_13()).runTest("CWE476_NULL_Pointer_Dereference__String_13");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_14()).runTest("CWE476_NULL_Pointer_Dereference__String_14");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_15()).runTest("CWE476_NULL_Pointer_Dereference__String_15");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_16()).runTest("CWE476_NULL_Pointer_Dereference__String_16");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_17()).runTest("CWE476_NULL_Pointer_Dereference__String_17");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_19()).runTest("CWE476_NULL_Pointer_Dereference__String_19");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_31()).runTest("CWE476_NULL_Pointer_Dereference__String_31");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_41()).runTest("CWE476_NULL_Pointer_Dereference__String_41");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_42()).runTest("CWE476_NULL_Pointer_Dereference__String_42");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_45()).runTest("CWE476_NULL_Pointer_Dereference__String_45");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_51a()).runTest("CWE476_NULL_Pointer_Dereference__String_51a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_52a()).runTest("CWE476_NULL_Pointer_Dereference__String_52a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_53a()).runTest("CWE476_NULL_Pointer_Dereference__String_53a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_54a()).runTest("CWE476_NULL_Pointer_Dereference__String_54a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_61a()).runTest("CWE476_NULL_Pointer_Dereference__String_61a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_66a()).runTest("CWE476_NULL_Pointer_Dereference__String_66a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_67a()).runTest("CWE476_NULL_Pointer_Dereference__String_67a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_68a()).runTest("CWE476_NULL_Pointer_Dereference__String_68a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__String_71a()).runTest("CWE476_NULL_Pointer_Dereference__String_71a");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_01()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_01");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_02()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_02");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_03()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_03");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_04()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_04");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_05()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_05");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_06()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_06");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_07()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_07");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_08()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_08");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_09()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_09");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_10()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_10");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_11()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_11");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_12()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_12");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_13()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_13");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_14()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_14");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_15()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_15");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_16()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_16");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_17()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_17");
			(new testcases.CWE476_NULL_Pointer_Dereference.CWE476_NULL_Pointer_Dereference__undefinedValue_19()).runTest("CWE476_NULL_Pointer_Dereference__undefinedValue_19");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_01()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_01");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_02()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_02");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_03()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_03");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_04()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_04");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_05()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_05");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_06()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_06");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_07()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_07");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_08()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_08");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_09()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_09");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_10()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_10");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_11()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_11");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_12()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_12");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_13()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_13");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_14()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_14");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_15()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_15");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_16()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_16");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_17()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_17");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__DataInputStream_readLine_19()).runTest("CWE477_Obsolete_Functions__DataInputStream_readLine_19");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_01()).runTest("CWE477_Obsolete_Functions__Date_parse_01");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_02()).runTest("CWE477_Obsolete_Functions__Date_parse_02");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_03()).runTest("CWE477_Obsolete_Functions__Date_parse_03");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_04()).runTest("CWE477_Obsolete_Functions__Date_parse_04");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_05()).runTest("CWE477_Obsolete_Functions__Date_parse_05");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_06()).runTest("CWE477_Obsolete_Functions__Date_parse_06");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_07()).runTest("CWE477_Obsolete_Functions__Date_parse_07");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_08()).runTest("CWE477_Obsolete_Functions__Date_parse_08");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_09()).runTest("CWE477_Obsolete_Functions__Date_parse_09");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_10()).runTest("CWE477_Obsolete_Functions__Date_parse_10");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_11()).runTest("CWE477_Obsolete_Functions__Date_parse_11");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_12()).runTest("CWE477_Obsolete_Functions__Date_parse_12");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_13()).runTest("CWE477_Obsolete_Functions__Date_parse_13");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_14()).runTest("CWE477_Obsolete_Functions__Date_parse_14");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_15()).runTest("CWE477_Obsolete_Functions__Date_parse_15");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_16()).runTest("CWE477_Obsolete_Functions__Date_parse_16");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_17()).runTest("CWE477_Obsolete_Functions__Date_parse_17");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__Date_parse_19()).runTest("CWE477_Obsolete_Functions__Date_parse_19");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_01()).runTest("CWE477_Obsolete_Functions__String_getBytes_01");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_02()).runTest("CWE477_Obsolete_Functions__String_getBytes_02");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_03()).runTest("CWE477_Obsolete_Functions__String_getBytes_03");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_04()).runTest("CWE477_Obsolete_Functions__String_getBytes_04");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_05()).runTest("CWE477_Obsolete_Functions__String_getBytes_05");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_06()).runTest("CWE477_Obsolete_Functions__String_getBytes_06");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_07()).runTest("CWE477_Obsolete_Functions__String_getBytes_07");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_08()).runTest("CWE477_Obsolete_Functions__String_getBytes_08");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_09()).runTest("CWE477_Obsolete_Functions__String_getBytes_09");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_10()).runTest("CWE477_Obsolete_Functions__String_getBytes_10");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_11()).runTest("CWE477_Obsolete_Functions__String_getBytes_11");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_12()).runTest("CWE477_Obsolete_Functions__String_getBytes_12");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_13()).runTest("CWE477_Obsolete_Functions__String_getBytes_13");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_14()).runTest("CWE477_Obsolete_Functions__String_getBytes_14");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_15()).runTest("CWE477_Obsolete_Functions__String_getBytes_15");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_16()).runTest("CWE477_Obsolete_Functions__String_getBytes_16");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_17()).runTest("CWE477_Obsolete_Functions__String_getBytes_17");
			(new testcases.CWE477_Obsolete_Functions.CWE477_Obsolete_Functions__String_getBytes_19()).runTest("CWE477_Obsolete_Functions__String_getBytes_19");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_01()).runTest("CWE481_Assigning_instead_of_Comparing__basic_01");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_02()).runTest("CWE481_Assigning_instead_of_Comparing__basic_02");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_03()).runTest("CWE481_Assigning_instead_of_Comparing__basic_03");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_04()).runTest("CWE481_Assigning_instead_of_Comparing__basic_04");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_05()).runTest("CWE481_Assigning_instead_of_Comparing__basic_05");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_06()).runTest("CWE481_Assigning_instead_of_Comparing__basic_06");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_07()).runTest("CWE481_Assigning_instead_of_Comparing__basic_07");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_08()).runTest("CWE481_Assigning_instead_of_Comparing__basic_08");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_09()).runTest("CWE481_Assigning_instead_of_Comparing__basic_09");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_10()).runTest("CWE481_Assigning_instead_of_Comparing__basic_10");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_11()).runTest("CWE481_Assigning_instead_of_Comparing__basic_11");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_12()).runTest("CWE481_Assigning_instead_of_Comparing__basic_12");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_13()).runTest("CWE481_Assigning_instead_of_Comparing__basic_13");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_14()).runTest("CWE481_Assigning_instead_of_Comparing__basic_14");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_15()).runTest("CWE481_Assigning_instead_of_Comparing__basic_15");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_16()).runTest("CWE481_Assigning_instead_of_Comparing__basic_16");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_17()).runTest("CWE481_Assigning_instead_of_Comparing__basic_17");
			(new testcases.CWE481_Assigning_instead_of_Comparing.CWE481_Assigning_instead_of_Comparing__basic_19()).runTest("CWE481_Assigning_instead_of_Comparing__basic_19");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_01()).runTest("CWE482_Comparing_instead_of_Assigning__basic_01");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_02()).runTest("CWE482_Comparing_instead_of_Assigning__basic_02");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_03()).runTest("CWE482_Comparing_instead_of_Assigning__basic_03");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_04()).runTest("CWE482_Comparing_instead_of_Assigning__basic_04");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_05()).runTest("CWE482_Comparing_instead_of_Assigning__basic_05");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_06()).runTest("CWE482_Comparing_instead_of_Assigning__basic_06");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_07()).runTest("CWE482_Comparing_instead_of_Assigning__basic_07");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_08()).runTest("CWE482_Comparing_instead_of_Assigning__basic_08");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_09()).runTest("CWE482_Comparing_instead_of_Assigning__basic_09");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_10()).runTest("CWE482_Comparing_instead_of_Assigning__basic_10");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_11()).runTest("CWE482_Comparing_instead_of_Assigning__basic_11");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_12()).runTest("CWE482_Comparing_instead_of_Assigning__basic_12");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_13()).runTest("CWE482_Comparing_instead_of_Assigning__basic_13");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_14()).runTest("CWE482_Comparing_instead_of_Assigning__basic_14");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_15()).runTest("CWE482_Comparing_instead_of_Assigning__basic_15");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_16()).runTest("CWE482_Comparing_instead_of_Assigning__basic_16");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_17()).runTest("CWE482_Comparing_instead_of_Assigning__basic_17");
			(new testcases.CWE482_Comparing_instead_of_Assigning.CWE482_Comparing_instead_of_Assigning__basic_19()).runTest("CWE482_Comparing_instead_of_Assigning__basic_19");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__if_without_braces_multiline_01()).runTest("CWE483_Incorrect_Block_Delimitation__if_without_braces_multiline_01");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__if_without_braces_single_line_01()).runTest("CWE483_Incorrect_Block_Delimitation__if_without_braces_single_line_01");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_01()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_01");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_02()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_02");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_03()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_03");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_04()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_04");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_05()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_05");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_06()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_06");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_07()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_07");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_08()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_08");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_09()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_09");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_10()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_10");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_11()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_11");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_12()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_12");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_13()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_13");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_14()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_14");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_15()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_15");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_16()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_16");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_17()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_17");
			(new testcases.CWE483_Incorrect_Block_Delimitation.CWE483_Incorrect_Block_Delimitation__semicolon_19()).runTest("CWE483_Incorrect_Block_Delimitation__semicolon_19");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_01()).runTest("CWE484_Omitted_Break_Statement__basic_01");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_02()).runTest("CWE484_Omitted_Break_Statement__basic_02");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_03()).runTest("CWE484_Omitted_Break_Statement__basic_03");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_04()).runTest("CWE484_Omitted_Break_Statement__basic_04");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_05()).runTest("CWE484_Omitted_Break_Statement__basic_05");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_06()).runTest("CWE484_Omitted_Break_Statement__basic_06");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_07()).runTest("CWE484_Omitted_Break_Statement__basic_07");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_08()).runTest("CWE484_Omitted_Break_Statement__basic_08");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_09()).runTest("CWE484_Omitted_Break_Statement__basic_09");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_10()).runTest("CWE484_Omitted_Break_Statement__basic_10");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_11()).runTest("CWE484_Omitted_Break_Statement__basic_11");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_12()).runTest("CWE484_Omitted_Break_Statement__basic_12");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_13()).runTest("CWE484_Omitted_Break_Statement__basic_13");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_14()).runTest("CWE484_Omitted_Break_Statement__basic_14");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_15()).runTest("CWE484_Omitted_Break_Statement__basic_15");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_16()).runTest("CWE484_Omitted_Break_Statement__basic_16");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_17()).runTest("CWE484_Omitted_Break_Statement__basic_17");
			(new testcases.CWE484_Omitted_Break_Statement.CWE484_Omitted_Break_Statement__basic_19()).runTest("CWE484_Omitted_Break_Statement__basic_19");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_01()).runTest("CWE486_Compare_Classes_By_Name__basic_01");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_02()).runTest("CWE486_Compare_Classes_By_Name__basic_02");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_03()).runTest("CWE486_Compare_Classes_By_Name__basic_03");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_04()).runTest("CWE486_Compare_Classes_By_Name__basic_04");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_05()).runTest("CWE486_Compare_Classes_By_Name__basic_05");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_06()).runTest("CWE486_Compare_Classes_By_Name__basic_06");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_07()).runTest("CWE486_Compare_Classes_By_Name__basic_07");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_08()).runTest("CWE486_Compare_Classes_By_Name__basic_08");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_09()).runTest("CWE486_Compare_Classes_By_Name__basic_09");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_10()).runTest("CWE486_Compare_Classes_By_Name__basic_10");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_11()).runTest("CWE486_Compare_Classes_By_Name__basic_11");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_12()).runTest("CWE486_Compare_Classes_By_Name__basic_12");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_13()).runTest("CWE486_Compare_Classes_By_Name__basic_13");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_14()).runTest("CWE486_Compare_Classes_By_Name__basic_14");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_15()).runTest("CWE486_Compare_Classes_By_Name__basic_15");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_16()).runTest("CWE486_Compare_Classes_By_Name__basic_16");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_17()).runTest("CWE486_Compare_Classes_By_Name__basic_17");
			(new testcases.CWE486_Compare_Classes_By_Name.CWE486_Compare_Classes_By_Name__basic_19()).runTest("CWE486_Compare_Classes_By_Name__basic_19");
			(new testcases.CWE491_Object_Hijack.CWE491_Object_Hijack__basic_01_bad()).runTest("CWE491_Object_Hijack__basic_01_bad");
			(new testcases.CWE491_Object_Hijack.CWE491_Object_Hijack__basic_01_good1()).runTest("CWE491_Object_Hijack__basic_01_good1");
			(new testcases.CWE499_Sensitive_Data_Serializable.CWE499_Sensitive_Data_Serializable__writeObject_01_bad()).runTest("CWE499_Sensitive_Data_Serializable__writeObject_01_bad");
			(new testcases.CWE499_Sensitive_Data_Serializable.CWE499_Sensitive_Data_Serializable__writeObject_01_good1()).runTest("CWE499_Sensitive_Data_Serializable__writeObject_01_good1");
			/* END-AUTOGENERATED-JAVA-TESTS-4 */
}

private static void runTestCWE5() {
/* BEGIN-AUTOGENERATED-JAVA-TESTS-5 */
			(new testcases.CWE500_Static_Field_Not_Final.CWE500_Static_Field_Not_Final__String_01_bad()).runTest("CWE500_Static_Field_Not_Final__String_01_bad");
			(new testcases.CWE500_Static_Field_Not_Final.CWE500_Static_Field_Not_Final__String_01_good1()).runTest("CWE500_Static_Field_Not_Final__String_01_good1");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_01()).runTest("CWE511_Logic_Time_Bomb__basic_01");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_02()).runTest("CWE511_Logic_Time_Bomb__basic_02");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_03()).runTest("CWE511_Logic_Time_Bomb__basic_03");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_04()).runTest("CWE511_Logic_Time_Bomb__basic_04");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_05()).runTest("CWE511_Logic_Time_Bomb__basic_05");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_06()).runTest("CWE511_Logic_Time_Bomb__basic_06");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_07()).runTest("CWE511_Logic_Time_Bomb__basic_07");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_08()).runTest("CWE511_Logic_Time_Bomb__basic_08");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_09()).runTest("CWE511_Logic_Time_Bomb__basic_09");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_10()).runTest("CWE511_Logic_Time_Bomb__basic_10");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_11()).runTest("CWE511_Logic_Time_Bomb__basic_11");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_12()).runTest("CWE511_Logic_Time_Bomb__basic_12");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_13()).runTest("CWE511_Logic_Time_Bomb__basic_13");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_14()).runTest("CWE511_Logic_Time_Bomb__basic_14");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_15()).runTest("CWE511_Logic_Time_Bomb__basic_15");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_16()).runTest("CWE511_Logic_Time_Bomb__basic_16");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_17()).runTest("CWE511_Logic_Time_Bomb__basic_17");
			(new testcases.CWE511_Logic_Time_Bomb.CWE511_Logic_Time_Bomb__basic_19()).runTest("CWE511_Logic_Time_Bomb__basic_19");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_01()).runTest("CWE546_Suspicious_Comment__bug_01");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_02()).runTest("CWE546_Suspicious_Comment__bug_02");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_03()).runTest("CWE546_Suspicious_Comment__bug_03");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_04()).runTest("CWE546_Suspicious_Comment__bug_04");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_05()).runTest("CWE546_Suspicious_Comment__bug_05");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_06()).runTest("CWE546_Suspicious_Comment__bug_06");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_07()).runTest("CWE546_Suspicious_Comment__bug_07");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_08()).runTest("CWE546_Suspicious_Comment__bug_08");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_09()).runTest("CWE546_Suspicious_Comment__bug_09");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_10()).runTest("CWE546_Suspicious_Comment__bug_10");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_11()).runTest("CWE546_Suspicious_Comment__bug_11");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_12()).runTest("CWE546_Suspicious_Comment__bug_12");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_13()).runTest("CWE546_Suspicious_Comment__bug_13");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_14()).runTest("CWE546_Suspicious_Comment__bug_14");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_15()).runTest("CWE546_Suspicious_Comment__bug_15");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_16()).runTest("CWE546_Suspicious_Comment__bug_16");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_17()).runTest("CWE546_Suspicious_Comment__bug_17");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__bug_19()).runTest("CWE546_Suspicious_Comment__bug_19");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_01()).runTest("CWE546_Suspicious_Comment__fixme_01");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_02()).runTest("CWE546_Suspicious_Comment__fixme_02");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_03()).runTest("CWE546_Suspicious_Comment__fixme_03");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_04()).runTest("CWE546_Suspicious_Comment__fixme_04");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_05()).runTest("CWE546_Suspicious_Comment__fixme_05");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_06()).runTest("CWE546_Suspicious_Comment__fixme_06");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_07()).runTest("CWE546_Suspicious_Comment__fixme_07");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_08()).runTest("CWE546_Suspicious_Comment__fixme_08");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_09()).runTest("CWE546_Suspicious_Comment__fixme_09");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_10()).runTest("CWE546_Suspicious_Comment__fixme_10");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_11()).runTest("CWE546_Suspicious_Comment__fixme_11");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_12()).runTest("CWE546_Suspicious_Comment__fixme_12");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_13()).runTest("CWE546_Suspicious_Comment__fixme_13");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_14()).runTest("CWE546_Suspicious_Comment__fixme_14");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_15()).runTest("CWE546_Suspicious_Comment__fixme_15");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_16()).runTest("CWE546_Suspicious_Comment__fixme_16");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_17()).runTest("CWE546_Suspicious_Comment__fixme_17");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__fixme_19()).runTest("CWE546_Suspicious_Comment__fixme_19");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_01()).runTest("CWE546_Suspicious_Comment__hack_01");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_02()).runTest("CWE546_Suspicious_Comment__hack_02");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_03()).runTest("CWE546_Suspicious_Comment__hack_03");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_04()).runTest("CWE546_Suspicious_Comment__hack_04");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_05()).runTest("CWE546_Suspicious_Comment__hack_05");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_06()).runTest("CWE546_Suspicious_Comment__hack_06");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_07()).runTest("CWE546_Suspicious_Comment__hack_07");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_08()).runTest("CWE546_Suspicious_Comment__hack_08");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_09()).runTest("CWE546_Suspicious_Comment__hack_09");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_10()).runTest("CWE546_Suspicious_Comment__hack_10");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_11()).runTest("CWE546_Suspicious_Comment__hack_11");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_12()).runTest("CWE546_Suspicious_Comment__hack_12");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_13()).runTest("CWE546_Suspicious_Comment__hack_13");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_14()).runTest("CWE546_Suspicious_Comment__hack_14");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_15()).runTest("CWE546_Suspicious_Comment__hack_15");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_16()).runTest("CWE546_Suspicious_Comment__hack_16");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_17()).runTest("CWE546_Suspicious_Comment__hack_17");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__hack_19()).runTest("CWE546_Suspicious_Comment__hack_19");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_01()).runTest("CWE546_Suspicious_Comment__later_01");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_02()).runTest("CWE546_Suspicious_Comment__later_02");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_03()).runTest("CWE546_Suspicious_Comment__later_03");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_04()).runTest("CWE546_Suspicious_Comment__later_04");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_05()).runTest("CWE546_Suspicious_Comment__later_05");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_06()).runTest("CWE546_Suspicious_Comment__later_06");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_07()).runTest("CWE546_Suspicious_Comment__later_07");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_08()).runTest("CWE546_Suspicious_Comment__later_08");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_09()).runTest("CWE546_Suspicious_Comment__later_09");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_10()).runTest("CWE546_Suspicious_Comment__later_10");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_11()).runTest("CWE546_Suspicious_Comment__later_11");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_12()).runTest("CWE546_Suspicious_Comment__later_12");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_13()).runTest("CWE546_Suspicious_Comment__later_13");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_14()).runTest("CWE546_Suspicious_Comment__later_14");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_15()).runTest("CWE546_Suspicious_Comment__later_15");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_16()).runTest("CWE546_Suspicious_Comment__later_16");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_17()).runTest("CWE546_Suspicious_Comment__later_17");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__later_19()).runTest("CWE546_Suspicious_Comment__later_19");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_01()).runTest("CWE546_Suspicious_Comment__todo_01");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_02()).runTest("CWE546_Suspicious_Comment__todo_02");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_03()).runTest("CWE546_Suspicious_Comment__todo_03");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_04()).runTest("CWE546_Suspicious_Comment__todo_04");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_05()).runTest("CWE546_Suspicious_Comment__todo_05");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_06()).runTest("CWE546_Suspicious_Comment__todo_06");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_07()).runTest("CWE546_Suspicious_Comment__todo_07");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_08()).runTest("CWE546_Suspicious_Comment__todo_08");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_09()).runTest("CWE546_Suspicious_Comment__todo_09");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_10()).runTest("CWE546_Suspicious_Comment__todo_10");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_11()).runTest("CWE546_Suspicious_Comment__todo_11");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_12()).runTest("CWE546_Suspicious_Comment__todo_12");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_13()).runTest("CWE546_Suspicious_Comment__todo_13");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_14()).runTest("CWE546_Suspicious_Comment__todo_14");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_15()).runTest("CWE546_Suspicious_Comment__todo_15");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_16()).runTest("CWE546_Suspicious_Comment__todo_16");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_17()).runTest("CWE546_Suspicious_Comment__todo_17");
			(new testcases.CWE546_Suspicious_Comment.CWE546_Suspicious_Comment__todo_19()).runTest("CWE546_Suspicious_Comment__todo_19");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_01()).runTest("CWE547_Hardcoded_Security_Constants__basic_01");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_02()).runTest("CWE547_Hardcoded_Security_Constants__basic_02");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_03()).runTest("CWE547_Hardcoded_Security_Constants__basic_03");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_04()).runTest("CWE547_Hardcoded_Security_Constants__basic_04");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_05()).runTest("CWE547_Hardcoded_Security_Constants__basic_05");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_06()).runTest("CWE547_Hardcoded_Security_Constants__basic_06");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_07()).runTest("CWE547_Hardcoded_Security_Constants__basic_07");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_08()).runTest("CWE547_Hardcoded_Security_Constants__basic_08");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_09()).runTest("CWE547_Hardcoded_Security_Constants__basic_09");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_10()).runTest("CWE547_Hardcoded_Security_Constants__basic_10");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_11()).runTest("CWE547_Hardcoded_Security_Constants__basic_11");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_12()).runTest("CWE547_Hardcoded_Security_Constants__basic_12");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_13()).runTest("CWE547_Hardcoded_Security_Constants__basic_13");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_14()).runTest("CWE547_Hardcoded_Security_Constants__basic_14");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_15()).runTest("CWE547_Hardcoded_Security_Constants__basic_15");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_16()).runTest("CWE547_Hardcoded_Security_Constants__basic_16");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_17()).runTest("CWE547_Hardcoded_Security_Constants__basic_17");
			(new testcases.CWE547_Hardcoded_Security_Constants.CWE547_Hardcoded_Security_Constants__basic_19()).runTest("CWE547_Hardcoded_Security_Constants__basic_19");
			(new testcases.CWE561_Dead_Code.CWE561_Dead_Code__unused_method_01_bad()).runTest("CWE561_Dead_Code__unused_method_01_bad");
			(new testcases.CWE561_Dead_Code.CWE561_Dead_Code__unused_method_01_good1()).runTest("CWE561_Dead_Code__unused_method_01_good1");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_01()).runTest("CWE563_Unused_Variable__unused_init_variable_int_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_02()).runTest("CWE563_Unused_Variable__unused_init_variable_int_02");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_03()).runTest("CWE563_Unused_Variable__unused_init_variable_int_03");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_04()).runTest("CWE563_Unused_Variable__unused_init_variable_int_04");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_05()).runTest("CWE563_Unused_Variable__unused_init_variable_int_05");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_06()).runTest("CWE563_Unused_Variable__unused_init_variable_int_06");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_07()).runTest("CWE563_Unused_Variable__unused_init_variable_int_07");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_08()).runTest("CWE563_Unused_Variable__unused_init_variable_int_08");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_09()).runTest("CWE563_Unused_Variable__unused_init_variable_int_09");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_10()).runTest("CWE563_Unused_Variable__unused_init_variable_int_10");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_11()).runTest("CWE563_Unused_Variable__unused_init_variable_int_11");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_13()).runTest("CWE563_Unused_Variable__unused_init_variable_int_13");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_14()).runTest("CWE563_Unused_Variable__unused_init_variable_int_14");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_15()).runTest("CWE563_Unused_Variable__unused_init_variable_int_15");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_16()).runTest("CWE563_Unused_Variable__unused_init_variable_int_16");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_int_17()).runTest("CWE563_Unused_Variable__unused_init_variable_int_17");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_01()).runTest("CWE563_Unused_Variable__unused_init_variable_long_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_02()).runTest("CWE563_Unused_Variable__unused_init_variable_long_02");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_03()).runTest("CWE563_Unused_Variable__unused_init_variable_long_03");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_04()).runTest("CWE563_Unused_Variable__unused_init_variable_long_04");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_05()).runTest("CWE563_Unused_Variable__unused_init_variable_long_05");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_06()).runTest("CWE563_Unused_Variable__unused_init_variable_long_06");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_07()).runTest("CWE563_Unused_Variable__unused_init_variable_long_07");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_08()).runTest("CWE563_Unused_Variable__unused_init_variable_long_08");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_09()).runTest("CWE563_Unused_Variable__unused_init_variable_long_09");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_10()).runTest("CWE563_Unused_Variable__unused_init_variable_long_10");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_11()).runTest("CWE563_Unused_Variable__unused_init_variable_long_11");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_13()).runTest("CWE563_Unused_Variable__unused_init_variable_long_13");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_14()).runTest("CWE563_Unused_Variable__unused_init_variable_long_14");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_15()).runTest("CWE563_Unused_Variable__unused_init_variable_long_15");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_16()).runTest("CWE563_Unused_Variable__unused_init_variable_long_16");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_long_17()).runTest("CWE563_Unused_Variable__unused_init_variable_long_17");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_01()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_02()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_02");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_03()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_03");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_04()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_04");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_05()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_05");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_06()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_06");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_07()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_07");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_08()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_08");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_09()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_09");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_10()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_10");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_11()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_11");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_13()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_13");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_14()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_14");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_15()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_15");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_16()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_16");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_StringBuilder_17()).runTest("CWE563_Unused_Variable__unused_init_variable_StringBuilder_17");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_01()).runTest("CWE563_Unused_Variable__unused_init_variable_String_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_02()).runTest("CWE563_Unused_Variable__unused_init_variable_String_02");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_03()).runTest("CWE563_Unused_Variable__unused_init_variable_String_03");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_04()).runTest("CWE563_Unused_Variable__unused_init_variable_String_04");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_05()).runTest("CWE563_Unused_Variable__unused_init_variable_String_05");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_06()).runTest("CWE563_Unused_Variable__unused_init_variable_String_06");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_07()).runTest("CWE563_Unused_Variable__unused_init_variable_String_07");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_08()).runTest("CWE563_Unused_Variable__unused_init_variable_String_08");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_09()).runTest("CWE563_Unused_Variable__unused_init_variable_String_09");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_10()).runTest("CWE563_Unused_Variable__unused_init_variable_String_10");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_11()).runTest("CWE563_Unused_Variable__unused_init_variable_String_11");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_13()).runTest("CWE563_Unused_Variable__unused_init_variable_String_13");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_14()).runTest("CWE563_Unused_Variable__unused_init_variable_String_14");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_15()).runTest("CWE563_Unused_Variable__unused_init_variable_String_15");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_16()).runTest("CWE563_Unused_Variable__unused_init_variable_String_16");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_init_variable_String_17()).runTest("CWE563_Unused_Variable__unused_init_variable_String_17");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_parameter_value_01()).runTest("CWE563_Unused_Variable__unused_parameter_value_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_parameter_variable_01()).runTest("CWE563_Unused_Variable__unused_parameter_variable_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_private_member_value_01_bad()).runTest("CWE563_Unused_Variable__unused_private_member_value_01_bad");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_private_member_value_01_good1()).runTest("CWE563_Unused_Variable__unused_private_member_value_01_good1");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_private_member_variable_01_bad()).runTest("CWE563_Unused_Variable__unused_private_member_variable_01_bad");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_private_member_variable_01_good1()).runTest("CWE563_Unused_Variable__unused_private_member_variable_01_good1");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_public_member_value_01_bad()).runTest("CWE563_Unused_Variable__unused_public_member_value_01_bad");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_public_member_value_01_good1()).runTest("CWE563_Unused_Variable__unused_public_member_value_01_good1");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_public_member_variable_01_bad()).runTest("CWE563_Unused_Variable__unused_public_member_variable_01_bad");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_public_member_variable_01_good1()).runTest("CWE563_Unused_Variable__unused_public_member_variable_01_good1");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_01()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_02()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_02");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_03()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_03");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_04()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_04");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_05()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_05");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_06()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_06");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_07()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_07");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_08()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_08");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_09()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_09");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_10()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_10");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_11()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_11");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_13()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_13");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_14()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_14");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_15()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_15");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_16()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_16");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_int_17()).runTest("CWE563_Unused_Variable__unused_uninit_variable_int_17");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_01()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_02()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_02");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_03()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_03");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_04()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_04");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_05()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_05");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_06()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_06");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_07()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_07");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_08()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_08");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_09()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_09");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_10()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_10");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_11()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_11");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_13()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_13");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_14()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_14");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_15()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_15");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_16()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_16");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_long_17()).runTest("CWE563_Unused_Variable__unused_uninit_variable_long_17");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_01()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_02()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_02");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_03()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_03");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_04()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_04");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_05()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_05");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_06()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_06");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_07()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_07");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_08()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_08");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_09()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_09");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_10()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_10");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_11()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_11");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_13()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_13");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_14()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_14");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_15()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_15");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_16()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_16");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_17()).runTest("CWE563_Unused_Variable__unused_uninit_variable_StringBuilder_17");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_01()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_02()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_02");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_03()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_03");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_04()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_04");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_05()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_05");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_06()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_06");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_07()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_07");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_08()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_08");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_09()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_09");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_10()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_10");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_11()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_11");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_13()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_13");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_14()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_14");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_15()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_15");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_16()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_16");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_uninit_variable_String_17()).runTest("CWE563_Unused_Variable__unused_uninit_variable_String_17");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_01()).runTest("CWE563_Unused_Variable__unused_value_int_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_02()).runTest("CWE563_Unused_Variable__unused_value_int_02");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_03()).runTest("CWE563_Unused_Variable__unused_value_int_03");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_04()).runTest("CWE563_Unused_Variable__unused_value_int_04");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_05()).runTest("CWE563_Unused_Variable__unused_value_int_05");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_06()).runTest("CWE563_Unused_Variable__unused_value_int_06");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_07()).runTest("CWE563_Unused_Variable__unused_value_int_07");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_08()).runTest("CWE563_Unused_Variable__unused_value_int_08");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_09()).runTest("CWE563_Unused_Variable__unused_value_int_09");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_10()).runTest("CWE563_Unused_Variable__unused_value_int_10");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_11()).runTest("CWE563_Unused_Variable__unused_value_int_11");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_13()).runTest("CWE563_Unused_Variable__unused_value_int_13");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_14()).runTest("CWE563_Unused_Variable__unused_value_int_14");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_15()).runTest("CWE563_Unused_Variable__unused_value_int_15");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_16()).runTest("CWE563_Unused_Variable__unused_value_int_16");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_int_17()).runTest("CWE563_Unused_Variable__unused_value_int_17");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_01()).runTest("CWE563_Unused_Variable__unused_value_long_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_02()).runTest("CWE563_Unused_Variable__unused_value_long_02");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_03()).runTest("CWE563_Unused_Variable__unused_value_long_03");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_04()).runTest("CWE563_Unused_Variable__unused_value_long_04");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_05()).runTest("CWE563_Unused_Variable__unused_value_long_05");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_06()).runTest("CWE563_Unused_Variable__unused_value_long_06");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_07()).runTest("CWE563_Unused_Variable__unused_value_long_07");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_08()).runTest("CWE563_Unused_Variable__unused_value_long_08");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_09()).runTest("CWE563_Unused_Variable__unused_value_long_09");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_10()).runTest("CWE563_Unused_Variable__unused_value_long_10");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_11()).runTest("CWE563_Unused_Variable__unused_value_long_11");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_13()).runTest("CWE563_Unused_Variable__unused_value_long_13");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_14()).runTest("CWE563_Unused_Variable__unused_value_long_14");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_15()).runTest("CWE563_Unused_Variable__unused_value_long_15");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_16()).runTest("CWE563_Unused_Variable__unused_value_long_16");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_long_17()).runTest("CWE563_Unused_Variable__unused_value_long_17");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_01()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_02()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_02");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_03()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_03");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_04()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_04");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_05()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_05");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_06()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_06");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_07()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_07");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_08()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_08");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_09()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_09");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_10()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_10");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_11()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_11");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_13()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_13");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_14()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_14");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_15()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_15");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_16()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_16");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_StringBuilder_17()).runTest("CWE563_Unused_Variable__unused_value_StringBuilder_17");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_01()).runTest("CWE563_Unused_Variable__unused_value_String_01");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_02()).runTest("CWE563_Unused_Variable__unused_value_String_02");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_03()).runTest("CWE563_Unused_Variable__unused_value_String_03");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_04()).runTest("CWE563_Unused_Variable__unused_value_String_04");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_05()).runTest("CWE563_Unused_Variable__unused_value_String_05");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_06()).runTest("CWE563_Unused_Variable__unused_value_String_06");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_07()).runTest("CWE563_Unused_Variable__unused_value_String_07");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_08()).runTest("CWE563_Unused_Variable__unused_value_String_08");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_09()).runTest("CWE563_Unused_Variable__unused_value_String_09");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_10()).runTest("CWE563_Unused_Variable__unused_value_String_10");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_11()).runTest("CWE563_Unused_Variable__unused_value_String_11");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_13()).runTest("CWE563_Unused_Variable__unused_value_String_13");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_14()).runTest("CWE563_Unused_Variable__unused_value_String_14");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_15()).runTest("CWE563_Unused_Variable__unused_value_String_15");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_16()).runTest("CWE563_Unused_Variable__unused_value_String_16");
			(new testcases.CWE563_Unused_Variable.CWE563_Unused_Variable__unused_value_String_17()).runTest("CWE563_Unused_Variable__unused_value_String_17");
			(new testcases.CWE568_finalize_Method_Without_superfinalize.CWE568_finalize_Method_Without_superfinalize__from_console_01_bad()).runTest("CWE568_finalize_Method_Without_superfinalize__from_console_01_bad");
			(new testcases.CWE568_finalize_Method_Without_superfinalize.CWE568_finalize_Method_Without_superfinalize__from_console_01_good1()).runTest("CWE568_finalize_Method_Without_superfinalize__from_console_01_good1");
			(new testcases.CWE570_Expression_Always_False.CWE570_Expression_Always_False__class_getClass_equal_01()).runTest("CWE570_Expression_Always_False__class_getClass_equal_01");
			(new testcases.CWE570_Expression_Always_False.CWE570_Expression_Always_False__false_01()).runTest("CWE570_Expression_Always_False__false_01");
			(new testcases.CWE570_Expression_Always_False.CWE570_Expression_Always_False__n_equal_n_minus_one_01()).runTest("CWE570_Expression_Always_False__n_equal_n_minus_one_01");
			(new testcases.CWE570_Expression_Always_False.CWE570_Expression_Always_False__n_less_int_min_01()).runTest("CWE570_Expression_Always_False__n_less_int_min_01");
			(new testcases.CWE570_Expression_Always_False.CWE570_Expression_Always_False__string_equals_01()).runTest("CWE570_Expression_Always_False__string_equals_01");
			(new testcases.CWE570_Expression_Always_False.CWE570_Expression_Always_False__two_equals_three_01()).runTest("CWE570_Expression_Always_False__two_equals_three_01");
			(new testcases.CWE571_Expression_Always_True.CWE571_Expression_Always_True__class_getClass_not_equal_01()).runTest("CWE571_Expression_Always_True__class_getClass_not_equal_01");
			(new testcases.CWE571_Expression_Always_True.CWE571_Expression_Always_True__n_equal_m_minus_one_01()).runTest("CWE571_Expression_Always_True__n_equal_m_minus_one_01");
			(new testcases.CWE571_Expression_Always_True.CWE571_Expression_Always_True__n_less_int_max_01()).runTest("CWE571_Expression_Always_True__n_less_int_max_01");
			(new testcases.CWE571_Expression_Always_True.CWE571_Expression_Always_True__string_equals_01()).runTest("CWE571_Expression_Always_True__string_equals_01");
			(new testcases.CWE571_Expression_Always_True.CWE571_Expression_Always_True__true_01()).runTest("CWE571_Expression_Always_True__true_01");
			(new testcases.CWE571_Expression_Always_True.CWE571_Expression_Always_True__two_equals_two_01()).runTest("CWE571_Expression_Always_True__two_equals_two_01");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_01()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_01");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_02()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_02");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_03()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_03");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_04()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_04");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_05()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_05");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_06()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_06");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_07()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_07");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_08()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_08");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_09()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_09");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_10()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_10");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_11()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_11");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_12()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_12");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_13()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_13");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_14()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_14");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_15()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_15");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_16()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_16");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_17()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_17");
			(new testcases.CWE572_Call_to_Thread_run_Instead_of_start.CWE572_Call_to_Thread_run_Instead_of_start__basic_19()).runTest("CWE572_Call_to_Thread_run_Instead_of_start__basic_19");
			(new testcases.CWE580_Clone_Without_Super.CWE580_Clone_Without_Super__clone_01_bad()).runTest("CWE580_Clone_Without_Super__clone_01_bad");
			(new testcases.CWE580_Clone_Without_Super.CWE580_Clone_Without_Super__clone_01_good1()).runTest("CWE580_Clone_Without_Super__clone_01_good1");
			(new testcases.CWE581_Object_Model_Violation.CWE581_Object_Model_Violation__hashCode_01_bad()).runTest("CWE581_Object_Model_Violation__hashCode_01_bad");
			(new testcases.CWE581_Object_Model_Violation.CWE581_Object_Model_Violation__hashCode_01_good1()).runTest("CWE581_Object_Model_Violation__hashCode_01_good1");
			(new testcases.CWE582_Array_Public_Final_Static.CWE582_Array_Public_Final_Static__basic_01_bad()).runTest("CWE582_Array_Public_Final_Static__basic_01_bad");
			(new testcases.CWE582_Array_Public_Final_Static.CWE582_Array_Public_Final_Static__basic_01_good1()).runTest("CWE582_Array_Public_Final_Static__basic_01_good1");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_01()).runTest("CWE584_Return_in_Finally__basic_01");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_02()).runTest("CWE584_Return_in_Finally__basic_02");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_03()).runTest("CWE584_Return_in_Finally__basic_03");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_04()).runTest("CWE584_Return_in_Finally__basic_04");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_05()).runTest("CWE584_Return_in_Finally__basic_05");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_06()).runTest("CWE584_Return_in_Finally__basic_06");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_07()).runTest("CWE584_Return_in_Finally__basic_07");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_08()).runTest("CWE584_Return_in_Finally__basic_08");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_09()).runTest("CWE584_Return_in_Finally__basic_09");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_10()).runTest("CWE584_Return_in_Finally__basic_10");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_11()).runTest("CWE584_Return_in_Finally__basic_11");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_12()).runTest("CWE584_Return_in_Finally__basic_12");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_13()).runTest("CWE584_Return_in_Finally__basic_13");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_14()).runTest("CWE584_Return_in_Finally__basic_14");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_15()).runTest("CWE584_Return_in_Finally__basic_15");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_16()).runTest("CWE584_Return_in_Finally__basic_16");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_17()).runTest("CWE584_Return_in_Finally__basic_17");
			(new testcases.CWE584_Return_in_Finally.CWE584_Return_in_Finally__basic_19()).runTest("CWE584_Return_in_Finally__basic_19");
			(new testcases.CWE585_Empty_Sync_Block.CWE585_Empty_Sync_Block__console_01()).runTest("CWE585_Empty_Sync_Block__console_01");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_01()).runTest("CWE586_Explicit_Call_to_Finalize__String_01");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_02()).runTest("CWE586_Explicit_Call_to_Finalize__String_02");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_03()).runTest("CWE586_Explicit_Call_to_Finalize__String_03");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_04()).runTest("CWE586_Explicit_Call_to_Finalize__String_04");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_05()).runTest("CWE586_Explicit_Call_to_Finalize__String_05");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_06()).runTest("CWE586_Explicit_Call_to_Finalize__String_06");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_07()).runTest("CWE586_Explicit_Call_to_Finalize__String_07");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_08()).runTest("CWE586_Explicit_Call_to_Finalize__String_08");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_09()).runTest("CWE586_Explicit_Call_to_Finalize__String_09");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_10()).runTest("CWE586_Explicit_Call_to_Finalize__String_10");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_11()).runTest("CWE586_Explicit_Call_to_Finalize__String_11");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_12()).runTest("CWE586_Explicit_Call_to_Finalize__String_12");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_13()).runTest("CWE586_Explicit_Call_to_Finalize__String_13");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_14()).runTest("CWE586_Explicit_Call_to_Finalize__String_14");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_15()).runTest("CWE586_Explicit_Call_to_Finalize__String_15");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_16()).runTest("CWE586_Explicit_Call_to_Finalize__String_16");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_17()).runTest("CWE586_Explicit_Call_to_Finalize__String_17");
			(new testcases.CWE586_Explicit_Call_to_Finalize.CWE586_Explicit_Call_to_Finalize__String_19()).runTest("CWE586_Explicit_Call_to_Finalize__String_19");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_01()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_01");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_02()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_02");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_03()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_03");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_04()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_04");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_05()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_05");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_06()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_06");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_07()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_07");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_08()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_08");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_09()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_09");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_10()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_10");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_11()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_11");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_12()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_12");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_13()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_13");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_14()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_14");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_15()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_15");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_16()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_16");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_17()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_17");
			(new testcases.CWE597_Wrong_Operator_String_Comparison.CWE597_Wrong_Operator_String_Comparison__basic_19()).runTest("CWE597_Wrong_Operator_String_Comparison__basic_19");
			/* END-AUTOGENERATED-JAVA-TESTS-5 */
}

private static void runTestCWE6() {
/* BEGIN-AUTOGENERATED-JAVA-TESTS-6 */
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_01()).runTest("CWE605_Multiple_Binds_Same_Port__basic_01");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_02()).runTest("CWE605_Multiple_Binds_Same_Port__basic_02");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_03()).runTest("CWE605_Multiple_Binds_Same_Port__basic_03");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_04()).runTest("CWE605_Multiple_Binds_Same_Port__basic_04");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_05()).runTest("CWE605_Multiple_Binds_Same_Port__basic_05");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_06()).runTest("CWE605_Multiple_Binds_Same_Port__basic_06");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_07()).runTest("CWE605_Multiple_Binds_Same_Port__basic_07");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_08()).runTest("CWE605_Multiple_Binds_Same_Port__basic_08");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_09()).runTest("CWE605_Multiple_Binds_Same_Port__basic_09");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_10()).runTest("CWE605_Multiple_Binds_Same_Port__basic_10");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_11()).runTest("CWE605_Multiple_Binds_Same_Port__basic_11");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_12()).runTest("CWE605_Multiple_Binds_Same_Port__basic_12");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_13()).runTest("CWE605_Multiple_Binds_Same_Port__basic_13");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_14()).runTest("CWE605_Multiple_Binds_Same_Port__basic_14");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_15()).runTest("CWE605_Multiple_Binds_Same_Port__basic_15");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_16()).runTest("CWE605_Multiple_Binds_Same_Port__basic_16");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_17()).runTest("CWE605_Multiple_Binds_Same_Port__basic_17");
			(new testcases.CWE605_Multiple_Binds_Same_Port.CWE605_Multiple_Binds_Same_Port__basic_19()).runTest("CWE605_Multiple_Binds_Same_Port__basic_19");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_01()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_01");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_02()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_02");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_03()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_03");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_04()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_04");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_05()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_05");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_06()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_06");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_07()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_07");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_08()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_08");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_09()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_09");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_10()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_10");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_11()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_11");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_12()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_12");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_13()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_13");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_14()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_14");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_15()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_15");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_16()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_16");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_17()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_17");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_19()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_19");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_31()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_31");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_41()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_41");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_42()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_42");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_45()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_45");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_51a()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_51a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_52a()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_52a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_53a()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_53a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_54a()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_54a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_61a()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_61a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_66a()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_66a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_67a()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_67a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_68a()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_68a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__connect_tcp_71a()).runTest("CWE606_Unchecked_Loop_Condition__connect_tcp_71a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_01()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_01");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_02()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_02");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_03()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_03");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_04()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_04");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_05()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_05");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_06()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_06");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_07()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_07");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_08()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_08");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_09()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_09");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_10()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_10");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_11()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_11");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_12()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_12");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_13()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_13");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_14()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_14");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_15()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_15");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_16()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_16");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_17()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_17");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_19()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_19");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_31()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_31");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_41()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_41");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_42()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_42");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_45()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_45");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_51a()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_51a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_52a()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_52a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_53a()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_53a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_54a()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_54a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_61a()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_61a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_66a()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_66a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_67a()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_67a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_68a()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_68a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__console_readLine_71a()).runTest("CWE606_Unchecked_Loop_Condition__console_readLine_71a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_01()).runTest("CWE606_Unchecked_Loop_Condition__Environment_01");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_02()).runTest("CWE606_Unchecked_Loop_Condition__Environment_02");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_03()).runTest("CWE606_Unchecked_Loop_Condition__Environment_03");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_04()).runTest("CWE606_Unchecked_Loop_Condition__Environment_04");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_05()).runTest("CWE606_Unchecked_Loop_Condition__Environment_05");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_06()).runTest("CWE606_Unchecked_Loop_Condition__Environment_06");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_07()).runTest("CWE606_Unchecked_Loop_Condition__Environment_07");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_08()).runTest("CWE606_Unchecked_Loop_Condition__Environment_08");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_09()).runTest("CWE606_Unchecked_Loop_Condition__Environment_09");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_10()).runTest("CWE606_Unchecked_Loop_Condition__Environment_10");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_11()).runTest("CWE606_Unchecked_Loop_Condition__Environment_11");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_12()).runTest("CWE606_Unchecked_Loop_Condition__Environment_12");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_13()).runTest("CWE606_Unchecked_Loop_Condition__Environment_13");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_14()).runTest("CWE606_Unchecked_Loop_Condition__Environment_14");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_15()).runTest("CWE606_Unchecked_Loop_Condition__Environment_15");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_16()).runTest("CWE606_Unchecked_Loop_Condition__Environment_16");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_17()).runTest("CWE606_Unchecked_Loop_Condition__Environment_17");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_19()).runTest("CWE606_Unchecked_Loop_Condition__Environment_19");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_31()).runTest("CWE606_Unchecked_Loop_Condition__Environment_31");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_41()).runTest("CWE606_Unchecked_Loop_Condition__Environment_41");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_42()).runTest("CWE606_Unchecked_Loop_Condition__Environment_42");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_45()).runTest("CWE606_Unchecked_Loop_Condition__Environment_45");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_51a()).runTest("CWE606_Unchecked_Loop_Condition__Environment_51a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_52a()).runTest("CWE606_Unchecked_Loop_Condition__Environment_52a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_53a()).runTest("CWE606_Unchecked_Loop_Condition__Environment_53a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_54a()).runTest("CWE606_Unchecked_Loop_Condition__Environment_54a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_61a()).runTest("CWE606_Unchecked_Loop_Condition__Environment_61a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_66a()).runTest("CWE606_Unchecked_Loop_Condition__Environment_66a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_67a()).runTest("CWE606_Unchecked_Loop_Condition__Environment_67a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_68a()).runTest("CWE606_Unchecked_Loop_Condition__Environment_68a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__Environment_71a()).runTest("CWE606_Unchecked_Loop_Condition__Environment_71a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_01()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_01");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_02()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_02");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_03()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_03");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_04()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_04");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_05()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_05");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_06()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_06");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_07()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_07");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_08()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_08");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_09()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_09");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_10()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_10");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_11()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_11");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_12()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_12");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_13()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_13");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_14()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_14");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_15()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_15");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_16()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_16");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_17()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_17");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_19()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_19");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_31()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_31");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_41()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_41");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_42()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_42");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_45()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_45");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_51a()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_51a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_52a()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_52a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_53a()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_53a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_54a()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_54a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_61a()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_61a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_66a()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_66a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_67a()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_67a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_68a()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_68a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromDB_71a()).runTest("CWE606_Unchecked_Loop_Condition__fromDB_71a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_01()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_01");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_02()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_02");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_03()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_03");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_04()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_04");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_05()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_05");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_06()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_06");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_07()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_07");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_08()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_08");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_09()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_09");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_10()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_10");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_11()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_11");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_12()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_12");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_13()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_13");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_14()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_14");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_15()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_15");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_16()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_16");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_17()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_17");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_19()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_19");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_31()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_31");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_41()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_41");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_42()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_42");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_45()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_45");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_51a()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_51a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_52a()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_52a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_53a()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_53a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_54a()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_54a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_61a()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_61a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_66a()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_66a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_67a()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_67a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_68a()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_68a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__fromFile_71a()).runTest("CWE606_Unchecked_Loop_Condition__fromFile_71a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_01()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_01");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_02()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_02");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_03()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_03");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_04()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_04");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_05()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_05");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_06()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_06");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_07()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_07");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_08()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_08");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_09()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_09");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_10()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_10");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_11()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_11");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_12()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_12");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_13()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_13");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_14()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_14");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_15()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_15");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_16()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_16");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_17()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_17");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_19()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_19");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_31()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_31");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_41()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_41");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_42()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_42");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_45()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_45");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_51a()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_51a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_52a()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_52a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_53a()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_53a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_54a()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_54a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_61a()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_61a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_66a()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_66a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_67a()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_67a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_68a()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_68a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__listen_tcp_71a()).runTest("CWE606_Unchecked_Loop_Condition__listen_tcp_71a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_01()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_01");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_02()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_02");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_03()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_03");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_04()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_04");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_05()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_05");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_06()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_06");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_07()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_07");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_08()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_08");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_09()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_09");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_10()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_10");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_11()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_11");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_12()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_12");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_13()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_13");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_14()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_14");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_15()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_15");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_16()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_16");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_17()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_17");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_19()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_19");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_31()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_31");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_41()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_41");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_42()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_42");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_45()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_45");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_51a()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_51a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_52a()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_52a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_53a()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_53a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_54a()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_54a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_61a()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_61a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_66a()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_66a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_67a()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_67a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_68a()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_68a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__PropertiesFile_71a()).runTest("CWE606_Unchecked_Loop_Condition__PropertiesFile_71a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_01()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_01");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_02()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_02");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_03()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_03");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_04()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_04");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_05()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_05");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_06()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_06");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_07()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_07");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_08()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_08");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_09()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_09");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_10()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_10");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_11()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_11");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_12()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_12");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_13()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_13");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_14()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_14");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_15()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_15");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_16()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_16");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_17()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_17");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_19()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_19");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_31()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_31");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_41()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_41");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_42()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_42");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_45()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_45");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_51a()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_51a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_52a()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_52a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_53a()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_53a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_54a()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_54a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_61a()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_61a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_66a()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_66a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_67a()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_67a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_68a()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_68a");
			(new testcases.CWE606_Unchecked_Loop_Condition.CWE606_Unchecked_Loop_Condition__URLConnection_71a()).runTest("CWE606_Unchecked_Loop_Condition__URLConnection_71a");
			(new testcases.CWE607_Public_Static_Final_Mutable.CWE607_Public_Static_Final_Mutable__console_01()).runTest("CWE607_Public_Static_Final_Mutable__console_01");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_01()).runTest("CWE617_Reachable_Assertion__connect_tcp_01");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_02()).runTest("CWE617_Reachable_Assertion__connect_tcp_02");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_03()).runTest("CWE617_Reachable_Assertion__connect_tcp_03");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_04()).runTest("CWE617_Reachable_Assertion__connect_tcp_04");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_05()).runTest("CWE617_Reachable_Assertion__connect_tcp_05");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_06()).runTest("CWE617_Reachable_Assertion__connect_tcp_06");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_07()).runTest("CWE617_Reachable_Assertion__connect_tcp_07");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_08()).runTest("CWE617_Reachable_Assertion__connect_tcp_08");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_09()).runTest("CWE617_Reachable_Assertion__connect_tcp_09");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_10()).runTest("CWE617_Reachable_Assertion__connect_tcp_10");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_11()).runTest("CWE617_Reachable_Assertion__connect_tcp_11");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_12()).runTest("CWE617_Reachable_Assertion__connect_tcp_12");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_13()).runTest("CWE617_Reachable_Assertion__connect_tcp_13");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_14()).runTest("CWE617_Reachable_Assertion__connect_tcp_14");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_15()).runTest("CWE617_Reachable_Assertion__connect_tcp_15");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_16()).runTest("CWE617_Reachable_Assertion__connect_tcp_16");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_17()).runTest("CWE617_Reachable_Assertion__connect_tcp_17");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_19()).runTest("CWE617_Reachable_Assertion__connect_tcp_19");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_31()).runTest("CWE617_Reachable_Assertion__connect_tcp_31");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_41()).runTest("CWE617_Reachable_Assertion__connect_tcp_41");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_42()).runTest("CWE617_Reachable_Assertion__connect_tcp_42");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_45()).runTest("CWE617_Reachable_Assertion__connect_tcp_45");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_51a()).runTest("CWE617_Reachable_Assertion__connect_tcp_51a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_52a()).runTest("CWE617_Reachable_Assertion__connect_tcp_52a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_53a()).runTest("CWE617_Reachable_Assertion__connect_tcp_53a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_54a()).runTest("CWE617_Reachable_Assertion__connect_tcp_54a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_61a()).runTest("CWE617_Reachable_Assertion__connect_tcp_61a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_66a()).runTest("CWE617_Reachable_Assertion__connect_tcp_66a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_67a()).runTest("CWE617_Reachable_Assertion__connect_tcp_67a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_68a()).runTest("CWE617_Reachable_Assertion__connect_tcp_68a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__connect_tcp_71a()).runTest("CWE617_Reachable_Assertion__connect_tcp_71a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_01()).runTest("CWE617_Reachable_Assertion__console_readLine_01");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_02()).runTest("CWE617_Reachable_Assertion__console_readLine_02");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_03()).runTest("CWE617_Reachable_Assertion__console_readLine_03");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_04()).runTest("CWE617_Reachable_Assertion__console_readLine_04");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_05()).runTest("CWE617_Reachable_Assertion__console_readLine_05");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_06()).runTest("CWE617_Reachable_Assertion__console_readLine_06");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_07()).runTest("CWE617_Reachable_Assertion__console_readLine_07");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_08()).runTest("CWE617_Reachable_Assertion__console_readLine_08");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_09()).runTest("CWE617_Reachable_Assertion__console_readLine_09");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_10()).runTest("CWE617_Reachable_Assertion__console_readLine_10");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_11()).runTest("CWE617_Reachable_Assertion__console_readLine_11");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_12()).runTest("CWE617_Reachable_Assertion__console_readLine_12");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_13()).runTest("CWE617_Reachable_Assertion__console_readLine_13");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_14()).runTest("CWE617_Reachable_Assertion__console_readLine_14");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_15()).runTest("CWE617_Reachable_Assertion__console_readLine_15");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_16()).runTest("CWE617_Reachable_Assertion__console_readLine_16");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_17()).runTest("CWE617_Reachable_Assertion__console_readLine_17");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_19()).runTest("CWE617_Reachable_Assertion__console_readLine_19");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_31()).runTest("CWE617_Reachable_Assertion__console_readLine_31");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_41()).runTest("CWE617_Reachable_Assertion__console_readLine_41");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_42()).runTest("CWE617_Reachable_Assertion__console_readLine_42");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_45()).runTest("CWE617_Reachable_Assertion__console_readLine_45");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_51a()).runTest("CWE617_Reachable_Assertion__console_readLine_51a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_52a()).runTest("CWE617_Reachable_Assertion__console_readLine_52a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_53a()).runTest("CWE617_Reachable_Assertion__console_readLine_53a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_54a()).runTest("CWE617_Reachable_Assertion__console_readLine_54a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_61a()).runTest("CWE617_Reachable_Assertion__console_readLine_61a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_66a()).runTest("CWE617_Reachable_Assertion__console_readLine_66a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_67a()).runTest("CWE617_Reachable_Assertion__console_readLine_67a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_68a()).runTest("CWE617_Reachable_Assertion__console_readLine_68a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__console_readLine_71a()).runTest("CWE617_Reachable_Assertion__console_readLine_71a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_01()).runTest("CWE617_Reachable_Assertion__Environment_01");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_02()).runTest("CWE617_Reachable_Assertion__Environment_02");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_03()).runTest("CWE617_Reachable_Assertion__Environment_03");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_04()).runTest("CWE617_Reachable_Assertion__Environment_04");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_05()).runTest("CWE617_Reachable_Assertion__Environment_05");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_06()).runTest("CWE617_Reachable_Assertion__Environment_06");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_07()).runTest("CWE617_Reachable_Assertion__Environment_07");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_08()).runTest("CWE617_Reachable_Assertion__Environment_08");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_09()).runTest("CWE617_Reachable_Assertion__Environment_09");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_10()).runTest("CWE617_Reachable_Assertion__Environment_10");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_11()).runTest("CWE617_Reachable_Assertion__Environment_11");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_12()).runTest("CWE617_Reachable_Assertion__Environment_12");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_13()).runTest("CWE617_Reachable_Assertion__Environment_13");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_14()).runTest("CWE617_Reachable_Assertion__Environment_14");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_15()).runTest("CWE617_Reachable_Assertion__Environment_15");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_16()).runTest("CWE617_Reachable_Assertion__Environment_16");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_17()).runTest("CWE617_Reachable_Assertion__Environment_17");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_19()).runTest("CWE617_Reachable_Assertion__Environment_19");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_31()).runTest("CWE617_Reachable_Assertion__Environment_31");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_41()).runTest("CWE617_Reachable_Assertion__Environment_41");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_42()).runTest("CWE617_Reachable_Assertion__Environment_42");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_45()).runTest("CWE617_Reachable_Assertion__Environment_45");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_51a()).runTest("CWE617_Reachable_Assertion__Environment_51a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_52a()).runTest("CWE617_Reachable_Assertion__Environment_52a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_53a()).runTest("CWE617_Reachable_Assertion__Environment_53a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_54a()).runTest("CWE617_Reachable_Assertion__Environment_54a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_61a()).runTest("CWE617_Reachable_Assertion__Environment_61a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_66a()).runTest("CWE617_Reachable_Assertion__Environment_66a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_67a()).runTest("CWE617_Reachable_Assertion__Environment_67a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_68a()).runTest("CWE617_Reachable_Assertion__Environment_68a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__Environment_71a()).runTest("CWE617_Reachable_Assertion__Environment_71a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_01()).runTest("CWE617_Reachable_Assertion__fromDB_01");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_02()).runTest("CWE617_Reachable_Assertion__fromDB_02");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_03()).runTest("CWE617_Reachable_Assertion__fromDB_03");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_04()).runTest("CWE617_Reachable_Assertion__fromDB_04");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_05()).runTest("CWE617_Reachable_Assertion__fromDB_05");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_06()).runTest("CWE617_Reachable_Assertion__fromDB_06");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_07()).runTest("CWE617_Reachable_Assertion__fromDB_07");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_08()).runTest("CWE617_Reachable_Assertion__fromDB_08");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_09()).runTest("CWE617_Reachable_Assertion__fromDB_09");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_10()).runTest("CWE617_Reachable_Assertion__fromDB_10");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_11()).runTest("CWE617_Reachable_Assertion__fromDB_11");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_12()).runTest("CWE617_Reachable_Assertion__fromDB_12");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_13()).runTest("CWE617_Reachable_Assertion__fromDB_13");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_14()).runTest("CWE617_Reachable_Assertion__fromDB_14");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_15()).runTest("CWE617_Reachable_Assertion__fromDB_15");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_16()).runTest("CWE617_Reachable_Assertion__fromDB_16");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_17()).runTest("CWE617_Reachable_Assertion__fromDB_17");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_19()).runTest("CWE617_Reachable_Assertion__fromDB_19");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_31()).runTest("CWE617_Reachable_Assertion__fromDB_31");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_41()).runTest("CWE617_Reachable_Assertion__fromDB_41");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_42()).runTest("CWE617_Reachable_Assertion__fromDB_42");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_45()).runTest("CWE617_Reachable_Assertion__fromDB_45");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_51a()).runTest("CWE617_Reachable_Assertion__fromDB_51a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_52a()).runTest("CWE617_Reachable_Assertion__fromDB_52a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_53a()).runTest("CWE617_Reachable_Assertion__fromDB_53a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_54a()).runTest("CWE617_Reachable_Assertion__fromDB_54a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_61a()).runTest("CWE617_Reachable_Assertion__fromDB_61a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_66a()).runTest("CWE617_Reachable_Assertion__fromDB_66a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_67a()).runTest("CWE617_Reachable_Assertion__fromDB_67a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_68a()).runTest("CWE617_Reachable_Assertion__fromDB_68a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromDB_71a()).runTest("CWE617_Reachable_Assertion__fromDB_71a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_01()).runTest("CWE617_Reachable_Assertion__fromFile_01");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_02()).runTest("CWE617_Reachable_Assertion__fromFile_02");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_03()).runTest("CWE617_Reachable_Assertion__fromFile_03");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_04()).runTest("CWE617_Reachable_Assertion__fromFile_04");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_05()).runTest("CWE617_Reachable_Assertion__fromFile_05");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_06()).runTest("CWE617_Reachable_Assertion__fromFile_06");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_07()).runTest("CWE617_Reachable_Assertion__fromFile_07");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_08()).runTest("CWE617_Reachable_Assertion__fromFile_08");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_09()).runTest("CWE617_Reachable_Assertion__fromFile_09");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_10()).runTest("CWE617_Reachable_Assertion__fromFile_10");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_11()).runTest("CWE617_Reachable_Assertion__fromFile_11");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_12()).runTest("CWE617_Reachable_Assertion__fromFile_12");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_13()).runTest("CWE617_Reachable_Assertion__fromFile_13");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_14()).runTest("CWE617_Reachable_Assertion__fromFile_14");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_15()).runTest("CWE617_Reachable_Assertion__fromFile_15");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_16()).runTest("CWE617_Reachable_Assertion__fromFile_16");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_17()).runTest("CWE617_Reachable_Assertion__fromFile_17");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_19()).runTest("CWE617_Reachable_Assertion__fromFile_19");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_31()).runTest("CWE617_Reachable_Assertion__fromFile_31");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_41()).runTest("CWE617_Reachable_Assertion__fromFile_41");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_42()).runTest("CWE617_Reachable_Assertion__fromFile_42");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_45()).runTest("CWE617_Reachable_Assertion__fromFile_45");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_51a()).runTest("CWE617_Reachable_Assertion__fromFile_51a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_52a()).runTest("CWE617_Reachable_Assertion__fromFile_52a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_53a()).runTest("CWE617_Reachable_Assertion__fromFile_53a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_54a()).runTest("CWE617_Reachable_Assertion__fromFile_54a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_61a()).runTest("CWE617_Reachable_Assertion__fromFile_61a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_66a()).runTest("CWE617_Reachable_Assertion__fromFile_66a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_67a()).runTest("CWE617_Reachable_Assertion__fromFile_67a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_68a()).runTest("CWE617_Reachable_Assertion__fromFile_68a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__fromFile_71a()).runTest("CWE617_Reachable_Assertion__fromFile_71a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_01()).runTest("CWE617_Reachable_Assertion__listen_tcp_01");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_02()).runTest("CWE617_Reachable_Assertion__listen_tcp_02");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_03()).runTest("CWE617_Reachable_Assertion__listen_tcp_03");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_04()).runTest("CWE617_Reachable_Assertion__listen_tcp_04");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_05()).runTest("CWE617_Reachable_Assertion__listen_tcp_05");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_06()).runTest("CWE617_Reachable_Assertion__listen_tcp_06");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_07()).runTest("CWE617_Reachable_Assertion__listen_tcp_07");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_08()).runTest("CWE617_Reachable_Assertion__listen_tcp_08");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_09()).runTest("CWE617_Reachable_Assertion__listen_tcp_09");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_10()).runTest("CWE617_Reachable_Assertion__listen_tcp_10");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_11()).runTest("CWE617_Reachable_Assertion__listen_tcp_11");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_12()).runTest("CWE617_Reachable_Assertion__listen_tcp_12");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_13()).runTest("CWE617_Reachable_Assertion__listen_tcp_13");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_14()).runTest("CWE617_Reachable_Assertion__listen_tcp_14");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_15()).runTest("CWE617_Reachable_Assertion__listen_tcp_15");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_16()).runTest("CWE617_Reachable_Assertion__listen_tcp_16");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_17()).runTest("CWE617_Reachable_Assertion__listen_tcp_17");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_19()).runTest("CWE617_Reachable_Assertion__listen_tcp_19");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_31()).runTest("CWE617_Reachable_Assertion__listen_tcp_31");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_41()).runTest("CWE617_Reachable_Assertion__listen_tcp_41");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_42()).runTest("CWE617_Reachable_Assertion__listen_tcp_42");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_45()).runTest("CWE617_Reachable_Assertion__listen_tcp_45");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_51a()).runTest("CWE617_Reachable_Assertion__listen_tcp_51a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_52a()).runTest("CWE617_Reachable_Assertion__listen_tcp_52a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_53a()).runTest("CWE617_Reachable_Assertion__listen_tcp_53a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_54a()).runTest("CWE617_Reachable_Assertion__listen_tcp_54a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_61a()).runTest("CWE617_Reachable_Assertion__listen_tcp_61a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_66a()).runTest("CWE617_Reachable_Assertion__listen_tcp_66a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_67a()).runTest("CWE617_Reachable_Assertion__listen_tcp_67a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_68a()).runTest("CWE617_Reachable_Assertion__listen_tcp_68a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__listen_tcp_71a()).runTest("CWE617_Reachable_Assertion__listen_tcp_71a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_01()).runTest("CWE617_Reachable_Assertion__PropertiesFile_01");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_02()).runTest("CWE617_Reachable_Assertion__PropertiesFile_02");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_03()).runTest("CWE617_Reachable_Assertion__PropertiesFile_03");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_04()).runTest("CWE617_Reachable_Assertion__PropertiesFile_04");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_05()).runTest("CWE617_Reachable_Assertion__PropertiesFile_05");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_06()).runTest("CWE617_Reachable_Assertion__PropertiesFile_06");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_07()).runTest("CWE617_Reachable_Assertion__PropertiesFile_07");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_08()).runTest("CWE617_Reachable_Assertion__PropertiesFile_08");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_09()).runTest("CWE617_Reachable_Assertion__PropertiesFile_09");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_10()).runTest("CWE617_Reachable_Assertion__PropertiesFile_10");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_11()).runTest("CWE617_Reachable_Assertion__PropertiesFile_11");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_12()).runTest("CWE617_Reachable_Assertion__PropertiesFile_12");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_13()).runTest("CWE617_Reachable_Assertion__PropertiesFile_13");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_14()).runTest("CWE617_Reachable_Assertion__PropertiesFile_14");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_15()).runTest("CWE617_Reachable_Assertion__PropertiesFile_15");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_16()).runTest("CWE617_Reachable_Assertion__PropertiesFile_16");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_17()).runTest("CWE617_Reachable_Assertion__PropertiesFile_17");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_19()).runTest("CWE617_Reachable_Assertion__PropertiesFile_19");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_31()).runTest("CWE617_Reachable_Assertion__PropertiesFile_31");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_41()).runTest("CWE617_Reachable_Assertion__PropertiesFile_41");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_42()).runTest("CWE617_Reachable_Assertion__PropertiesFile_42");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_45()).runTest("CWE617_Reachable_Assertion__PropertiesFile_45");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_51a()).runTest("CWE617_Reachable_Assertion__PropertiesFile_51a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_52a()).runTest("CWE617_Reachable_Assertion__PropertiesFile_52a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_53a()).runTest("CWE617_Reachable_Assertion__PropertiesFile_53a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_54a()).runTest("CWE617_Reachable_Assertion__PropertiesFile_54a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_61a()).runTest("CWE617_Reachable_Assertion__PropertiesFile_61a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_66a()).runTest("CWE617_Reachable_Assertion__PropertiesFile_66a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_67a()).runTest("CWE617_Reachable_Assertion__PropertiesFile_67a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_68a()).runTest("CWE617_Reachable_Assertion__PropertiesFile_68a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__PropertiesFile_71a()).runTest("CWE617_Reachable_Assertion__PropertiesFile_71a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_01()).runTest("CWE617_Reachable_Assertion__URLConnection_01");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_02()).runTest("CWE617_Reachable_Assertion__URLConnection_02");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_03()).runTest("CWE617_Reachable_Assertion__URLConnection_03");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_04()).runTest("CWE617_Reachable_Assertion__URLConnection_04");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_05()).runTest("CWE617_Reachable_Assertion__URLConnection_05");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_06()).runTest("CWE617_Reachable_Assertion__URLConnection_06");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_07()).runTest("CWE617_Reachable_Assertion__URLConnection_07");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_08()).runTest("CWE617_Reachable_Assertion__URLConnection_08");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_09()).runTest("CWE617_Reachable_Assertion__URLConnection_09");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_10()).runTest("CWE617_Reachable_Assertion__URLConnection_10");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_11()).runTest("CWE617_Reachable_Assertion__URLConnection_11");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_12()).runTest("CWE617_Reachable_Assertion__URLConnection_12");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_13()).runTest("CWE617_Reachable_Assertion__URLConnection_13");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_14()).runTest("CWE617_Reachable_Assertion__URLConnection_14");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_15()).runTest("CWE617_Reachable_Assertion__URLConnection_15");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_16()).runTest("CWE617_Reachable_Assertion__URLConnection_16");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_17()).runTest("CWE617_Reachable_Assertion__URLConnection_17");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_19()).runTest("CWE617_Reachable_Assertion__URLConnection_19");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_31()).runTest("CWE617_Reachable_Assertion__URLConnection_31");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_41()).runTest("CWE617_Reachable_Assertion__URLConnection_41");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_42()).runTest("CWE617_Reachable_Assertion__URLConnection_42");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_45()).runTest("CWE617_Reachable_Assertion__URLConnection_45");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_51a()).runTest("CWE617_Reachable_Assertion__URLConnection_51a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_52a()).runTest("CWE617_Reachable_Assertion__URLConnection_52a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_53a()).runTest("CWE617_Reachable_Assertion__URLConnection_53a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_54a()).runTest("CWE617_Reachable_Assertion__URLConnection_54a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_61a()).runTest("CWE617_Reachable_Assertion__URLConnection_61a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_66a()).runTest("CWE617_Reachable_Assertion__URLConnection_66a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_67a()).runTest("CWE617_Reachable_Assertion__URLConnection_67a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_68a()).runTest("CWE617_Reachable_Assertion__URLConnection_68a");
			(new testcases.CWE617_Reachable_Assertion.CWE617_Reachable_Assertion__URLConnection_71a()).runTest("CWE617_Reachable_Assertion__URLConnection_71a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_01()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_01");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_02()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_02");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_03()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_03");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_04()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_04");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_05()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_05");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_06()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_06");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_07()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_07");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_08()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_08");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_09()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_09");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_10()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_10");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_11()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_11");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_12()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_12");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_13()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_13");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_14()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_14");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_15()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_15");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_16()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_16");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_17()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_17");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_19()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_19");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_31()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_31");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_41()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_41");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_42()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_42");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_45()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_45");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_51a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_51a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_52a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_52a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_53a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_53a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_54a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_54a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_61a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_61a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_66a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_66a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_67a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_67a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_68a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_68a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_71a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__connect_tcp_71a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_01()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_01");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_02()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_02");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_03()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_03");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_04()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_04");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_05()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_05");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_06()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_06");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_07()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_07");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_08()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_08");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_09()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_09");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_10()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_10");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_11()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_11");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_12()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_12");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_13()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_13");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_14()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_14");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_15()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_15");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_16()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_16");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_17()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_17");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_19()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_19");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_31()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_31");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_41()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_41");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_42()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_42");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_45()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_45");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_51a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_51a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_52a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_52a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_53a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_53a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_54a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_54a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_61a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_61a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_66a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_66a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_67a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_67a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_68a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_68a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_71a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__console_readLine_71a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_01()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_01");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_02()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_02");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_03()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_03");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_04()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_04");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_05()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_05");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_06()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_06");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_07()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_07");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_08()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_08");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_09()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_09");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_10()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_10");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_11()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_11");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_12()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_12");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_13()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_13");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_14()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_14");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_15()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_15");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_16()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_16");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_17()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_17");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_19()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_19");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_31()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_31");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_41()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_41");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_42()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_42");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_45()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_45");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_51a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_51a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_52a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_52a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_53a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_53a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_54a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_54a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_61a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_61a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_66a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_66a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_67a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_67a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_68a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_68a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__Environment_71a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__Environment_71a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_01()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_01");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_02()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_02");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_03()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_03");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_04()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_04");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_05()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_05");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_06()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_06");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_07()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_07");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_08()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_08");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_09()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_09");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_10()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_10");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_11()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_11");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_12()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_12");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_13()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_13");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_14()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_14");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_15()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_15");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_16()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_16");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_17()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_17");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_19()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_19");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_31()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_31");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_41()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_41");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_42()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_42");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_45()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_45");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_51a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_51a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_52a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_52a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_53a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_53a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_54a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_54a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_61a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_61a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_66a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_66a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_67a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_67a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_68a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_68a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_71a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromDB_71a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_01()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_01");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_02()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_02");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_03()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_03");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_04()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_04");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_05()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_05");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_06()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_06");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_07()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_07");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_08()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_08");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_09()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_09");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_10()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_10");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_11()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_11");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_12()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_12");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_13()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_13");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_14()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_14");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_15()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_15");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_16()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_16");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_17()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_17");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_19()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_19");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_31()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_31");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_41()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_41");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_42()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_42");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_45()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_45");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_51a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_51a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_52a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_52a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_53a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_53a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_54a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_54a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_61a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_61a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_66a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_66a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_67a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_67a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_68a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_68a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_71a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__fromFile_71a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_01()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_01");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_02()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_02");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_03()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_03");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_04()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_04");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_05()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_05");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_06()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_06");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_07()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_07");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_08()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_08");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_09()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_09");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_10()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_10");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_11()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_11");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_12()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_12");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_13()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_13");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_14()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_14");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_15()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_15");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_16()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_16");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_17()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_17");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_19()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_19");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_31()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_31");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_41()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_41");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_42()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_42");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_45()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_45");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_51a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_51a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_52a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_52a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_53a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_53a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_54a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_54a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_61a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_61a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_66a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_66a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_67a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_67a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_68a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_68a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_71a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__listen_tcp_71a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_01()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_01");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_02()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_02");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_03()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_03");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_04()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_04");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_05()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_05");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_06()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_06");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_07()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_07");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_08()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_08");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_09()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_09");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_10()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_10");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_11()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_11");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_12()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_12");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_13()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_13");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_14()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_14");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_15()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_15");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_16()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_16");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_17()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_17");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_19()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_19");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_31()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_31");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_41()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_41");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_42()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_42");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_45()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_45");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_51a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_51a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_52a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_52a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_53a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_53a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_54a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_54a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_61a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_61a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_66a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_66a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_67a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_67a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_68a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_68a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_71a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__PropertiesFile_71a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_01()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_01");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_02()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_02");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_03()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_03");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_04()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_04");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_05()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_05");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_06()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_06");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_07()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_07");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_08()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_08");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_09()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_09");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_10()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_10");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_11()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_11");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_12()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_12");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_13()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_13");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_14()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_14");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_15()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_15");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_16()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_16");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_17()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_17");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_19()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_19");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_31()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_31");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_41()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_41");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_42()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_42");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_45()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_45");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_51a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_51a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_52a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_52a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_53a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_53a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_54a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_54a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_61a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_61a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_66a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_66a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_67a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_67a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_68a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_68a");
			(new testcases.CWE643_Unsafe_Treatment_of_XPath_Input.CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_71a()).runTest("CWE643_Unsafe_Treatment_of_XPath_Input__URLConnection_71a");
			(new testcases.CWE663_Use_Non_Reentrant_Function_Unsync.CWE663_Use_Non_Reentrant_Function_Unsync__console_reentrantlock_01_bad()).runTest("CWE663_Use_Non_Reentrant_Function_Unsync__console_reentrantlock_01_bad");
			(new testcases.CWE663_Use_Non_Reentrant_Function_Unsync.CWE663_Use_Non_Reentrant_Function_Unsync__console_reentrantlock_01_good1()).runTest("CWE663_Use_Non_Reentrant_Function_Unsync__console_reentrantlock_01_good1");
			(new testcases.CWE663_Use_Non_Reentrant_Function_Unsync.CWE663_Use_Non_Reentrant_Function_Unsync__console_synchronized_01_bad()).runTest("CWE663_Use_Non_Reentrant_Function_Unsync__console_synchronized_01_bad");
			(new testcases.CWE663_Use_Non_Reentrant_Function_Unsync.CWE663_Use_Non_Reentrant_Function_Unsync__console_synchronized_01_good1()).runTest("CWE663_Use_Non_Reentrant_Function_Unsync__console_synchronized_01_good1");
			(new testcases.CWE665_Improper_Initialization.CWE665_Improper_Initialization__console_01()).runTest("CWE665_Improper_Initialization__console_01");
			(new testcases.CWE674_Uncontrolled_Recursion.CWE674_Uncontrolled_Recursion__console_01()).runTest("CWE674_Uncontrolled_Recursion__console_01");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_01()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_01");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_02()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_02");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_03()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_03");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_04()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_04");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_05()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_05");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_06()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_06");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_07()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_07");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_08()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_08");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_09()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_09");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_10()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_10");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_11()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_11");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_12()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_12");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_13()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_13");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_14()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_14");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_15()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_15");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_16()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_16");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_17()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_17");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_19()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__double2float_19");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_01()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_01");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_02()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_02");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_03()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_03");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_04()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_04");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_05()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_05");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_06()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_06");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_07()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_07");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_08()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_08");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_09()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_09");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_10()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_10");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_11()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_11");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_12()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_12");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_13()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_13");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_14()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_14");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_15()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_15");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_16()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_16");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_17()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_17");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_19()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__float2int_19");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_01()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_01");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_02()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_02");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_03()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_03");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_04()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_04");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_05()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_05");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_06()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_06");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_07()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_07");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_08()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_08");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_09()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_09");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_10()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_10");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_11()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_11");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_12()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_12");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_13()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_13");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_14()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_14");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_15()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_15");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_16()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_16");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_17()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_17");
			(new testcases.CWE681_Incorrect_Conversion_between_Numeric_Types.CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_19()).runTest("CWE681_Incorrect_Conversion_between_Numeric_Types__floatNaN2int_19");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_01()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_01");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_02()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_02");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_03()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_03");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_04()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_04");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_05()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_05");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_06()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_06");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_07()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_07");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_08()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_08");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_09()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_09");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_10()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_10");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_11()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_11");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_12()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_12");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_13()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_13");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_14()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_14");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_15()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_15");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_16()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_16");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_17()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_17");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_19()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_19");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_31()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_31");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_41()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_41");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_42()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_42");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_45()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_45");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_51a()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_51a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_52a()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_52a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_53a()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_53a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_54a()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_54a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_61a()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_61a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_66a()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_66a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_67a()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_67a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_68a()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_68a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_StringBuilder_71a()).runTest("CWE690_NULL_Deref_from_Return__Class_StringBuilder_71a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_01()).runTest("CWE690_NULL_Deref_from_Return__Class_String_01");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_02()).runTest("CWE690_NULL_Deref_from_Return__Class_String_02");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_03()).runTest("CWE690_NULL_Deref_from_Return__Class_String_03");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_04()).runTest("CWE690_NULL_Deref_from_Return__Class_String_04");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_05()).runTest("CWE690_NULL_Deref_from_Return__Class_String_05");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_06()).runTest("CWE690_NULL_Deref_from_Return__Class_String_06");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_07()).runTest("CWE690_NULL_Deref_from_Return__Class_String_07");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_08()).runTest("CWE690_NULL_Deref_from_Return__Class_String_08");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_09()).runTest("CWE690_NULL_Deref_from_Return__Class_String_09");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_10()).runTest("CWE690_NULL_Deref_from_Return__Class_String_10");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_11()).runTest("CWE690_NULL_Deref_from_Return__Class_String_11");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_12()).runTest("CWE690_NULL_Deref_from_Return__Class_String_12");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_13()).runTest("CWE690_NULL_Deref_from_Return__Class_String_13");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_14()).runTest("CWE690_NULL_Deref_from_Return__Class_String_14");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_15()).runTest("CWE690_NULL_Deref_from_Return__Class_String_15");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_16()).runTest("CWE690_NULL_Deref_from_Return__Class_String_16");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_17()).runTest("CWE690_NULL_Deref_from_Return__Class_String_17");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_19()).runTest("CWE690_NULL_Deref_from_Return__Class_String_19");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_31()).runTest("CWE690_NULL_Deref_from_Return__Class_String_31");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_41()).runTest("CWE690_NULL_Deref_from_Return__Class_String_41");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_42()).runTest("CWE690_NULL_Deref_from_Return__Class_String_42");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_45()).runTest("CWE690_NULL_Deref_from_Return__Class_String_45");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_51a()).runTest("CWE690_NULL_Deref_from_Return__Class_String_51a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_52a()).runTest("CWE690_NULL_Deref_from_Return__Class_String_52a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_53a()).runTest("CWE690_NULL_Deref_from_Return__Class_String_53a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_54a()).runTest("CWE690_NULL_Deref_from_Return__Class_String_54a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_61a()).runTest("CWE690_NULL_Deref_from_Return__Class_String_61a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_66a()).runTest("CWE690_NULL_Deref_from_Return__Class_String_66a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_67a()).runTest("CWE690_NULL_Deref_from_Return__Class_String_67a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_68a()).runTest("CWE690_NULL_Deref_from_Return__Class_String_68a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Class_String_71a()).runTest("CWE690_NULL_Deref_from_Return__Class_String_71a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_01()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_01");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_02()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_02");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_03()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_03");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_04()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_04");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_05()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_05");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_06()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_06");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_07()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_07");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_08()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_08");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_09()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_09");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_10()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_10");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_11()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_11");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_12()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_12");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_13()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_13");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_14()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_14");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_15()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_15");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_16()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_16");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_17()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_17");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_19()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_19");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_31()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_31");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_41()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_41");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_42()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_42");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_45()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_45");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_51a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_51a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_52a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_52a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_53a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_53a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_54a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_54a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_61a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_61a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_66a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_66a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_67a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_67a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_68a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_68a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_71a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_equals_71a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_01()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_01");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_02()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_02");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_03()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_03");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_04()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_04");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_05()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_05");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_06()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_06");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_07()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_07");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_08()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_08");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_09()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_09");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_10()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_10");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_11()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_11");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_12()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_12");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_13()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_13");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_14()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_14");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_15()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_15");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_16()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_16");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_17()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_17");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_19()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_19");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_31()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_31");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_41()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_41");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_42()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_42");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_45()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_45");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_51a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_51a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_52a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_52a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_53a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_53a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_54a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_54a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_61a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_61a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_66a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_66a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_67a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_67a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_68a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_68a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_71a()).runTest("CWE690_NULL_Deref_from_Return__Properties_getProperty_trim_71a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_01()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_01");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_02()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_02");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_03()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_03");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_04()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_04");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_05()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_05");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_06()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_06");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_07()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_07");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_08()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_08");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_09()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_09");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_10()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_10");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_11()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_11");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_12()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_12");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_13()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_13");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_14()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_14");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_15()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_15");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_16()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_16");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_17()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_17");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_19()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_19");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_31()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_31");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_41()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_41");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_42()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_42");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_45()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_45");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_51a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_51a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_52a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_52a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_53a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_53a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_54a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_54a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_61a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_61a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_66a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_66a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_67a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_67a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_68a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_68a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_equals_71a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_equals_71a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_01()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_01");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_02()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_02");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_03()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_03");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_04()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_04");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_05()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_05");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_06()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_06");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_07()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_07");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_08()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_08");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_09()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_09");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_10()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_10");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_11()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_11");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_12()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_12");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_13()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_13");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_14()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_14");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_15()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_15");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_16()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_16");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_17()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_17");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_19()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_19");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_31()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_31");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_41()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_41");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_42()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_42");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_45()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_45");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_51a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_51a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_52a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_52a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_53a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_53a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_54a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_54a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_61a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_61a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_66a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_66a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_67a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_67a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_68a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_68a");
			(new testcases.CWE690_NULL_Deref_from_Return.CWE690_NULL_Deref_from_Return__System_getProperty_trim_71a()).runTest("CWE690_NULL_Deref_from_Return__System_getProperty_trim_71a");
			/* END-AUTOGENERATED-JAVA-TESTS-6 */
}

private static void runTestCWE7() {
/* BEGIN-AUTOGENERATED-JAVA-TESTS-7 */
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_01()).runTest("CWE759_Unsalted_One_Way_Hash__basic_01");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_02()).runTest("CWE759_Unsalted_One_Way_Hash__basic_02");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_03()).runTest("CWE759_Unsalted_One_Way_Hash__basic_03");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_04()).runTest("CWE759_Unsalted_One_Way_Hash__basic_04");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_05()).runTest("CWE759_Unsalted_One_Way_Hash__basic_05");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_06()).runTest("CWE759_Unsalted_One_Way_Hash__basic_06");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_07()).runTest("CWE759_Unsalted_One_Way_Hash__basic_07");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_08()).runTest("CWE759_Unsalted_One_Way_Hash__basic_08");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_09()).runTest("CWE759_Unsalted_One_Way_Hash__basic_09");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_10()).runTest("CWE759_Unsalted_One_Way_Hash__basic_10");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_11()).runTest("CWE759_Unsalted_One_Way_Hash__basic_11");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_12()).runTest("CWE759_Unsalted_One_Way_Hash__basic_12");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_13()).runTest("CWE759_Unsalted_One_Way_Hash__basic_13");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_14()).runTest("CWE759_Unsalted_One_Way_Hash__basic_14");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_15()).runTest("CWE759_Unsalted_One_Way_Hash__basic_15");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_16()).runTest("CWE759_Unsalted_One_Way_Hash__basic_16");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_17()).runTest("CWE759_Unsalted_One_Way_Hash__basic_17");
			(new testcases.CWE759_Unsalted_One_Way_Hash.CWE759_Unsalted_One_Way_Hash__basic_19()).runTest("CWE759_Unsalted_One_Way_Hash__basic_19");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_01()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_01");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_02()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_02");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_03()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_03");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_04()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_04");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_05()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_05");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_06()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_06");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_07()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_07");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_08()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_08");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_09()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_09");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_10()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_10");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_11()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_11");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_12()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_12");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_13()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_13");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_14()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_14");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_15()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_15");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_16()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_16");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_17()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_17");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_19()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_19");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_31()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_31");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_41()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_41");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_42()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_42");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_45()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_45");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_51a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_51a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_52a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_52a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_53a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_53a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_54a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_54a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_61a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_61a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_66a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_66a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_67a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_67a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_68a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_68a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_71a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__connect_tcp_71a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_01()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_01");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_02()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_02");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_03()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_03");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_04()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_04");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_05()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_05");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_06()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_06");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_07()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_07");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_08()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_08");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_09()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_09");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_10()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_10");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_11()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_11");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_12()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_12");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_13()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_13");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_14()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_14");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_15()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_15");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_16()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_16");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_17()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_17");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_19()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_19");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_31()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_31");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_41()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_41");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_42()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_42");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_45()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_45");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_51a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_51a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_52a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_52a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_53a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_53a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_54a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_54a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_61a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_61a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_66a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_66a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_67a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_67a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_68a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_68a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__console_readLine_71a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__console_readLine_71a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_01()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_01");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_02()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_02");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_03()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_03");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_04()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_04");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_05()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_05");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_06()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_06");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_07()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_07");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_08()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_08");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_09()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_09");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_10()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_10");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_11()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_11");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_12()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_12");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_13()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_13");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_14()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_14");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_15()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_15");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_16()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_16");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_17()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_17");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_19()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_19");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_31()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_31");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_41()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_41");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_42()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_42");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_45()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_45");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_51a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_51a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_52a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_52a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_53a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_53a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_54a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_54a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_61a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_61a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_66a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_66a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_67a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_67a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_68a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_68a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__Environment_71a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__Environment_71a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_01()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_01");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_02()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_02");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_03()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_03");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_04()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_04");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_05()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_05");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_06()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_06");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_07()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_07");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_08()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_08");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_09()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_09");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_10()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_10");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_11()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_11");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_12()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_12");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_13()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_13");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_14()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_14");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_15()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_15");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_16()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_16");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_17()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_17");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_19()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_19");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_31()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_31");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_41()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_41");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_42()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_42");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_45()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_45");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_51a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_51a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_52a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_52a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_53a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_53a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_54a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_54a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_61a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_61a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_66a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_66a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_67a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_67a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_68a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_68a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromDB_71a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromDB_71a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_01()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_01");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_02()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_02");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_03()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_03");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_04()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_04");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_05()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_05");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_06()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_06");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_07()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_07");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_08()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_08");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_09()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_09");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_10()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_10");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_11()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_11");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_12()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_12");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_13()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_13");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_14()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_14");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_15()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_15");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_16()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_16");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_17()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_17");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_19()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_19");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_31()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_31");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_41()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_41");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_42()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_42");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_45()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_45");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_51a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_51a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_52a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_52a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_53a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_53a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_54a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_54a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_61a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_61a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_66a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_66a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_67a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_67a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_68a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_68a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__fromFile_71a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__fromFile_71a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_01()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_01");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_02()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_02");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_03()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_03");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_04()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_04");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_05()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_05");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_06()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_06");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_07()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_07");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_08()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_08");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_09()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_09");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_10()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_10");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_11()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_11");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_12()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_12");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_13()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_13");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_14()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_14");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_15()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_15");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_16()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_16");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_17()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_17");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_19()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_19");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_31()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_31");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_41()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_41");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_42()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_42");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_45()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_45");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_51a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_51a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_52a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_52a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_53a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_53a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_54a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_54a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_61a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_61a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_66a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_66a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_67a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_67a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_68a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_68a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_71a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__listen_tcp_71a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_01()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_01");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_02()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_02");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_03()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_03");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_04()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_04");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_05()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_05");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_06()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_06");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_07()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_07");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_08()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_08");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_09()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_09");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_10()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_10");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_11()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_11");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_12()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_12");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_13()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_13");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_14()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_14");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_15()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_15");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_16()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_16");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_17()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_17");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_19()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_19");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_31()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_31");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_41()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_41");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_42()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_42");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_45()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_45");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_51a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_51a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_52a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_52a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_53a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_53a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_54a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_54a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_61a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_61a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_66a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_66a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_67a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_67a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_68a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_68a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_71a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__PropertiesFile_71a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_01()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_01");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_02()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_02");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_03()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_03");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_04()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_04");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_05()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_05");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_06()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_06");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_07()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_07");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_08()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_08");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_09()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_09");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_10()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_10");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_11()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_11");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_12()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_12");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_13()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_13");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_14()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_14");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_15()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_15");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_16()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_16");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_17()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_17");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_19()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_19");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_31()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_31");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_41()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_41");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_42()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_42");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_45()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_45");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_51a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_51a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_52a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_52a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_53a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_53a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_54a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_54a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_61a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_61a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_66a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_66a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_67a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_67a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_68a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_68a");
			(new testcases.CWE760_Predictable_Salt_One_Way_Hash.CWE760_Predictable_Salt_One_Way_Hash__URLConnection_71a()).runTest("CWE760_Predictable_Salt_One_Way_Hash__URLConnection_71a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_01()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_01");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_02()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_02");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_03()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_03");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_04()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_04");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_05()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_05");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_06()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_06");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_07()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_07");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_08()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_08");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_09()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_09");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_10()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_10");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_11()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_11");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_12()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_12");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_13()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_13");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_14()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_14");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_15()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_15");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_16()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_16");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_17()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_17");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_19()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_19");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_31()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_31");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_41()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_41");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_42()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_42");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_45()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_45");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_51a()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_51a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_52a()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_52a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_53a()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_53a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_54a()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_54a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_61a()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_61a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_66a()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_66a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_67a()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_67a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__connect_tcp_68a()).runTest("CWE789_Uncontrolled_Mem_Alloc__connect_tcp_68a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_01()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_01");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_02()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_02");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_03()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_03");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_04()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_04");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_05()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_05");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_06()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_06");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_07()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_07");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_08()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_08");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_09()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_09");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_10()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_10");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_11()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_11");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_12()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_12");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_13()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_13");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_14()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_14");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_15()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_15");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_16()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_16");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_17()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_17");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_19()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_19");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_31()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_31");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_41()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_41");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_42()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_42");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_45()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_45");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_51a()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_51a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_52a()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_52a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_53a()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_53a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_54a()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_54a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_61a()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_61a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_66a()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_66a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_67a()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_67a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__console_readLine_68a()).runTest("CWE789_Uncontrolled_Mem_Alloc__console_readLine_68a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_01()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_01");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_02()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_02");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_03()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_03");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_04()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_04");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_05()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_05");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_06()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_06");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_07()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_07");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_08()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_08");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_09()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_09");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_10()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_10");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_11()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_11");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_12()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_12");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_13()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_13");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_14()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_14");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_15()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_15");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_16()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_16");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_17()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_17");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_19()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_19");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_31()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_31");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_41()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_41");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_42()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_42");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_45()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_45");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_51a()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_51a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_52a()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_52a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_53a()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_53a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_54a()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_54a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_61a()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_61a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_66a()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_66a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_67a()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_67a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__Environment_68a()).runTest("CWE789_Uncontrolled_Mem_Alloc__Environment_68a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_01()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_01");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_02()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_02");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_03()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_03");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_04()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_04");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_05()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_05");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_06()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_06");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_07()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_07");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_08()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_08");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_09()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_09");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_10()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_10");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_11()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_11");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_12()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_12");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_13()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_13");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_14()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_14");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_15()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_15");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_16()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_16");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_17()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_17");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_19()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_19");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_31()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_31");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_41()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_41");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_42()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_42");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_45()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_45");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_51a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_51a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_52a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_52a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_53a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_53a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_54a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_54a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_61a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_61a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_66a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_66a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_67a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_67a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromDB_68a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromDB_68a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_01()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_01");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_02()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_02");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_03()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_03");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_04()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_04");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_05()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_05");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_06()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_06");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_07()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_07");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_08()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_08");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_09()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_09");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_10()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_10");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_11()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_11");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_12()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_12");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_13()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_13");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_14()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_14");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_15()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_15");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_16()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_16");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_17()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_17");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_19()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_19");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_31()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_31");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_41()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_41");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_42()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_42");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_45()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_45");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_51a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_51a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_52a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_52a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_53a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_53a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_54a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_54a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_61a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_61a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_66a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_66a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_67a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_67a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__fromFile_68a()).runTest("CWE789_Uncontrolled_Mem_Alloc__fromFile_68a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_01()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_01");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_02()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_02");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_03()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_03");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_04()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_04");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_05()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_05");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_06()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_06");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_07()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_07");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_08()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_08");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_09()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_09");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_10()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_10");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_11()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_11");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_12()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_12");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_13()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_13");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_14()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_14");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_15()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_15");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_16()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_16");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_17()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_17");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_19()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_19");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_31()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_31");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_41()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_41");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_42()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_42");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_45()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_45");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_51a()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_51a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_52a()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_52a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_53a()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_53a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_54a()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_54a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_61a()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_61a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_66a()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_66a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_67a()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_67a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__listen_tcp_68a()).runTest("CWE789_Uncontrolled_Mem_Alloc__listen_tcp_68a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_01()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_01");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_02()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_02");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_03()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_03");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_04()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_04");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_05()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_05");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_06()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_06");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_07()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_07");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_08()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_08");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_09()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_09");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_10()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_10");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_11()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_11");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_12()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_12");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_13()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_13");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_14()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_14");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_15()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_15");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_16()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_16");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_17()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_17");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_19()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_19");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_31()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_31");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_41()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_41");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_42()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_42");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_45()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_45");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_51a()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_51a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_52a()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_52a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_53a()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_53a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_54a()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_54a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_61a()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_61a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_66a()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_66a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_67a()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_67a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__max_value_68a()).runTest("CWE789_Uncontrolled_Mem_Alloc__max_value_68a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_01()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_01");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_02()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_02");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_03()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_03");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_04()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_04");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_05()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_05");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_06()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_06");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_07()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_07");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_08()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_08");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_09()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_09");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_10()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_10");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_11()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_11");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_12()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_12");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_13()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_13");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_14()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_14");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_15()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_15");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_16()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_16");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_17()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_17");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_19()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_19");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_31()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_31");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_41()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_41");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_42()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_42");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_45()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_45");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_51a()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_51a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_52a()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_52a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_53a()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_53a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_54a()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_54a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_61a()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_61a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_66a()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_66a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_67a()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_67a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_68a()).runTest("CWE789_Uncontrolled_Mem_Alloc__PropertiesFile_68a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_01()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_01");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_02()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_02");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_03()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_03");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_04()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_04");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_05()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_05");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_06()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_06");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_07()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_07");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_08()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_08");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_09()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_09");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_10()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_10");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_11()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_11");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_12()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_12");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_13()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_13");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_14()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_14");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_15()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_15");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_16()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_16");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_17()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_17");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_19()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_19");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_31()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_31");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_41()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_41");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_42()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_42");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_45()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_45");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_51a()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_51a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_52a()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_52a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_53a()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_53a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_54a()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_54a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_61a()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_61a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_66a()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_66a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_67a()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_67a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__random_68a()).runTest("CWE789_Uncontrolled_Mem_Alloc__random_68a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_01()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_01");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_02()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_02");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_03()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_03");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_04()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_04");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_05()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_05");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_06()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_06");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_07()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_07");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_08()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_08");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_09()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_09");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_10()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_10");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_11()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_11");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_12()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_12");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_13()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_13");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_14()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_14");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_15()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_15");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_16()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_16");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_17()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_17");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_19()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_19");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_31()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_31");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_41()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_41");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_42()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_42");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_45()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_45");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_51a()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_51a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_52a()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_52a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_53a()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_53a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_54a()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_54a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_61a()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_61a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_66a()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_66a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_67a()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_67a");
			(new testcases.CWE789_Uncontrolled_Mem_Alloc.CWE789_Uncontrolled_Mem_Alloc__URLConnection_68a()).runTest("CWE789_Uncontrolled_Mem_Alloc__URLConnection_68a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_01()).runTest("CWE78_OS_Command_Injection__connect_tcp_01");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_02()).runTest("CWE78_OS_Command_Injection__connect_tcp_02");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_03()).runTest("CWE78_OS_Command_Injection__connect_tcp_03");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_04()).runTest("CWE78_OS_Command_Injection__connect_tcp_04");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_05()).runTest("CWE78_OS_Command_Injection__connect_tcp_05");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_06()).runTest("CWE78_OS_Command_Injection__connect_tcp_06");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_07()).runTest("CWE78_OS_Command_Injection__connect_tcp_07");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_08()).runTest("CWE78_OS_Command_Injection__connect_tcp_08");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_09()).runTest("CWE78_OS_Command_Injection__connect_tcp_09");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_10()).runTest("CWE78_OS_Command_Injection__connect_tcp_10");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_11()).runTest("CWE78_OS_Command_Injection__connect_tcp_11");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_12()).runTest("CWE78_OS_Command_Injection__connect_tcp_12");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_13()).runTest("CWE78_OS_Command_Injection__connect_tcp_13");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_14()).runTest("CWE78_OS_Command_Injection__connect_tcp_14");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_15()).runTest("CWE78_OS_Command_Injection__connect_tcp_15");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_16()).runTest("CWE78_OS_Command_Injection__connect_tcp_16");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_17()).runTest("CWE78_OS_Command_Injection__connect_tcp_17");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_19()).runTest("CWE78_OS_Command_Injection__connect_tcp_19");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_31()).runTest("CWE78_OS_Command_Injection__connect_tcp_31");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_41()).runTest("CWE78_OS_Command_Injection__connect_tcp_41");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_42()).runTest("CWE78_OS_Command_Injection__connect_tcp_42");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_45()).runTest("CWE78_OS_Command_Injection__connect_tcp_45");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_51a()).runTest("CWE78_OS_Command_Injection__connect_tcp_51a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_52a()).runTest("CWE78_OS_Command_Injection__connect_tcp_52a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_53a()).runTest("CWE78_OS_Command_Injection__connect_tcp_53a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_54a()).runTest("CWE78_OS_Command_Injection__connect_tcp_54a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_61a()).runTest("CWE78_OS_Command_Injection__connect_tcp_61a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_66a()).runTest("CWE78_OS_Command_Injection__connect_tcp_66a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_67a()).runTest("CWE78_OS_Command_Injection__connect_tcp_67a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_68a()).runTest("CWE78_OS_Command_Injection__connect_tcp_68a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__connect_tcp_71a()).runTest("CWE78_OS_Command_Injection__connect_tcp_71a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_01()).runTest("CWE78_OS_Command_Injection__console_readLine_01");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_02()).runTest("CWE78_OS_Command_Injection__console_readLine_02");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_03()).runTest("CWE78_OS_Command_Injection__console_readLine_03");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_04()).runTest("CWE78_OS_Command_Injection__console_readLine_04");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_05()).runTest("CWE78_OS_Command_Injection__console_readLine_05");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_06()).runTest("CWE78_OS_Command_Injection__console_readLine_06");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_07()).runTest("CWE78_OS_Command_Injection__console_readLine_07");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_08()).runTest("CWE78_OS_Command_Injection__console_readLine_08");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_09()).runTest("CWE78_OS_Command_Injection__console_readLine_09");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_10()).runTest("CWE78_OS_Command_Injection__console_readLine_10");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_11()).runTest("CWE78_OS_Command_Injection__console_readLine_11");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_12()).runTest("CWE78_OS_Command_Injection__console_readLine_12");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_13()).runTest("CWE78_OS_Command_Injection__console_readLine_13");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_14()).runTest("CWE78_OS_Command_Injection__console_readLine_14");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_15()).runTest("CWE78_OS_Command_Injection__console_readLine_15");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_16()).runTest("CWE78_OS_Command_Injection__console_readLine_16");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_17()).runTest("CWE78_OS_Command_Injection__console_readLine_17");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_19()).runTest("CWE78_OS_Command_Injection__console_readLine_19");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_31()).runTest("CWE78_OS_Command_Injection__console_readLine_31");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_41()).runTest("CWE78_OS_Command_Injection__console_readLine_41");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_42()).runTest("CWE78_OS_Command_Injection__console_readLine_42");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_45()).runTest("CWE78_OS_Command_Injection__console_readLine_45");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_51a()).runTest("CWE78_OS_Command_Injection__console_readLine_51a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_52a()).runTest("CWE78_OS_Command_Injection__console_readLine_52a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_53a()).runTest("CWE78_OS_Command_Injection__console_readLine_53a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_54a()).runTest("CWE78_OS_Command_Injection__console_readLine_54a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_61a()).runTest("CWE78_OS_Command_Injection__console_readLine_61a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_66a()).runTest("CWE78_OS_Command_Injection__console_readLine_66a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_67a()).runTest("CWE78_OS_Command_Injection__console_readLine_67a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_68a()).runTest("CWE78_OS_Command_Injection__console_readLine_68a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__console_readLine_71a()).runTest("CWE78_OS_Command_Injection__console_readLine_71a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_01()).runTest("CWE78_OS_Command_Injection__Environment_01");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_02()).runTest("CWE78_OS_Command_Injection__Environment_02");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_03()).runTest("CWE78_OS_Command_Injection__Environment_03");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_04()).runTest("CWE78_OS_Command_Injection__Environment_04");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_05()).runTest("CWE78_OS_Command_Injection__Environment_05");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_06()).runTest("CWE78_OS_Command_Injection__Environment_06");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_07()).runTest("CWE78_OS_Command_Injection__Environment_07");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_08()).runTest("CWE78_OS_Command_Injection__Environment_08");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_09()).runTest("CWE78_OS_Command_Injection__Environment_09");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_10()).runTest("CWE78_OS_Command_Injection__Environment_10");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_11()).runTest("CWE78_OS_Command_Injection__Environment_11");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_12()).runTest("CWE78_OS_Command_Injection__Environment_12");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_13()).runTest("CWE78_OS_Command_Injection__Environment_13");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_14()).runTest("CWE78_OS_Command_Injection__Environment_14");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_15()).runTest("CWE78_OS_Command_Injection__Environment_15");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_16()).runTest("CWE78_OS_Command_Injection__Environment_16");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_17()).runTest("CWE78_OS_Command_Injection__Environment_17");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_19()).runTest("CWE78_OS_Command_Injection__Environment_19");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_31()).runTest("CWE78_OS_Command_Injection__Environment_31");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_41()).runTest("CWE78_OS_Command_Injection__Environment_41");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_42()).runTest("CWE78_OS_Command_Injection__Environment_42");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_45()).runTest("CWE78_OS_Command_Injection__Environment_45");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_51a()).runTest("CWE78_OS_Command_Injection__Environment_51a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_52a()).runTest("CWE78_OS_Command_Injection__Environment_52a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_53a()).runTest("CWE78_OS_Command_Injection__Environment_53a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_54a()).runTest("CWE78_OS_Command_Injection__Environment_54a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_61a()).runTest("CWE78_OS_Command_Injection__Environment_61a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_66a()).runTest("CWE78_OS_Command_Injection__Environment_66a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_67a()).runTest("CWE78_OS_Command_Injection__Environment_67a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_68a()).runTest("CWE78_OS_Command_Injection__Environment_68a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__Environment_71a()).runTest("CWE78_OS_Command_Injection__Environment_71a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_01()).runTest("CWE78_OS_Command_Injection__fromDB_01");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_02()).runTest("CWE78_OS_Command_Injection__fromDB_02");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_03()).runTest("CWE78_OS_Command_Injection__fromDB_03");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_04()).runTest("CWE78_OS_Command_Injection__fromDB_04");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_05()).runTest("CWE78_OS_Command_Injection__fromDB_05");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_06()).runTest("CWE78_OS_Command_Injection__fromDB_06");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_07()).runTest("CWE78_OS_Command_Injection__fromDB_07");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_08()).runTest("CWE78_OS_Command_Injection__fromDB_08");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_09()).runTest("CWE78_OS_Command_Injection__fromDB_09");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_10()).runTest("CWE78_OS_Command_Injection__fromDB_10");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_11()).runTest("CWE78_OS_Command_Injection__fromDB_11");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_12()).runTest("CWE78_OS_Command_Injection__fromDB_12");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_13()).runTest("CWE78_OS_Command_Injection__fromDB_13");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_14()).runTest("CWE78_OS_Command_Injection__fromDB_14");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_15()).runTest("CWE78_OS_Command_Injection__fromDB_15");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_16()).runTest("CWE78_OS_Command_Injection__fromDB_16");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_17()).runTest("CWE78_OS_Command_Injection__fromDB_17");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_19()).runTest("CWE78_OS_Command_Injection__fromDB_19");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_31()).runTest("CWE78_OS_Command_Injection__fromDB_31");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_41()).runTest("CWE78_OS_Command_Injection__fromDB_41");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_42()).runTest("CWE78_OS_Command_Injection__fromDB_42");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_45()).runTest("CWE78_OS_Command_Injection__fromDB_45");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_51a()).runTest("CWE78_OS_Command_Injection__fromDB_51a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_52a()).runTest("CWE78_OS_Command_Injection__fromDB_52a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_53a()).runTest("CWE78_OS_Command_Injection__fromDB_53a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_54a()).runTest("CWE78_OS_Command_Injection__fromDB_54a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_61a()).runTest("CWE78_OS_Command_Injection__fromDB_61a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_66a()).runTest("CWE78_OS_Command_Injection__fromDB_66a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_67a()).runTest("CWE78_OS_Command_Injection__fromDB_67a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_68a()).runTest("CWE78_OS_Command_Injection__fromDB_68a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromDB_71a()).runTest("CWE78_OS_Command_Injection__fromDB_71a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_01()).runTest("CWE78_OS_Command_Injection__fromFile_01");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_02()).runTest("CWE78_OS_Command_Injection__fromFile_02");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_03()).runTest("CWE78_OS_Command_Injection__fromFile_03");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_04()).runTest("CWE78_OS_Command_Injection__fromFile_04");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_05()).runTest("CWE78_OS_Command_Injection__fromFile_05");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_06()).runTest("CWE78_OS_Command_Injection__fromFile_06");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_07()).runTest("CWE78_OS_Command_Injection__fromFile_07");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_08()).runTest("CWE78_OS_Command_Injection__fromFile_08");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_09()).runTest("CWE78_OS_Command_Injection__fromFile_09");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_10()).runTest("CWE78_OS_Command_Injection__fromFile_10");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_11()).runTest("CWE78_OS_Command_Injection__fromFile_11");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_12()).runTest("CWE78_OS_Command_Injection__fromFile_12");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_13()).runTest("CWE78_OS_Command_Injection__fromFile_13");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_14()).runTest("CWE78_OS_Command_Injection__fromFile_14");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_15()).runTest("CWE78_OS_Command_Injection__fromFile_15");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_16()).runTest("CWE78_OS_Command_Injection__fromFile_16");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_17()).runTest("CWE78_OS_Command_Injection__fromFile_17");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_19()).runTest("CWE78_OS_Command_Injection__fromFile_19");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_31()).runTest("CWE78_OS_Command_Injection__fromFile_31");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_41()).runTest("CWE78_OS_Command_Injection__fromFile_41");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_42()).runTest("CWE78_OS_Command_Injection__fromFile_42");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_45()).runTest("CWE78_OS_Command_Injection__fromFile_45");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_51a()).runTest("CWE78_OS_Command_Injection__fromFile_51a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_52a()).runTest("CWE78_OS_Command_Injection__fromFile_52a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_53a()).runTest("CWE78_OS_Command_Injection__fromFile_53a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_54a()).runTest("CWE78_OS_Command_Injection__fromFile_54a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_61a()).runTest("CWE78_OS_Command_Injection__fromFile_61a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_66a()).runTest("CWE78_OS_Command_Injection__fromFile_66a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_67a()).runTest("CWE78_OS_Command_Injection__fromFile_67a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_68a()).runTest("CWE78_OS_Command_Injection__fromFile_68a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__fromFile_71a()).runTest("CWE78_OS_Command_Injection__fromFile_71a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_01()).runTest("CWE78_OS_Command_Injection__listen_tcp_01");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_02()).runTest("CWE78_OS_Command_Injection__listen_tcp_02");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_03()).runTest("CWE78_OS_Command_Injection__listen_tcp_03");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_04()).runTest("CWE78_OS_Command_Injection__listen_tcp_04");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_05()).runTest("CWE78_OS_Command_Injection__listen_tcp_05");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_06()).runTest("CWE78_OS_Command_Injection__listen_tcp_06");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_07()).runTest("CWE78_OS_Command_Injection__listen_tcp_07");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_08()).runTest("CWE78_OS_Command_Injection__listen_tcp_08");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_09()).runTest("CWE78_OS_Command_Injection__listen_tcp_09");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_10()).runTest("CWE78_OS_Command_Injection__listen_tcp_10");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_11()).runTest("CWE78_OS_Command_Injection__listen_tcp_11");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_12()).runTest("CWE78_OS_Command_Injection__listen_tcp_12");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_13()).runTest("CWE78_OS_Command_Injection__listen_tcp_13");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_14()).runTest("CWE78_OS_Command_Injection__listen_tcp_14");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_15()).runTest("CWE78_OS_Command_Injection__listen_tcp_15");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_16()).runTest("CWE78_OS_Command_Injection__listen_tcp_16");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_17()).runTest("CWE78_OS_Command_Injection__listen_tcp_17");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_19()).runTest("CWE78_OS_Command_Injection__listen_tcp_19");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_31()).runTest("CWE78_OS_Command_Injection__listen_tcp_31");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_41()).runTest("CWE78_OS_Command_Injection__listen_tcp_41");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_42()).runTest("CWE78_OS_Command_Injection__listen_tcp_42");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_45()).runTest("CWE78_OS_Command_Injection__listen_tcp_45");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_51a()).runTest("CWE78_OS_Command_Injection__listen_tcp_51a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_52a()).runTest("CWE78_OS_Command_Injection__listen_tcp_52a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_53a()).runTest("CWE78_OS_Command_Injection__listen_tcp_53a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_54a()).runTest("CWE78_OS_Command_Injection__listen_tcp_54a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_61a()).runTest("CWE78_OS_Command_Injection__listen_tcp_61a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_66a()).runTest("CWE78_OS_Command_Injection__listen_tcp_66a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_67a()).runTest("CWE78_OS_Command_Injection__listen_tcp_67a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_68a()).runTest("CWE78_OS_Command_Injection__listen_tcp_68a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__listen_tcp_71a()).runTest("CWE78_OS_Command_Injection__listen_tcp_71a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_01()).runTest("CWE78_OS_Command_Injection__PropertiesFile_01");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_02()).runTest("CWE78_OS_Command_Injection__PropertiesFile_02");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_03()).runTest("CWE78_OS_Command_Injection__PropertiesFile_03");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_04()).runTest("CWE78_OS_Command_Injection__PropertiesFile_04");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_05()).runTest("CWE78_OS_Command_Injection__PropertiesFile_05");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_06()).runTest("CWE78_OS_Command_Injection__PropertiesFile_06");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_07()).runTest("CWE78_OS_Command_Injection__PropertiesFile_07");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_08()).runTest("CWE78_OS_Command_Injection__PropertiesFile_08");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_09()).runTest("CWE78_OS_Command_Injection__PropertiesFile_09");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_10()).runTest("CWE78_OS_Command_Injection__PropertiesFile_10");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_11()).runTest("CWE78_OS_Command_Injection__PropertiesFile_11");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_12()).runTest("CWE78_OS_Command_Injection__PropertiesFile_12");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_13()).runTest("CWE78_OS_Command_Injection__PropertiesFile_13");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_14()).runTest("CWE78_OS_Command_Injection__PropertiesFile_14");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_15()).runTest("CWE78_OS_Command_Injection__PropertiesFile_15");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_16()).runTest("CWE78_OS_Command_Injection__PropertiesFile_16");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_17()).runTest("CWE78_OS_Command_Injection__PropertiesFile_17");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_19()).runTest("CWE78_OS_Command_Injection__PropertiesFile_19");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_31()).runTest("CWE78_OS_Command_Injection__PropertiesFile_31");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_41()).runTest("CWE78_OS_Command_Injection__PropertiesFile_41");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_42()).runTest("CWE78_OS_Command_Injection__PropertiesFile_42");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_45()).runTest("CWE78_OS_Command_Injection__PropertiesFile_45");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_51a()).runTest("CWE78_OS_Command_Injection__PropertiesFile_51a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_52a()).runTest("CWE78_OS_Command_Injection__PropertiesFile_52a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_53a()).runTest("CWE78_OS_Command_Injection__PropertiesFile_53a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_54a()).runTest("CWE78_OS_Command_Injection__PropertiesFile_54a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_61a()).runTest("CWE78_OS_Command_Injection__PropertiesFile_61a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_66a()).runTest("CWE78_OS_Command_Injection__PropertiesFile_66a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_67a()).runTest("CWE78_OS_Command_Injection__PropertiesFile_67a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_68a()).runTest("CWE78_OS_Command_Injection__PropertiesFile_68a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__PropertiesFile_71a()).runTest("CWE78_OS_Command_Injection__PropertiesFile_71a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_01()).runTest("CWE78_OS_Command_Injection__URLConnection_01");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_02()).runTest("CWE78_OS_Command_Injection__URLConnection_02");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_03()).runTest("CWE78_OS_Command_Injection__URLConnection_03");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_04()).runTest("CWE78_OS_Command_Injection__URLConnection_04");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_05()).runTest("CWE78_OS_Command_Injection__URLConnection_05");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_06()).runTest("CWE78_OS_Command_Injection__URLConnection_06");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_07()).runTest("CWE78_OS_Command_Injection__URLConnection_07");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_08()).runTest("CWE78_OS_Command_Injection__URLConnection_08");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_09()).runTest("CWE78_OS_Command_Injection__URLConnection_09");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_10()).runTest("CWE78_OS_Command_Injection__URLConnection_10");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_11()).runTest("CWE78_OS_Command_Injection__URLConnection_11");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_12()).runTest("CWE78_OS_Command_Injection__URLConnection_12");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_13()).runTest("CWE78_OS_Command_Injection__URLConnection_13");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_14()).runTest("CWE78_OS_Command_Injection__URLConnection_14");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_15()).runTest("CWE78_OS_Command_Injection__URLConnection_15");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_16()).runTest("CWE78_OS_Command_Injection__URLConnection_16");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_17()).runTest("CWE78_OS_Command_Injection__URLConnection_17");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_19()).runTest("CWE78_OS_Command_Injection__URLConnection_19");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_31()).runTest("CWE78_OS_Command_Injection__URLConnection_31");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_41()).runTest("CWE78_OS_Command_Injection__URLConnection_41");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_42()).runTest("CWE78_OS_Command_Injection__URLConnection_42");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_45()).runTest("CWE78_OS_Command_Injection__URLConnection_45");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_51a()).runTest("CWE78_OS_Command_Injection__URLConnection_51a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_52a()).runTest("CWE78_OS_Command_Injection__URLConnection_52a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_53a()).runTest("CWE78_OS_Command_Injection__URLConnection_53a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_54a()).runTest("CWE78_OS_Command_Injection__URLConnection_54a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_61a()).runTest("CWE78_OS_Command_Injection__URLConnection_61a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_66a()).runTest("CWE78_OS_Command_Injection__URLConnection_66a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_67a()).runTest("CWE78_OS_Command_Injection__URLConnection_67a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_68a()).runTest("CWE78_OS_Command_Injection__URLConnection_68a");
			(new testcases.CWE78_OS_Command_Injection.CWE78_OS_Command_Injection__URLConnection_71a()).runTest("CWE78_OS_Command_Injection__URLConnection_71a");
			/* END-AUTOGENERATED-JAVA-TESTS-7 */
}

private static void runTestCWE8() {
/* BEGIN-AUTOGENERATED-JAVA-TESTS-8 */
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_01()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_02()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_03()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_04()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_05()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_06()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_07()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_08()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_09()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_10()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_11()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_12()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_13()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_14()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_15()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_16()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_17()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_19()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_31()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_41()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_42()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_45()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_51a()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_52a()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_53a()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_54a()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_61a()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_66a()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_67a()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_68a()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeBatch_71a()).runTest("CWE89_SQL_Injection__connect_tcp_executeBatch_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_01()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_02()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_03()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_04()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_05()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_06()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_07()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_08()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_09()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_10()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_11()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_12()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_13()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_14()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_15()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_16()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_17()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_19()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_31()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_41()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_42()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_45()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_51a()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_52a()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_53a()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_54a()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_61a()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_66a()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_67a()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_68a()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeQuery_71a()).runTest("CWE89_SQL_Injection__connect_tcp_executeQuery_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_01()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_02()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_03()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_04()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_05()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_06()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_07()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_08()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_09()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_10()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_11()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_12()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_13()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_14()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_15()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_16()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_17()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_19()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_31()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_41()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_42()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_45()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_51a()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_52a()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_53a()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_54a()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_61a()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_66a()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_67a()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_68a()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_executeUpdate_71a()).runTest("CWE89_SQL_Injection__connect_tcp_executeUpdate_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_01()).runTest("CWE89_SQL_Injection__connect_tcp_execute_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_02()).runTest("CWE89_SQL_Injection__connect_tcp_execute_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_03()).runTest("CWE89_SQL_Injection__connect_tcp_execute_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_04()).runTest("CWE89_SQL_Injection__connect_tcp_execute_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_05()).runTest("CWE89_SQL_Injection__connect_tcp_execute_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_06()).runTest("CWE89_SQL_Injection__connect_tcp_execute_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_07()).runTest("CWE89_SQL_Injection__connect_tcp_execute_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_08()).runTest("CWE89_SQL_Injection__connect_tcp_execute_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_09()).runTest("CWE89_SQL_Injection__connect_tcp_execute_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_10()).runTest("CWE89_SQL_Injection__connect_tcp_execute_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_11()).runTest("CWE89_SQL_Injection__connect_tcp_execute_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_12()).runTest("CWE89_SQL_Injection__connect_tcp_execute_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_13()).runTest("CWE89_SQL_Injection__connect_tcp_execute_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_14()).runTest("CWE89_SQL_Injection__connect_tcp_execute_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_15()).runTest("CWE89_SQL_Injection__connect_tcp_execute_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_16()).runTest("CWE89_SQL_Injection__connect_tcp_execute_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_17()).runTest("CWE89_SQL_Injection__connect_tcp_execute_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_19()).runTest("CWE89_SQL_Injection__connect_tcp_execute_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_31()).runTest("CWE89_SQL_Injection__connect_tcp_execute_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_41()).runTest("CWE89_SQL_Injection__connect_tcp_execute_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_42()).runTest("CWE89_SQL_Injection__connect_tcp_execute_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_45()).runTest("CWE89_SQL_Injection__connect_tcp_execute_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_51a()).runTest("CWE89_SQL_Injection__connect_tcp_execute_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_52a()).runTest("CWE89_SQL_Injection__connect_tcp_execute_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_53a()).runTest("CWE89_SQL_Injection__connect_tcp_execute_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_54a()).runTest("CWE89_SQL_Injection__connect_tcp_execute_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_61a()).runTest("CWE89_SQL_Injection__connect_tcp_execute_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_66a()).runTest("CWE89_SQL_Injection__connect_tcp_execute_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_67a()).runTest("CWE89_SQL_Injection__connect_tcp_execute_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_68a()).runTest("CWE89_SQL_Injection__connect_tcp_execute_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__connect_tcp_execute_71a()).runTest("CWE89_SQL_Injection__connect_tcp_execute_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_01()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_02()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_03()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_04()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_05()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_06()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_07()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_08()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_09()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_10()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_11()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_12()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_13()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_14()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_15()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_16()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_17()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_19()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_31()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_41()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_42()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_45()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_51a()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_52a()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_53a()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_54a()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_61a()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_66a()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_67a()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_68a()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeBatch_71a()).runTest("CWE89_SQL_Injection__console_readLine_executeBatch_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_01()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_02()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_03()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_04()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_05()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_06()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_07()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_08()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_09()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_10()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_11()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_12()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_13()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_14()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_15()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_16()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_17()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_19()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_31()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_41()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_42()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_45()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_51a()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_52a()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_53a()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_54a()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_61a()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_66a()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_67a()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_68a()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeQuery_71a()).runTest("CWE89_SQL_Injection__console_readLine_executeQuery_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_01()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_02()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_03()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_04()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_05()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_06()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_07()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_08()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_09()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_10()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_11()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_12()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_13()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_14()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_15()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_16()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_17()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_19()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_31()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_41()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_42()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_45()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_51a()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_52a()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_53a()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_54a()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_61a()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_66a()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_67a()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_68a()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_executeUpdate_71a()).runTest("CWE89_SQL_Injection__console_readLine_executeUpdate_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_01()).runTest("CWE89_SQL_Injection__console_readLine_execute_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_02()).runTest("CWE89_SQL_Injection__console_readLine_execute_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_03()).runTest("CWE89_SQL_Injection__console_readLine_execute_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_04()).runTest("CWE89_SQL_Injection__console_readLine_execute_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_05()).runTest("CWE89_SQL_Injection__console_readLine_execute_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_06()).runTest("CWE89_SQL_Injection__console_readLine_execute_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_07()).runTest("CWE89_SQL_Injection__console_readLine_execute_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_08()).runTest("CWE89_SQL_Injection__console_readLine_execute_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_09()).runTest("CWE89_SQL_Injection__console_readLine_execute_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_10()).runTest("CWE89_SQL_Injection__console_readLine_execute_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_11()).runTest("CWE89_SQL_Injection__console_readLine_execute_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_12()).runTest("CWE89_SQL_Injection__console_readLine_execute_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_13()).runTest("CWE89_SQL_Injection__console_readLine_execute_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_14()).runTest("CWE89_SQL_Injection__console_readLine_execute_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_15()).runTest("CWE89_SQL_Injection__console_readLine_execute_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_16()).runTest("CWE89_SQL_Injection__console_readLine_execute_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_17()).runTest("CWE89_SQL_Injection__console_readLine_execute_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_19()).runTest("CWE89_SQL_Injection__console_readLine_execute_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_31()).runTest("CWE89_SQL_Injection__console_readLine_execute_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_41()).runTest("CWE89_SQL_Injection__console_readLine_execute_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_42()).runTest("CWE89_SQL_Injection__console_readLine_execute_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_45()).runTest("CWE89_SQL_Injection__console_readLine_execute_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_51a()).runTest("CWE89_SQL_Injection__console_readLine_execute_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_52a()).runTest("CWE89_SQL_Injection__console_readLine_execute_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_53a()).runTest("CWE89_SQL_Injection__console_readLine_execute_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_54a()).runTest("CWE89_SQL_Injection__console_readLine_execute_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_61a()).runTest("CWE89_SQL_Injection__console_readLine_execute_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_66a()).runTest("CWE89_SQL_Injection__console_readLine_execute_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_67a()).runTest("CWE89_SQL_Injection__console_readLine_execute_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_68a()).runTest("CWE89_SQL_Injection__console_readLine_execute_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__console_readLine_execute_71a()).runTest("CWE89_SQL_Injection__console_readLine_execute_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_01()).runTest("CWE89_SQL_Injection__Environment_executeBatch_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_02()).runTest("CWE89_SQL_Injection__Environment_executeBatch_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_03()).runTest("CWE89_SQL_Injection__Environment_executeBatch_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_04()).runTest("CWE89_SQL_Injection__Environment_executeBatch_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_05()).runTest("CWE89_SQL_Injection__Environment_executeBatch_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_06()).runTest("CWE89_SQL_Injection__Environment_executeBatch_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_07()).runTest("CWE89_SQL_Injection__Environment_executeBatch_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_08()).runTest("CWE89_SQL_Injection__Environment_executeBatch_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_09()).runTest("CWE89_SQL_Injection__Environment_executeBatch_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_10()).runTest("CWE89_SQL_Injection__Environment_executeBatch_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_11()).runTest("CWE89_SQL_Injection__Environment_executeBatch_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_12()).runTest("CWE89_SQL_Injection__Environment_executeBatch_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_13()).runTest("CWE89_SQL_Injection__Environment_executeBatch_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_14()).runTest("CWE89_SQL_Injection__Environment_executeBatch_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_15()).runTest("CWE89_SQL_Injection__Environment_executeBatch_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_16()).runTest("CWE89_SQL_Injection__Environment_executeBatch_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_17()).runTest("CWE89_SQL_Injection__Environment_executeBatch_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_19()).runTest("CWE89_SQL_Injection__Environment_executeBatch_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_31()).runTest("CWE89_SQL_Injection__Environment_executeBatch_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_41()).runTest("CWE89_SQL_Injection__Environment_executeBatch_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_42()).runTest("CWE89_SQL_Injection__Environment_executeBatch_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_45()).runTest("CWE89_SQL_Injection__Environment_executeBatch_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_51a()).runTest("CWE89_SQL_Injection__Environment_executeBatch_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_52a()).runTest("CWE89_SQL_Injection__Environment_executeBatch_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_53a()).runTest("CWE89_SQL_Injection__Environment_executeBatch_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_54a()).runTest("CWE89_SQL_Injection__Environment_executeBatch_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_61a()).runTest("CWE89_SQL_Injection__Environment_executeBatch_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_66a()).runTest("CWE89_SQL_Injection__Environment_executeBatch_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_67a()).runTest("CWE89_SQL_Injection__Environment_executeBatch_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_68a()).runTest("CWE89_SQL_Injection__Environment_executeBatch_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeBatch_71a()).runTest("CWE89_SQL_Injection__Environment_executeBatch_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_01()).runTest("CWE89_SQL_Injection__Environment_executeQuery_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_02()).runTest("CWE89_SQL_Injection__Environment_executeQuery_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_03()).runTest("CWE89_SQL_Injection__Environment_executeQuery_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_04()).runTest("CWE89_SQL_Injection__Environment_executeQuery_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_05()).runTest("CWE89_SQL_Injection__Environment_executeQuery_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_06()).runTest("CWE89_SQL_Injection__Environment_executeQuery_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_07()).runTest("CWE89_SQL_Injection__Environment_executeQuery_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_08()).runTest("CWE89_SQL_Injection__Environment_executeQuery_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_09()).runTest("CWE89_SQL_Injection__Environment_executeQuery_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_10()).runTest("CWE89_SQL_Injection__Environment_executeQuery_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_11()).runTest("CWE89_SQL_Injection__Environment_executeQuery_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_12()).runTest("CWE89_SQL_Injection__Environment_executeQuery_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_13()).runTest("CWE89_SQL_Injection__Environment_executeQuery_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_14()).runTest("CWE89_SQL_Injection__Environment_executeQuery_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_15()).runTest("CWE89_SQL_Injection__Environment_executeQuery_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_16()).runTest("CWE89_SQL_Injection__Environment_executeQuery_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_17()).runTest("CWE89_SQL_Injection__Environment_executeQuery_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_19()).runTest("CWE89_SQL_Injection__Environment_executeQuery_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_31()).runTest("CWE89_SQL_Injection__Environment_executeQuery_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_41()).runTest("CWE89_SQL_Injection__Environment_executeQuery_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_42()).runTest("CWE89_SQL_Injection__Environment_executeQuery_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_45()).runTest("CWE89_SQL_Injection__Environment_executeQuery_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_51a()).runTest("CWE89_SQL_Injection__Environment_executeQuery_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_52a()).runTest("CWE89_SQL_Injection__Environment_executeQuery_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_53a()).runTest("CWE89_SQL_Injection__Environment_executeQuery_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_54a()).runTest("CWE89_SQL_Injection__Environment_executeQuery_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_61a()).runTest("CWE89_SQL_Injection__Environment_executeQuery_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_66a()).runTest("CWE89_SQL_Injection__Environment_executeQuery_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_67a()).runTest("CWE89_SQL_Injection__Environment_executeQuery_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_68a()).runTest("CWE89_SQL_Injection__Environment_executeQuery_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeQuery_71a()).runTest("CWE89_SQL_Injection__Environment_executeQuery_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_01()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_02()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_03()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_04()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_05()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_06()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_07()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_08()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_09()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_10()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_11()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_12()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_13()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_14()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_15()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_16()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_17()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_19()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_31()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_41()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_42()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_45()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_51a()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_52a()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_53a()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_54a()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_61a()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_66a()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_67a()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_68a()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_executeUpdate_71a()).runTest("CWE89_SQL_Injection__Environment_executeUpdate_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_01()).runTest("CWE89_SQL_Injection__Environment_execute_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_02()).runTest("CWE89_SQL_Injection__Environment_execute_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_03()).runTest("CWE89_SQL_Injection__Environment_execute_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_04()).runTest("CWE89_SQL_Injection__Environment_execute_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_05()).runTest("CWE89_SQL_Injection__Environment_execute_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_06()).runTest("CWE89_SQL_Injection__Environment_execute_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_07()).runTest("CWE89_SQL_Injection__Environment_execute_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_08()).runTest("CWE89_SQL_Injection__Environment_execute_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_09()).runTest("CWE89_SQL_Injection__Environment_execute_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_10()).runTest("CWE89_SQL_Injection__Environment_execute_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_11()).runTest("CWE89_SQL_Injection__Environment_execute_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_12()).runTest("CWE89_SQL_Injection__Environment_execute_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_13()).runTest("CWE89_SQL_Injection__Environment_execute_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_14()).runTest("CWE89_SQL_Injection__Environment_execute_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_15()).runTest("CWE89_SQL_Injection__Environment_execute_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_16()).runTest("CWE89_SQL_Injection__Environment_execute_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_17()).runTest("CWE89_SQL_Injection__Environment_execute_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_19()).runTest("CWE89_SQL_Injection__Environment_execute_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_31()).runTest("CWE89_SQL_Injection__Environment_execute_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_41()).runTest("CWE89_SQL_Injection__Environment_execute_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_42()).runTest("CWE89_SQL_Injection__Environment_execute_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_45()).runTest("CWE89_SQL_Injection__Environment_execute_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_51a()).runTest("CWE89_SQL_Injection__Environment_execute_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_52a()).runTest("CWE89_SQL_Injection__Environment_execute_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_53a()).runTest("CWE89_SQL_Injection__Environment_execute_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_54a()).runTest("CWE89_SQL_Injection__Environment_execute_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_61a()).runTest("CWE89_SQL_Injection__Environment_execute_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_66a()).runTest("CWE89_SQL_Injection__Environment_execute_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_67a()).runTest("CWE89_SQL_Injection__Environment_execute_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_68a()).runTest("CWE89_SQL_Injection__Environment_execute_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__Environment_execute_71a()).runTest("CWE89_SQL_Injection__Environment_execute_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_01()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_02()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_03()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_04()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_05()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_06()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_07()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_08()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_09()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_10()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_11()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_12()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_13()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_14()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_15()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_16()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_17()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_19()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_31()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_41()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_42()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_45()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_51a()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_52a()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_53a()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_54a()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_61a()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_66a()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_67a()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_68a()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeBatch_71a()).runTest("CWE89_SQL_Injection__fromDB_executeBatch_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_01()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_02()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_03()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_04()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_05()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_06()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_07()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_08()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_09()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_10()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_11()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_12()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_13()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_14()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_15()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_16()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_17()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_19()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_31()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_41()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_42()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_45()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_51a()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_52a()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_53a()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_54a()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_61a()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_66a()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_67a()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_68a()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeQuery_71a()).runTest("CWE89_SQL_Injection__fromDB_executeQuery_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_01()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_02()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_03()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_04()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_05()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_06()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_07()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_08()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_09()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_10()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_11()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_12()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_13()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_14()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_15()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_16()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_17()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_19()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_31()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_41()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_42()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_45()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_51a()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_52a()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_53a()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_54a()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_61a()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_66a()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_67a()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_68a()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_executeUpdate_71a()).runTest("CWE89_SQL_Injection__fromDB_executeUpdate_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_01()).runTest("CWE89_SQL_Injection__fromDB_execute_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_02()).runTest("CWE89_SQL_Injection__fromDB_execute_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_03()).runTest("CWE89_SQL_Injection__fromDB_execute_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_04()).runTest("CWE89_SQL_Injection__fromDB_execute_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_05()).runTest("CWE89_SQL_Injection__fromDB_execute_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_06()).runTest("CWE89_SQL_Injection__fromDB_execute_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_07()).runTest("CWE89_SQL_Injection__fromDB_execute_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_08()).runTest("CWE89_SQL_Injection__fromDB_execute_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_09()).runTest("CWE89_SQL_Injection__fromDB_execute_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_10()).runTest("CWE89_SQL_Injection__fromDB_execute_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_11()).runTest("CWE89_SQL_Injection__fromDB_execute_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_12()).runTest("CWE89_SQL_Injection__fromDB_execute_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_13()).runTest("CWE89_SQL_Injection__fromDB_execute_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_14()).runTest("CWE89_SQL_Injection__fromDB_execute_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_15()).runTest("CWE89_SQL_Injection__fromDB_execute_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_16()).runTest("CWE89_SQL_Injection__fromDB_execute_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_17()).runTest("CWE89_SQL_Injection__fromDB_execute_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_19()).runTest("CWE89_SQL_Injection__fromDB_execute_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_31()).runTest("CWE89_SQL_Injection__fromDB_execute_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_41()).runTest("CWE89_SQL_Injection__fromDB_execute_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_42()).runTest("CWE89_SQL_Injection__fromDB_execute_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_45()).runTest("CWE89_SQL_Injection__fromDB_execute_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_51a()).runTest("CWE89_SQL_Injection__fromDB_execute_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_52a()).runTest("CWE89_SQL_Injection__fromDB_execute_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_53a()).runTest("CWE89_SQL_Injection__fromDB_execute_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_54a()).runTest("CWE89_SQL_Injection__fromDB_execute_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_61a()).runTest("CWE89_SQL_Injection__fromDB_execute_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_66a()).runTest("CWE89_SQL_Injection__fromDB_execute_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_67a()).runTest("CWE89_SQL_Injection__fromDB_execute_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_68a()).runTest("CWE89_SQL_Injection__fromDB_execute_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromDB_execute_71a()).runTest("CWE89_SQL_Injection__fromDB_execute_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_01()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_02()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_03()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_04()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_05()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_06()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_07()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_08()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_09()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_10()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_11()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_12()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_13()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_14()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_15()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_16()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_17()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_19()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_31()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_41()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_42()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_45()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_51a()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_52a()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_53a()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_54a()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_61a()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_66a()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_67a()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_68a()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeBatch_71a()).runTest("CWE89_SQL_Injection__fromFile_executeBatch_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_01()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_02()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_03()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_04()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_05()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_06()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_07()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_08()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_09()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_10()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_11()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_12()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_13()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_14()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_15()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_16()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_17()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_19()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_31()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_41()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_42()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_45()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_51a()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_52a()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_53a()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_54a()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_61a()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_66a()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_67a()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_68a()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeQuery_71a()).runTest("CWE89_SQL_Injection__fromFile_executeQuery_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_01()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_02()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_03()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_04()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_05()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_06()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_07()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_08()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_09()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_10()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_11()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_12()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_13()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_14()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_15()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_16()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_17()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_19()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_31()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_41()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_42()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_45()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_51a()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_52a()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_53a()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_54a()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_61a()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_66a()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_67a()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_68a()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_executeUpdate_71a()).runTest("CWE89_SQL_Injection__fromFile_executeUpdate_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_01()).runTest("CWE89_SQL_Injection__fromFile_execute_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_02()).runTest("CWE89_SQL_Injection__fromFile_execute_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_03()).runTest("CWE89_SQL_Injection__fromFile_execute_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_04()).runTest("CWE89_SQL_Injection__fromFile_execute_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_05()).runTest("CWE89_SQL_Injection__fromFile_execute_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_06()).runTest("CWE89_SQL_Injection__fromFile_execute_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_07()).runTest("CWE89_SQL_Injection__fromFile_execute_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_08()).runTest("CWE89_SQL_Injection__fromFile_execute_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_09()).runTest("CWE89_SQL_Injection__fromFile_execute_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_10()).runTest("CWE89_SQL_Injection__fromFile_execute_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_11()).runTest("CWE89_SQL_Injection__fromFile_execute_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_12()).runTest("CWE89_SQL_Injection__fromFile_execute_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_13()).runTest("CWE89_SQL_Injection__fromFile_execute_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_14()).runTest("CWE89_SQL_Injection__fromFile_execute_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_15()).runTest("CWE89_SQL_Injection__fromFile_execute_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_16()).runTest("CWE89_SQL_Injection__fromFile_execute_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_17()).runTest("CWE89_SQL_Injection__fromFile_execute_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_19()).runTest("CWE89_SQL_Injection__fromFile_execute_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_31()).runTest("CWE89_SQL_Injection__fromFile_execute_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_41()).runTest("CWE89_SQL_Injection__fromFile_execute_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_42()).runTest("CWE89_SQL_Injection__fromFile_execute_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_45()).runTest("CWE89_SQL_Injection__fromFile_execute_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_51a()).runTest("CWE89_SQL_Injection__fromFile_execute_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_52a()).runTest("CWE89_SQL_Injection__fromFile_execute_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_53a()).runTest("CWE89_SQL_Injection__fromFile_execute_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_54a()).runTest("CWE89_SQL_Injection__fromFile_execute_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_61a()).runTest("CWE89_SQL_Injection__fromFile_execute_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_66a()).runTest("CWE89_SQL_Injection__fromFile_execute_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_67a()).runTest("CWE89_SQL_Injection__fromFile_execute_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_68a()).runTest("CWE89_SQL_Injection__fromFile_execute_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__fromFile_execute_71a()).runTest("CWE89_SQL_Injection__fromFile_execute_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_01()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_02()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_03()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_04()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_05()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_06()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_07()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_08()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_09()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_10()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_11()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_12()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_13()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_14()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_15()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_16()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_17()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_19()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_31()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_41()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_42()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_45()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_51a()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_52a()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_53a()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_54a()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_61a()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_66a()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_67a()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_68a()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeBatch_71a()).runTest("CWE89_SQL_Injection__listen_tcp_executeBatch_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_01()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_02()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_03()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_04()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_05()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_06()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_07()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_08()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_09()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_10()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_11()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_12()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_13()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_14()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_15()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_16()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_17()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_19()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_31()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_41()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_42()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_45()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_51a()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_52a()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_53a()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_54a()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_61a()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_66a()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_67a()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_68a()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeQuery_71a()).runTest("CWE89_SQL_Injection__listen_tcp_executeQuery_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_01()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_02()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_03()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_04()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_05()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_06()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_07()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_08()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_09()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_10()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_11()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_12()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_13()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_14()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_15()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_16()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_17()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_19()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_31()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_41()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_42()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_45()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_51a()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_52a()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_53a()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_54a()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_61a()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_66a()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_67a()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_68a()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_executeUpdate_71a()).runTest("CWE89_SQL_Injection__listen_tcp_executeUpdate_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_01()).runTest("CWE89_SQL_Injection__listen_tcp_execute_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_02()).runTest("CWE89_SQL_Injection__listen_tcp_execute_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_03()).runTest("CWE89_SQL_Injection__listen_tcp_execute_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_04()).runTest("CWE89_SQL_Injection__listen_tcp_execute_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_05()).runTest("CWE89_SQL_Injection__listen_tcp_execute_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_06()).runTest("CWE89_SQL_Injection__listen_tcp_execute_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_07()).runTest("CWE89_SQL_Injection__listen_tcp_execute_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_08()).runTest("CWE89_SQL_Injection__listen_tcp_execute_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_09()).runTest("CWE89_SQL_Injection__listen_tcp_execute_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_10()).runTest("CWE89_SQL_Injection__listen_tcp_execute_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_11()).runTest("CWE89_SQL_Injection__listen_tcp_execute_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_12()).runTest("CWE89_SQL_Injection__listen_tcp_execute_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_13()).runTest("CWE89_SQL_Injection__listen_tcp_execute_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_14()).runTest("CWE89_SQL_Injection__listen_tcp_execute_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_15()).runTest("CWE89_SQL_Injection__listen_tcp_execute_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_16()).runTest("CWE89_SQL_Injection__listen_tcp_execute_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_17()).runTest("CWE89_SQL_Injection__listen_tcp_execute_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_19()).runTest("CWE89_SQL_Injection__listen_tcp_execute_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_31()).runTest("CWE89_SQL_Injection__listen_tcp_execute_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_41()).runTest("CWE89_SQL_Injection__listen_tcp_execute_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_42()).runTest("CWE89_SQL_Injection__listen_tcp_execute_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_45()).runTest("CWE89_SQL_Injection__listen_tcp_execute_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_51a()).runTest("CWE89_SQL_Injection__listen_tcp_execute_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_52a()).runTest("CWE89_SQL_Injection__listen_tcp_execute_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_53a()).runTest("CWE89_SQL_Injection__listen_tcp_execute_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_54a()).runTest("CWE89_SQL_Injection__listen_tcp_execute_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_61a()).runTest("CWE89_SQL_Injection__listen_tcp_execute_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_66a()).runTest("CWE89_SQL_Injection__listen_tcp_execute_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_67a()).runTest("CWE89_SQL_Injection__listen_tcp_execute_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_68a()).runTest("CWE89_SQL_Injection__listen_tcp_execute_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__listen_tcp_execute_71a()).runTest("CWE89_SQL_Injection__listen_tcp_execute_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_01()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_02()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_03()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_04()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_05()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_06()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_07()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_08()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_09()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_10()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_11()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_12()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_13()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_14()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_15()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_16()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_17()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_19()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_31()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_41()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_42()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_45()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_51a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_52a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_53a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_54a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_61a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_66a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_67a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_68a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeBatch_71a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeBatch_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_01()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_02()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_03()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_04()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_05()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_06()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_07()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_08()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_09()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_10()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_11()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_12()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_13()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_14()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_15()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_16()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_17()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_19()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_31()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_41()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_42()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_45()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_51a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_52a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_53a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_54a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_61a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_66a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_67a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_68a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeQuery_71a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeQuery_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_01()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_02()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_03()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_04()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_05()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_06()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_07()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_08()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_09()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_10()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_11()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_12()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_13()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_14()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_15()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_16()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_17()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_19()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_31()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_41()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_42()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_45()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_51a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_52a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_53a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_54a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_61a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_66a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_67a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_68a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_executeUpdate_71a()).runTest("CWE89_SQL_Injection__PropertiesFile_executeUpdate_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_01()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_02()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_03()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_04()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_05()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_06()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_07()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_08()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_09()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_10()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_11()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_12()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_13()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_14()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_15()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_16()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_17()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_19()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_31()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_41()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_42()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_45()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_51a()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_52a()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_53a()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_54a()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_61a()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_66a()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_67a()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_68a()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__PropertiesFile_execute_71a()).runTest("CWE89_SQL_Injection__PropertiesFile_execute_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_01()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_02()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_03()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_04()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_05()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_06()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_07()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_08()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_09()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_10()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_11()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_12()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_13()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_14()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_15()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_16()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_17()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_19()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_31()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_41()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_42()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_45()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_51a()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_52a()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_53a()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_54a()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_61a()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_66a()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_67a()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_68a()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeBatch_71a()).runTest("CWE89_SQL_Injection__URLConnection_executeBatch_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_01()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_02()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_03()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_04()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_05()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_06()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_07()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_08()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_09()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_10()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_11()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_12()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_13()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_14()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_15()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_16()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_17()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_19()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_31()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_41()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_42()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_45()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_51a()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_52a()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_53a()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_54a()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_61a()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_66a()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_67a()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_68a()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeQuery_71a()).runTest("CWE89_SQL_Injection__URLConnection_executeQuery_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_01()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_02()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_03()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_04()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_05()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_06()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_07()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_08()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_09()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_10()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_11()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_12()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_13()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_14()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_15()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_16()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_17()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_19()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_31()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_41()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_42()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_45()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_51a()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_52a()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_53a()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_54a()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_61a()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_66a()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_67a()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_68a()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_executeUpdate_71a()).runTest("CWE89_SQL_Injection__URLConnection_executeUpdate_71a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_01()).runTest("CWE89_SQL_Injection__URLConnection_execute_01");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_02()).runTest("CWE89_SQL_Injection__URLConnection_execute_02");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_03()).runTest("CWE89_SQL_Injection__URLConnection_execute_03");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_04()).runTest("CWE89_SQL_Injection__URLConnection_execute_04");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_05()).runTest("CWE89_SQL_Injection__URLConnection_execute_05");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_06()).runTest("CWE89_SQL_Injection__URLConnection_execute_06");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_07()).runTest("CWE89_SQL_Injection__URLConnection_execute_07");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_08()).runTest("CWE89_SQL_Injection__URLConnection_execute_08");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_09()).runTest("CWE89_SQL_Injection__URLConnection_execute_09");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_10()).runTest("CWE89_SQL_Injection__URLConnection_execute_10");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_11()).runTest("CWE89_SQL_Injection__URLConnection_execute_11");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_12()).runTest("CWE89_SQL_Injection__URLConnection_execute_12");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_13()).runTest("CWE89_SQL_Injection__URLConnection_execute_13");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_14()).runTest("CWE89_SQL_Injection__URLConnection_execute_14");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_15()).runTest("CWE89_SQL_Injection__URLConnection_execute_15");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_16()).runTest("CWE89_SQL_Injection__URLConnection_execute_16");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_17()).runTest("CWE89_SQL_Injection__URLConnection_execute_17");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_19()).runTest("CWE89_SQL_Injection__URLConnection_execute_19");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_31()).runTest("CWE89_SQL_Injection__URLConnection_execute_31");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_41()).runTest("CWE89_SQL_Injection__URLConnection_execute_41");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_42()).runTest("CWE89_SQL_Injection__URLConnection_execute_42");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_45()).runTest("CWE89_SQL_Injection__URLConnection_execute_45");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_51a()).runTest("CWE89_SQL_Injection__URLConnection_execute_51a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_52a()).runTest("CWE89_SQL_Injection__URLConnection_execute_52a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_53a()).runTest("CWE89_SQL_Injection__URLConnection_execute_53a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_54a()).runTest("CWE89_SQL_Injection__URLConnection_execute_54a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_61a()).runTest("CWE89_SQL_Injection__URLConnection_execute_61a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_66a()).runTest("CWE89_SQL_Injection__URLConnection_execute_66a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_67a()).runTest("CWE89_SQL_Injection__URLConnection_execute_67a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_68a()).runTest("CWE89_SQL_Injection__URLConnection_execute_68a");
			(new testcases.CWE89_SQL_Injection.CWE89_SQL_Injection__URLConnection_execute_71a()).runTest("CWE89_SQL_Injection__URLConnection_execute_71a");
			/* END-AUTOGENERATED-JAVA-TESTS-8 */
}

private static void runTestCWE9() {
/* BEGIN-AUTOGENERATED-JAVA-TESTS-9 */

			/* END-AUTOGENERATED-JAVA-TESTS-9 */
}
}